# -*- coding: utf-8 -*-
from __future__ import division
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt


def input_data_analogos_rellenoh(df_loc, df_pred):
    """Funcion para obtener la informacion necesaria para el metodo de analogos para el relleno de huecos

    Args:
        df_loc: Dataframe de datos locales (1 variable)
        df_pred: Dataframe de datos de los predictores (1 o mas variables).

    Returns:
        df_loc_ttrain: Dataframe de observaciones o datos locales durante el periodo de entrenamiento (1 variable).
        df_pred_ttrain: Dataframe de predictores en el periodo de entrenamiento (1 o mas variables).
        df_pred_trec: Matriz de datos de predictores en el periodo a reconstruir (1 o mas variables).


    """

    df_loc_sn = df_loc.dropna()
    df_pred_sn = df_pred.dropna()

    df_inter = pd.merge(df_loc_sn, df_pred_sn, how='inner', left_index=True, right_index=True)

    df_loc_ttrain = df_loc_sn.reindex(df_inter.index)
    df_pred_ttrain = df_pred_sn.reindex(df_inter.index)

    df_loc_trec = df_loc.drop(pd.Timestamp(i) for i in df_inter.index)
    df_loc_trec = df_loc_trec[df_loc_trec.isnull().any(axis=1)]
    df_pred_trec = df_pred.reindex(df_loc_trec.index)

    df_pred_trec_null = df_pred_trec.isnull().all(axis=1).any()
    if df_pred_trec_null:
        print('El dataframe de datos de los predictores no contiene suficiente informacion. Revisar los resultados de la reconstrucción')

    return df_loc_ttrain, df_pred_ttrain, df_pred_trec


def input_data_analogos_downscaling(df_loc, df_pred):
    """Funcion para obtener la informacion necesaria para el metodo de analogos

    Args:
        df_loc: Dataframe de datos locales (1 variable)
        df_pred: Dataframe de datos de los predictores (1 o mas variables).

    Returns:
        df_loc_ttrain: Dataframe de observaciones o datos locales durante el periodo de entrenamiento (1 variable).
        df_pred_ttrain: Dataframe de predictores en el periodo de entrenamiento (1 o mas variables).
        df_pred_trec: Matriz de datos de predictores en el periodo a reconstruir (1 o mas variables).


    """

    df_loc_sn = df_loc.dropna()
    df_pred_sn = df_pred.dropna()

    df_inter = pd.merge(df_loc_sn, df_pred_sn, how='inner', left_index=True, right_index=True)

    df_loc_ttrain = df_loc_sn.reindex(df_inter.index)
    df_pred_ttrain = df_pred_sn.reindex(df_inter.index)

    df_pred_trec = df_pred_sn.drop(pd.Timestamp(i) for i in df_inter.index)

    return df_loc_ttrain, df_pred_ttrain, df_pred_trec


def reconstruct_analogs(df_loc_ttrain, df_pred_ttrain, df_pred_trec, esc_pred, circ_pred, par):
    """Funcion para reconstruir una serie temporal mediante el metodo de analogos

    Args:
        df_loc_ttrain: Dataframe de observaciones o datos locales durante el periodo de entrenamiento (1 variable).
        df_pred_ttrain: Dataframe de predictores en el periodo de entrenamiento (1 o mas variables).
        df_pred_trec: Dataframe de datos de predictores en el periodo a reconstruir (1 o mas variables).
        esc_pred: Columnas de predictores correspondientes a variables escalares (empieza en 1).
        circ_pred: Columnas de predictores correspondientes a variables circulares (empieza en 1).
        par: Diccionario que contiene los parametros del metodo (`k``, ``a_trend``, ``sw`` y ``wp``). 

                * ``k`` -> Numero de analogos escogido para reconstruir cada instante.
                * ``a_trend`` -> Tendencia temporal de los analogos (intervalo de comparacion en horas).
                * ``sw`` -> Ventana de busqueda para los candidatos a analogos en horas.
                * ``wp`` -> Vector de ponderacion de pesos para la metrica de analogos.

    Returns:
        Dataframe de serie temporal reconstruida (la primera columna es la media ponderada - mean y la
            segunda la desviacion estandar - std).

    """

    # Parametros del metodo
    k = par['k']
    a_trend = par['a_trend']
    sw = par['sw']
    wp = par['wp']

    date_train = df_loc_ttrain.index
    date_pred = df_pred_trec.index
    s_p = df_pred_trec.std()
    columns = ['mean', 'std']
    df_loc_trec = pd.DataFrame(index=df_pred_trec.index, columns=columns)

    # a_n = np.zeros(k) # TODO comprobar Pedro: no tiene sentido inicializar cuando abajo se copia tal cual
    # o_k = np.zeros(k)
    # gamma_k = np.zeros(k)

    for jj in range(df_pred_trec.shape[0]):
        # Candidatos a analogos (misma hora del dia +- sw)
        pos_cand = np.where(np.abs(date_pred[jj].hour - date_train.hour) <= sw)
        mt = np.zeros(len(pos_cand[0]))

        # Tendencia del instante a reconstruir
        mask = np.abs((date_pred - date_pred[jj]) / pd.Timedelta(1.0, unit='D')) <= a_trend / 24.0
        f = df_pred_trec.iloc[mask, :]

        for j in range(len(pos_cand[0])):  # TODO comprobar Pedro
            # Tendencia del candidato a analogo
            mask1 = np.abs((date_train - date_train[pos_cand[0][j]]) / pd.Timedelta(1.0, unit='D')) <= a_trend / 24.0
            # TODO comprobar Pedro: date_train[pos_cand[0][j]]
            a = df_pred_ttrain.iloc[mask1, :]
            if a.shape[0] == f.shape[0]:
                mt_t = 0
                for l in range(len(esc_pred)):  # TODO comprobar Pedro: esc_pred debe ser array para esc_pred[l]. Lo mismo para circ_pred
                    s = (f.iloc[:, esc_pred[l]-1].values - a.iloc[:, esc_pred[l]-1].values) ** 2  # TODO comprobar Pedro. esc_pred[l]-1
                    mt_t = mt_t + wp[esc_pred[l]-1] / s_p[esc_pred[l]-1] * np.sqrt(np.sum(s))
                for l in range(len(circ_pred)):
                    aux = np.abs(f.iloc[:, circ_pred[l]-1].values - a.iloc[:, circ_pred[l]-1].values)
                    d = np.min([aux, 360 - aux], axis=0)  # TODO comprobar Pedro
                    s = d ** 2
                    mt_t = mt_t + wp[circ_pred[l]-1] / s_p[circ_pred[l]-1] * np.sqrt(np.sum(s))
                mt[j] = mt_t
            else:
                mt[j] = 10000.0

        index = np.argsort(mt)  # TODO comprobar Pedro: lo hace distinto a Matlab solo el 1er valor != 10000
        a_n = index[0:k]
        o_k = df_loc_ttrain.iloc[pos_cand[0][a_n]]
        mt_k = mt[a_n]
        mt_k_si = np.sum(mt_k ** (-1))
        gamma_k = mt_k ** (-1) / mt_k_si
        df_loc_trec.iloc[jj, 0] = np.dot(o_k.T, gamma_k)[0]  # TODO comprobar Pedro:multiplicacion matricial y narray[0]
        df_loc_trec.iloc[jj, 1] = o_k.std()[0]

    return df_loc_trec


def output_data_analogos(df_loc, df_loc_trec):
    """Funcion para unir los datos locales con los datos reconstruidos

    Args:
        df_loc: Dataframe de datos locales (1 variable)
        df_loc_trec: Dataframe de serie temporal reconstruida (la primera columna es la media ponderada - mean y la
        segunda la desviacion estandar - std).

    Returns:
        df_loc_reconst: Dataframe de observaciones o datos locales reconstruido (1 variable).

    """
    df_loc_sn = df_loc.dropna()
    df_loc_sn['std'] = 0
    namecol = df_loc_sn.columns.values.tolist()[0]

    df_loc_trec = df_loc_trec.rename(columns={'mean': namecol})

    df_reconst = df_loc_sn.combine_first(df_loc_trec)

    return df_reconst


def plot_reconstruct_analogos(df_loc, df_loc_trec):
    """Funcion para la salida gráfica de la reconstrucción mediante análogos

    Args:
        df_loc: Dataframe de datos locales (1 variable)
        df_loc_trec: Dataframe de serie temporal reconstruida (la primera columna es la media ponderada - mean y la
        segunda la desviacion estandar - std).

    """
    name_df = ['hs', 'tp', 'dh', 'vv', 'dv', 'pa', 'niv', 'mas', 'mme']
    name_ylabel = ['$H_s (m)$', '$T_p (s)$', r'$\theta(^\circ)$', '$U (m/s)$', r'$\theta_U(^\circ)$', '$P_s (Hpa)$',
                   r'$\eta_{NM} (m)$', r'$\eta_{MAs} (m)$', r'$\eta_{MMe} (m)$']

    plt.style.use('ggplot')
    plt.figure()
    plt.plot(df_loc_trec.index, df_loc_trec.iloc[:, 0] + df_loc_trec.iloc[:, 1], '.', color='gray',
             label='Intervalos de confianza')
    plt.plot(df_loc_trec.index, df_loc_trec.iloc[:, 0] - df_loc_trec.iloc[:, 1], '.', color='gray')
    plt.plot(df_loc_trec.index, df_loc_trec.iloc[:, 0], '.', color='orange', label='Serie reconstruida')
    plt.plot(df_loc.index, df_loc.iloc[:, 0], '.', color='b', label='Serie local')
    ind_ylb = name_df.index(df_loc.iloc[:, 0].name)
    plt.ylabel(name_ylabel[ind_ylb])
    plt.xticks(rotation=30)
    if (ind_ylb == 2) | (ind_ylb == 4):
        plt.ylim(ymin=0, ymax=360)
    else:
        plt.ylim(ymin=0)
    plt.legend(numpoints=1, loc='best')
