#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import division
import numpy as np
import pandas as pd
from datetime import datetime, timedelta
import time
import matplotlib.pyplot as plt


def define_ciclos(df_picos_umb, df, v_i, umbral, sep, dur):
    """Funcion para definir los ciclos de solicitacion a sociados a la serie
    temporal de una variable.

    Args:
        df_picos_umb: Dataframe de los picos sobre umbral de la variable principal
        df: Dataframe de los descriptores de estado.
        v_i: Variable de interes.
        umbral: Umbral para la definicion de los ciclos.
        sep: Separacion minima entre temporales. En numero de puntos
          respecto de la serie temporal, es dos veces el espaciado minimo
          entre picos.
        dur: Duracion minima del ciclo de solicitacion en datetime.

    Returns:
        Una tupla que contiene:

        * Dataframe de los descriptores en los instantes de pico.
        * Dataframe de tiempos de inicio y duracion de los ciclos de
            solicitacion.
        * Vector de pasos ascendentes sobre el umbral.
        * Vector de pasos descendentes sobre el umbral.

    """
    v = df[v_i]

    cic, asc, desc = pasos_ciclo(v, df_picos_umb, umbral)
    filtro_cic = filtro_ciclo(cic, df_picos_umb, dur)

    return df.iloc[filtro_cic.index], cic.iloc[filtro_cic], asc.iloc[filtro_cic], desc.iloc[filtro_cic]


def pasos_ciclo(v, picos, umbral):
    """Funcion para obtener los pasos ascendentes y descendentes sobre,
    umbral, la duracion y el tiempo de inicio de cada ciclo.

    Args:
        v: Serie temporal de la variable de interes.
        picos: Dataframe de los instantes de pico.
        umbral: Umbral para la definicion de los ciclos.

    Returns:
        Una tupla que contiene:

       *``cic``: Dataframe de tiempos de inicio y duracion de los ciclos de
           solicitacion.
       *``asc``: Vector de pasos ascendentes sobre el umbral.
       *``desc``: Vector de pasos descendentes sobre el umbral.

    """

    n_picos = picos.shape[0]
    asc = pd.Series(np.zeros(n_picos, dtype=int), index=picos.index)
    desc = pd.Series(np.zeros(n_picos, dtype=int), index=picos.index)
    cic = pd.Series()

    for ind, pico in picos.iteritems():
        i = picos.index.get_loc(ind)

        k = ind
        while v[k] > umbral:
            k -= 1
        asc[ind] = k

        k = ind
        while v[k] > umbral:
            k += 1
        desc[ind] = k

        ciclo_asc = v[asc.iat[i]:asc.iat[i]+2].sort_values()
        tiempo = [time.mktime(t.to_datetime().timetuple()) for t in ciclo_asc.index]
        t0 = datetime.fromtimestamp(np.interp(umbral, ciclo_asc, tiempo))

        ciclo_desc = v[desc.iat[i]-1:desc.iat[i]+1].sort_values()
        tiempo = [time.mktime(t.to_datetime().timetuple()) for t in ciclo_desc.index]
        t1 = datetime.fromtimestamp(np.interp(umbral, ciclo_desc, tiempo))

        cic[t0] = t1 - t0

    return cic, asc, desc


def filtro_ciclo(cic, picos, dur):
    """Funcion para filtrar variables asociadas a ciclos de solicitacion por
    duracion minima.

    Args:
        cic: Dataframe de tiempos de inicio y duracion de los ciclos de solicitacion.
        picos: Dataframe de los instantes de pico.
        dur: Duracion minima del ciclo de solicitacion en datetime.

    Returns:
        Mascara de filtro.

    """

    condicion = np.where(cic >= dur)[0]
    mascara = pd.Series(condicion, index=picos.index[condicion])
    return mascara


def estadisticos_ciclos(df_total, v_ppal, df_condicionado, ciclos_condicionado, p_asc_condicionado,
                        p_desc_condicionado, *name_v_ppal2):
    """Función para obtener los descriptores de estado y parametros adimensionales
    caracteristicos de los ciclos de solicitacion asociados a una serie
    temporal.

    Args:
        df_total: Dataframe de los descriptores de estado por
            columnas. La primera columna es el tiempo, la cuarta el periodo y
            la quinta la velocidad de viento.
        v_ppal: Vector de picos de la variable principal.
        df_condicionado: Dataframe de los descriptores en los instantes de pico.
        ciclos_condicionado: Dataframe de tiempos de inicio y duracion de los ciclos de solicitacion.
        p_asc_condicionado: Vector de pasos ascendentes sobre el umbral.
        p_desc_condicionado: Vector de pasos descendentes sobre el umbral.
        name_v_ppal2 [opc]: Nombre de la segunda variable que condiciona el modo de fallo (si existe).

    Returns:
        Una estructura ``s`` que contiene:

        *``gx_vv2``: Monomio adimensional gx/(U10)^2, donde x es la variable
            principal evaluada en cada ciclo, g es la aceleracion de la
            gravedad y U10 es la velocidad de viento a 10m de altura.
        *``gtp_vv``: Monomio adimensional gTp/U10, donde Tp es el periodo de
            pico en el instante en que ocurre Hp en cada ciclo.
        *``gd_vv``: Monomio adimensional gD/U10, donde D es la duracion de cada
            ciclo.
        *``gdt1_vv``: Monomio adimensional gDt1/U10, donde Dt1 es el intervalo
            de tiempo comprendido entre el inicio del ciclo y la ocurrencia del
            pico correspondiente.
        *``df_ciclo``: Dataframe descriptores condicionado a la ocurrencia de
            los ciclos de solicitacion.
        *``df_pico``: Dataframe de descriptores en los instantes de pico.
        *``ciclos_anuales``: Numero de ciclos por ano.

    """

    s = {}

    n_ciclos = df_condicionado.shape[0]

    gx_vv2 = 9.81 * df_condicionado[v_ppal].values / df_condicionado['vv']**2
    gtp_vv = 9.81 * df_condicionado['tp'].values / df_condicionado['vv']
    gd_vv = 9.81 * ciclos_condicionado.apply(timedelta.total_seconds).values / df_condicionado['vv']
    dt_asc = (df_condicionado.index - ciclos_condicionado.index).total_seconds()
    gdt1_vv = 9.81 * dt_asc / df_condicionado['vv']

    df_ciclo = pd.DataFrame()

    if name_v_ppal2:
        name_v_ppal2 = name_v_ppal2[0]
        df_v2 = pd.DataFrame(data=np.zeros((n_ciclos, 5)), columns=[name_v_ppal2+'_p', name_v_ppal2+'_i',
                             name_v_ppal2+'_f', name_v_ppal2+'_max', name_v_ppal2+'_delta'])
        for i in range(n_ciclos):
            df_v2[name_v_ppal2+'_p'].iloc[i] = df_condicionado[name_v_ppal2].iloc[i]
            df_v2[name_v_ppal2+'_i'].iloc[i] = df_total[name_v_ppal2][p_asc_condicionado.iloc[i]+1]
            df_v2[name_v_ppal2+'_f'].iloc[i] = df_total[name_v_ppal2][p_desc_condicionado.iloc[i]-1]
            df_v2[name_v_ppal2+'_max'].iloc[i] = df_total[name_v_ppal2][
                p_asc_condicionado.iloc[i]+1:p_desc_condicionado.iloc[i]].max()
            desfase = df_total[name_v_ppal2][p_asc_condicionado.iloc[i]+1:p_desc_condicionado.iloc[i]].idxmax(
                ) - df_condicionado.index[i]
            df_v2[name_v_ppal2+'_delta'].iloc[i] = desfase.total_seconds()

        s['df_v2'] = df_v2

    for i in range(n_ciclos):
        df_ciclo = df_ciclo.append(df_total[p_asc_condicionado.iloc[i]+1:p_desc_condicionado.iloc[i]])

    ciclos_anuales = df_condicionado.groupby(df_condicionado.index.year).count()['hs'].rename()

    s['gx_vv2'] = gx_vv2
    s['gtp_vv'] = gtp_vv
    s['gd_vv'] = gd_vv
    s['gdt1_vv'] = gdt1_vv
    s['df_ciclo'] = df_ciclo
    s['df_pico'] = df_condicionado
    s['ciclos_anuales'] = ciclos_anuales
    s['duracion'] = ciclos_condicionado.apply(timedelta.total_seconds).values
    s['duracion_asc'] = dt_asc

    return s


def ciclo_sintetico(sector, reg, dt_simar, umbral, v_max, v_pri_sec, *v_ppal2):
    """Funcion para obtener un ciclo sintetico de solicitacion de estados meteorologicos asociados a un valor de la
    variable principal 1

    Args:
        sector: Estructura que contiene los estadidisticos de los ciclos.
        reg: Matriz de regresiones de cada variable con la principal 1.
        dt_simar: Resolucion temporal de la serie de datos.
        umbral: Valor del umbral del ciclo de solicitacion.
        v_max: Valor de pico de la variable principal 1.
        v_pri_sec: Variables secundarias.
        v_ppal2[opc]: Lista con el nombre y el valor de la variable principal 2 concomitante con el pico de la variable
            principal 1 (si existe).

    Returns:
        Dataframe con las series temporales de todos los descriptores de estado.

    """
    g = 9.81
    u10 = np.sqrt(g*v_max/np.mean(sector['gx_vv2']))
    d_t = np.mean(sector['gd_vv'])*u10/g/3600
    d_asc = np.mean(sector['gdt1_vv'])*u10/g/3600

    d_t, d_asc = np.round(d_t/dt_simar)*dt_simar, np.round(d_asc/dt_simar)*dt_simar
    t = np.arange(0, d_t+dt_simar, dt_simar)
    s_asc = (v_max-umbral)/d_asc*t[t <= d_asc] + umbral
    s_desc = (umbral - v_max)/(d_t - d_asc)*(t[t > d_asc] - d_asc) + v_max

    serie = np.append(s_asc, s_desc)
    plt.figure()
    plt.plot(t, serie)
    df_ciclo_s = pd.DataFrame(0, index=t, columns=v_pri_sec)
    df_ciclo_s[v_pri_sec[0]] = serie

    for i in v_pri_sec[1:]:
        df_ciclo_s[i] = np.interp(serie, reg[i].iloc[:, 0], reg[i].iloc[:, 2])

    if any(v_ppal2):
        n_vppal2, val_p, reg2, delta_d = v_ppal2[0], v_ppal2[1], v_ppal2[2], v_ppal2[3]
        val_ppal2 = np.array([])
        delta = delta_d*d_t
        for i in reg2.keys():
            val_ppal2 = np.append(val_ppal2, np.interp(val_p, reg2[i].iloc[:, 0], reg2[i].iloc[:, 2]))

        val_ppal2 = np.append(val_ppal2, val_p)
        t_ord = np.array([0, d_asc, d_t, d_asc+delta])
        ind_ = np.argsort(t_ord)
        val_ppal2, t_ord = val_ppal2[ind_], t_ord[ind_]

        s_asc_21 = (val_ppal2[1]-val_ppal2[0])/t_ord[1]*t[t <= t_ord[1]] + val_ppal2[0]  # tramo 1
        s_asc_22 = (val_ppal2[2]-val_ppal2[1])/(t_ord[2] - t_ord[1])*(t[((t > t_ord[1]) & (
                t <= t_ord[2]))] - t_ord[1]) + val_ppal2[1]  # tramo 2
        s_desc = (val_ppal2[3] - val_ppal2[2])/(t_ord[3] - t_ord[2])*(t[t > t_ord[2]] - t_ord[2]) + val_ppal2[2]
        df_ciclo_s[n_vppal2] = np.concatenate((s_asc_21, s_asc_22, s_desc))
        plt.figure()
        plt.plot(df_ciclo_s[n_vppal2])

    return df_ciclo_s

