# -*- coding: utf-8 -*-
import matplotlib.pyplot as plt
import numpy as np
from scipy.stats import expon, poisson, norm


def plot_d_n_delta(dur, n, delta_d):
    """ Funcion que dibuja los ajustes probabilisticos de la duracion de los ciclos, el numero de ciclos por año y
    desfase entre picos de las variables principales.

    Args:
        dur: duracion de cada ciclo
        n: numero de ciclos
        delta_d: desfase adimensional entre los picos de las variables principales

    Returns:

    """

    plt.style.use('ggplot')
    plt.figure(figsize=(9, 3))
    plt.subplot(1, 3, 1)
    hist, edg = np.histogram(dur, normed=True)
    plt.plot((edg[:-1] + np.diff(edg))/1e5, hist, '.', color='gray', label='Medido')
    par = expon.fit(dur)
    x = np.linspace(edg[0], edg[-1], 100)
    plt.plot(x/1e5, expon.pdf(x, *par)*1e6, 'k', lw=2, label='Teorico')
    plt.xlabel('Duracion (s) [/1e5]', fontweight='bold')
    plt.ylabel('Pdf [x1e6]', fontweight='bold')

    plt.subplot(1, 3, 2)
    hist, edg = np.histogram(n, np.arange(0, 11), normed=True)
    plt.plot(np.arange(0, 10), hist, '.', color='gray', label='Medido')
    x = np.arange(0, 10)
    pdf = np.zeros(len(x))
    for i in range(len(x)):
        pdf[i] = np.exp(-np.mean(n))*np.mean(n)**x[i]/np.math.factorial(x[i])
    plt.plot(x, pdf, 'k', lw=2, label='Teorico')
    plt.xlabel('No. ciclos anual', fontweight='bold')
    plt.ylabel('Pdf', fontweight='bold')

    plt.subplot(1, 3, 3)
    hist, edg = np.histogram(delta_d, normed=True)
    plt.plot(edg[:-1] + np.diff(edg), hist, '.', color='gray', label='Medido')
    par = norm.fit(delta_d)
    x = np.linspace(edg[0], edg[-1], 100)
    plt.plot(x, norm.pdf(x, *par), 'k', lw=2, label='Teorico')
    plt.xlabel('Desfase', fontweight='bold')
    plt.ylabel('Pdf', fontweight='bold')
    plt.tight_layout(pad=0.5)
    plt.legend(loc='best', numpoints=1)
    plt.gcf().subplots_adjust(bottom=0.2, left=0.2)


def plot_monomios_reg(df_v2, reg, name_v_ppal2, tipo_reg, umb):
    """ Funcion que dibuja las relaciones entre el valor inicial, el final y el maximo con el valor de pico junto con
    las funciones de regresion asociadas a cada una de ellas.

    Args:
        df_v2: Dataframe con toda la informacion.
        name_v_ppal2: nombre de la segunda variable principal.
        tipo_reg: Tipo de regresion seleccionada.
        umb: umbral

    Returns:

    """

    plt.style.use('ggplot')
    plt.figure(figsize=(9, 3))
    term = ['_i', '_f', '_max']
    for i in range(0, 3):
        plt.subplot(1, 3, i+1)
        plt.plot(df_v2[name_v_ppal2+'_p'], df_v2[name_v_ppal2+term[i]], '.', color='gray', label='Medido')
        plt.plot(reg[name_v_ppal2+term[i]].iloc[:, 0], reg[name_v_ppal2+term[i]].iloc[:, 2], 'k', lw=2, label='Ajuste')
        plt.plot(reg[name_v_ppal2+term[i]].iloc[:, 0], reg[name_v_ppal2+term[i]].iloc[:, 1], '--k', label=r'$I.C._{sup}$')
        plt.plot(reg[name_v_ppal2+term[i]].iloc[:, 0], reg[name_v_ppal2+term[i]].iloc[:, 3], ':k', label=r'$I.C._{inf}$')
        plt.xlabel(name_v_ppal2 + '_p', fontweight='bold')
        plt.ylabel(name_v_ppal2 + term[i], fontweight='bold')
        plt.xlim([-0.2, 1.6])

    plt.legend(loc='best', numpoints=1, frameon=False)
    plt.tight_layout(pad=0.5)


def plot_reg(df, reg):
    """ Funcion que dibuja las regresiones entre las variables y la principal.

    Args:
        df: Dataframe con toda la informacion.
        name_v_ppal: nombre de la segunda variable principal.
        tipo_reg: Tipo de regresion seleccionada.
        umb: umbral

    Returns:

    """

    nomb = df.columns.values
    plt.style.use('ggplot')
    plt.figure(figsize=(9, 6))
    for i in range(1, np.shape(df)[1]):
        plt.subplot(2, 3, i)
        plt.plot(df[nomb[0]], df[nomb[i]], '.', color='gray', label='Medido')
        plt.plot(reg[nomb[i]].iloc[:, 0], reg[nomb[i]].iloc[:, 2], 'k', lw=2, label='Ajuste')
        plt.plot(reg[nomb[i]].iloc[:, 0], reg[nomb[i]].iloc[:, 1], '--k', label='I.C.-sup')
        plt.plot(reg[nomb[i]].iloc[:, 0], reg[nomb[i]].iloc[:, 3], ':k', label='I.C.-inf')
        plt.xlabel(nomb[0], fontweight='bold')
        plt.ylabel(nomb[i], fontweight='bold')

    plt.legend(loc='best', numpoints=1, frameon=False)
    plt.tight_layout(pad=0.5)

def plot_monomios(mon1, mon2, mon3, nomb_v_ppal1):
    """Funcion que dibuja la serie temporal que define la evolucion de los monomios
    """

    plt.style.use('ggplot')
    plt.figure(figsize=(9, 3))
    plt.subplot(3, 1, 1)
    x = np.arange(1, len(mon1)+1)
    plt.plot(x, mon1, '.', color='gray')
    plt.plot(x, np.ones(len(mon1))*np.mean(mon1), 'k')
    plt.ylabel(r'$\frac{g'+nomb_v_ppal1+'}{U_{10}^2}$', rotation='horizontal', horizontalalignment='right',
               fontsize='large', fontweight='bold')
    

    plt.subplot(3, 1, 2)
    x = np.arange(1, len(mon2)+1)
    plt.plot(x, mon2/1e4, '.', color='gray')
    plt.plot(x, np.ones(len(mon2))*np.mean(mon2)/1e4, 'k')
    plt.ylabel(r'$\frac{gD}{U_{10}} (x 10^4)$', rotation='horizontal', horizontalalignment='right',
               fontsize='large', fontweight='bold')

    plt.subplot(3, 1, 3)
    x = np.arange(1, len(mon3)+1)
    plt.plot(x, mon3/1e4, '.', color='gray')
    plt.plot(x, np.ones(len(mon3))*np.mean(mon3)/1e4, 'k')
#    plt.xticks(Visible=False)
    plt.ylabel(r'$\frac{gD_{asc}}{U_{10}} (x 10^4)$', rotation='horizontal', horizontalalignment='right',
               fontsize='large', fontweight='bold')
    plt.xlabel('Ciclos de solicitacion', fontweight='bold')
    plt.gcf().subplots_adjust(bottom=0.2, left=0.2)
