#!/usr/bin/env python2
# -*- coding: utf-8 -*-

import matplotlib.pyplot as plt
from scipy.stats import genpareto, kstest, weibull_min, anderson
import pandas as pd
import numpy as np
import os
from matplotlib import dates
from datetime import datetime

from clima_maritimo.clima_maritimo import analisis
from clima_maritimo.clima_maritimo.fdist.copula import Copula as Copula


def ciclos_registro_inicial_DI(df, var_ciclos, umbral, min_tD, min_tI):
    """Funcion que genera la serie de ciclos de solicitación del registro inicial

       Args:
           df: dataframe con las variables climaticas
           var_ciclos: variable que define los ciclos de solicitacion
           umbral: umbral incial para el corte de los ciclos 
           min_tD: numero mínimo de horas para que sea considerado ciclo
           min_tI: numero mínimo de horas para que sea considerado calma

       Returns:
           Un dataframe con información de los ciclos del registro original

       """

    hs_max_c, dur_c, dur_i, index_max_date = identificacion_ciclos_i_DI(df, var_ciclos, umbral)

    if hs_max_c is None:
        print('Warning: En el caso del umbral {:.2f}'.format(umbral) + ' m y t0' + str(min_tI) +
              ' h no existen suficientes tormentas para realizar el analsiis')
        df_vars_ciclos = pd.DataFrame(
            data={var_ciclos + '_max': np.NaN, 'dur_c': np.NaN, 'dur_i': np.NaN}, index=[0])
    else:
        hs_max_c, dur_c, dur_i = np.asarray(hs_max_c), np.asarray(dur_c), np.asarray(dur_i),

       # Quitar las filas donde la calma es menor de 6 horas. Se suma la calma y el ciclo siguiente al ciclo y se coge la calma del siguiente
        while (dur_i[:-1] < np.timedelta64(min_tI, 'h')).any():
            indx_i_no = np.where(dur_i < np.timedelta64(min_tI, 'h'))[0]
            indx_i_no = np.delete(indx_i_no, [np.where(np.diff(indx_i_no) == 1)[0] + 1])

            if indx_i_no[-1] + 1 == len(dur_c):
                hs_max_c = np.delete(hs_max_c, indx_i_no[-1])
                dur_c = np.delete(dur_c, indx_i_no[-1])
                dur_i = np.delete(dur_i, indx_i_no[-1])
                index_max_date = np.delete(index_max_date, indx_i_no[-1])
                indx_i_no = np.delete(indx_i_no, -1)

            dur_c[indx_i_no] = dur_c[indx_i_no] + dur_i[indx_i_no] + dur_c[indx_i_no + 1]
            dur_i[indx_i_no] = dur_i[indx_i_no + 1]

            for indx_i_no_ind in indx_i_no:
                index_max_date[indx_i_no_ind] = index_max_date[[indx_i_no_ind, indx_i_no_ind + 1]
                [np.argmax([hs_max_c[indx_i_no_ind], hs_max_c[indx_i_no_ind + 1]])]]
            hs_max_c[indx_i_no] = np.max([hs_max_c[indx_i_no], hs_max_c[indx_i_no + 1]], axis=0)

            hs_max_c = np.delete(hs_max_c, indx_i_no + 1)
            dur_c = np.delete(dur_c, indx_i_no + 1)
            dur_i = np.delete(dur_i, indx_i_no + 1)
            index_max_date = np.delete(index_max_date, indx_i_no + 1)

        # Quitar las filas donde el ciclo es menor de 6 horas. Se suma la calma y el ciclo a la calma del anterior
        while (dur_c < np.timedelta64(min_tD, 'h')).any():
            indx_c_no = np.where(dur_c < np.timedelta64(min_tD, 'h'))[0]
            indx_c_no = np.delete(indx_c_no, [np.where(np.diff(indx_c_no) == 1)[0] + 1])

            dur_i[indx_c_no - 1] = dur_i[indx_c_no - 1] + dur_c[indx_c_no] + dur_i[indx_c_no]

            hs_max_c = np.delete(hs_max_c, indx_c_no)
            dur_c = np.delete(dur_c, indx_c_no)
            dur_i = np.delete(dur_i, indx_c_no)
            index_max_date = np.delete(index_max_date, indx_c_no)

        df_vars_ciclos = pd.DataFrame(
            data={var_ciclos + '_max': hs_max_c, 'dur_c': dur_c, 'dur_i': dur_i})
        df_vars_ciclos.set_index(index_max_date, inplace=True)
        df_vars_ciclos['dur_c_hrs'] = df_vars_ciclos['dur_c'] / np.timedelta64(1, 'h')
        df_vars_ciclos['dur_i_hrs'] = df_vars_ciclos['dur_i'] / np.timedelta64(1, 'h')

    return df_vars_ciclos


def ciclos_registro_inicial_DIM(df, var_ciclos, umbral, min_tD, min_tI):
    """Funcion que genera la serie de ciclos de solicitación del registro inicial

       Args:
           df: dataframe con las variables climaticas
           var_ciclos: variable que define los ciclos de solicitacion
           umbral: umbral incial para el corte de los ciclos
           min_t: numero mínimo de horas para que sea considerado ciclo/calma

       Returns:
           Un dataframe con información de los ciclos del registro original

       """
    hs_max_c, dur_c, dur_i, index_max_date = identificacion_ciclos_i_DI(df, var_ciclos, umbral)
    print(hs_max_c, type(hs_max_c))
    if isinstance(hs_max_c, list):
        print('Warning: En el caso del umbral {:.2f}'.format(umbral) + ' m y t0' + str(min_tI) +
              ' h no existen suficientes tormentas para realizar el analisis')
        df_vars_ciclos = pd.DataFrame(
            data={var_ciclos + '_max': np.NaN, 'dur_c': np.NaN, 'dur_i': np.NaN}, index=[0])
    else:
        # hs_max_c, dur_c, dur_i = np.asarray(hs_max_c), np.asarray(dur_c), np.asarray(dur_i),

       # Quitar las filas donde la calma (interarrival-'i') es menor de min_t horas. Se suma la calma y el ciclo siguiente al ciclo y se coge la calma del siguiente
        ind_i = np.where(dur_i <= min_tI)[0]
        if ind_i.size > 1:
            while ind_i[-1]+1 == len(dur_i):
                dur_c, dur_i, hs_max_c, index_max_date = dur_c[:-1], dur_i[:-1], hs_max_c[:-1], index_max_date[:-1]
                ind_i = np.where(dur_i <= min_tI)[0]

            ind_0 = np.diff(ind_i) - 2 < 0
            # INTERVALO DE INDICES REPETIDOS
            ind0_c = np.asarray(np.where((ind_0[:-1] & ind_0[1:]) | (ind_0[:-1] & ~ ind_0[1:]))[0], dtype=np.int16)
            if ind0_c.size > 1:
                for i in ind0_c:
                    dur_c[ind_i[i]] += np.sum(dur_i[ind_i[i]]) + np.sum(dur_c[ind_i[i + 1]])

            ind_m1 = ind_i + 1
            dur_c[ind_i] += dur_i[ind_i] + dur_c[ind_m1]
            dur_i = np.delete(dur_i, ind_i)
            dur_c = np.delete(dur_c, ind_m1)

            hs_max_aux = np.max([hs_max_c[ind_i], hs_max_c[ind_m1]], axis=0)
            ind_max_aux = np.argmax([hs_max_c[ind_i], hs_max_c[ind_m1]], axis=0)
            mask = ind_max_aux == 1

            hs_max_c[ind_i] = hs_max_aux
            hs_max_c = np.delete(hs_max_c, ind_m1)
            index_max_date[ind_i[mask]] = index_max_date[ind_m1[mask]]
            index_max_date = np.delete(index_max_date, ind_m1)


            # Quitar las filas donde el ciclo es menor de 6 horas. Se suma la calma y el ciclo a la calma del anterior

            ind_c = np.where(dur_c <= np.array(min_tD, dtype='timedelta64[h]'))[0]
            if ind_c.size > 1:
                while ind_c[-1]+1 == len(dur_c):
                    dur_c, dur_i, hs_max_c, index_max_date = dur_c[:-1], dur_i[:-1], hs_max_c[:-1], index_max_date[:-1]
                    ind_c = np.where(dur_c <= np.array(min_tD, dtype='timedelta64[h]'))[0]

                ind_m1 = ind_c + 1
                dur_i[ind_c] += dur_i[ind_c] + dur_c[ind_m1]
                dur_i = np.delete(dur_i, ind_m1)
                dur_c = np.delete(dur_c, ind_c)

                hs_max_c = np.delete(hs_max_c, ind_c)
                index_max_date = np.delete(index_max_date, ind_c)

        df_vars_ciclos = pd.DataFrame(data={var_ciclos + '_max': hs_max_c, 'dur_c': dur_c, 'dur_i': dur_i},
                                      index= index_max_date)
        df_vars_ciclos['dur_c_hrs'] = df_vars_ciclos['dur_c'] / np.timedelta64(1, 'h')
        df_vars_ciclos['dur_i_hrs'] = df_vars_ciclos['dur_i'] / np.timedelta64(1, 'h')

    return df_vars_ciclos


def ciclos_registro_inicial(df, var_ciclos, umbral, min_t):
    """Funcion que genera la serie de ciclos de solicitación del registro inicial

       Args:
           df: dataframe con las variables climaticas
           var_ciclos: variable que define los ciclos de solicitacion
           umbral: umbral incial para el corte de los ciclos 
           min_t: numero mínimo de horas para que sea considerado ciclo/calma
           
       Returns:
           Un dataframe con información de los ciclos del registro original

       """
    hs_max_c, dur_c, dur_i, index_max, mag_c, ciclo_i, _ = identificacion_ciclos_i(df, var_ciclos, umbral, min_t)

    index_max_date = np.asarray(index_max)

    # Quitar las filas donde la calma es menor de 6 horas. Se suma la calma y el ciclo siguiente al ciclo y se coge la calma del siguiente
    while (dur_i < np.timedelta64(min_t, 'h')).any():
        indx_i_no = np.where(dur_i < np.timedelta64(min_t, 'h'))[0]
        indx_i_no = np.delete(indx_i_no, [np.where(np.diff(indx_i_no) == 1)[0] + 1])

        if indx_i_no[-1] + 1 == len(dur_c):
            hs_max_c = np.delete(hs_max_c, indx_i_no[-1])
            dur_c = np.delete(dur_c, indx_i_no[-1])
            dur_i = np.delete(dur_i, indx_i_no[-1])
            mag_c = np.delete(mag_c, indx_i_no[-1])
            index_max_date = np.delete(index_max_date, indx_i_no[-1])
            indx_i_no = np.delete(indx_i_no, -1)

        dur_c[indx_i_no] = dur_c[indx_i_no] + dur_i[indx_i_no] + dur_c[indx_i_no + 1]
        dur_i[indx_i_no] = dur_i[indx_i_no + 1]
        mag_c[indx_i_no] = mag_c[indx_i_no] + mag_c[indx_i_no + 1]

        for indx_i_no_ind in indx_i_no:
            ciclo_i[indx_i_no_ind] = ciclo_i[indx_i_no_ind].append(ciclo_i[indx_i_no_ind + 1])
            index_max_date[indx_i_no_ind] = index_max_date[[indx_i_no_ind, indx_i_no_ind + 1]
            [np.argmax([hs_max_c[indx_i_no_ind], hs_max_c[indx_i_no_ind + 1]])]]

        hs_max_c[indx_i_no] = np.max([hs_max_c[indx_i_no], hs_max_c[indx_i_no + 1]], axis=0)

        hs_max_c = np.delete(hs_max_c, indx_i_no + 1)
        dur_c = np.delete(dur_c, indx_i_no + 1)
        dur_i = np.delete(dur_i, indx_i_no + 1)
        mag_c = np.delete(mag_c, indx_i_no + 1)
        ciclo_i[:] = [item for i, item in enumerate(ciclo_i) if i not in indx_i_no + 1]
        index_max_date = np.delete(index_max_date, indx_i_no + 1)

    # Quitar las filas donde el ciclo es menor de 6 horas. Se suma la calma y el ciclo a la calma del anterior
    while (dur_c < np.timedelta64(min_t, 'h')).any():
        indx_c_no = np.where(dur_c < np.timedelta64(min_t, 'h'))[0]
        indx_c_no = np.delete(indx_c_no, [np.where(np.diff(indx_c_no) == 1)[0] + 1])

        dur_i[indx_c_no - 1] = dur_i[indx_c_no - 1] + dur_c[indx_c_no] + dur_i[indx_c_no]

        hs_max_c = np.delete(hs_max_c, indx_c_no)
        dur_c = np.delete(dur_c, indx_c_no)
        dur_i = np.delete(dur_i, indx_c_no)
        mag_c = np.delete(mag_c, indx_c_no)
        ciclo_i[:] = [item for i, item in enumerate(ciclo_i) if i not in indx_c_no]
        index_max_date = np.delete(index_max_date, indx_c_no)

    df_vars_ciclos = pd.DataFrame(
        data={var_ciclos + '_max': hs_max_c, 'dur_c': dur_c, 'dur_i': dur_i, 'mag_c': mag_c, 'df_ciclo_i': ciclo_i})
    df_vars_ciclos.set_index(index_max_date, inplace=True)
    df_vars_ciclos['dur_c_hrs'] = df_vars_ciclos['dur_c'] / np.timedelta64(1, 'h')
    df_vars_ciclos['dur_i_hrs'] = df_vars_ciclos['dur_i'] / np.timedelta64(1, 'h')

    df_vars_ciclos['tp'] = df.loc[df_vars_ciclos.index]['tp']
    df_vars_ciclos['dh'] = df.loc[df_vars_ciclos.index]['dh']
    rho = 1000
    g = 9.81
    E_hsmax = (rho * g * (df_vars_ciclos['hs_max'].values ** 2)) / 8  # N/m
    P_hsmax = (g * df_vars_ciclos['tp'].values * E_hsmax) / (4 * np.pi)  # W/m
    df_vars_ciclos['P'] = P_hsmax

    return df_vars_ciclos


def duracion_ciclos(df, umbral, var_ciclos):
    df['tag'] = df[var_ciclos] > umbral

    # DURACION CICLOS
    # first row is a True preceded by a False
    fst_c = np.asarray(df.index[df['tag'] & ~ df['tag'].shift(1).fillna(False)])
    fst_ca = np.asarray(df.index[~ df['tag'] & df['tag'].shift(-1).fillna(False)])

    if len(fst_ca) == len(fst_c):
        fst_ca = fst_ca[:-1]

    if len(fst_c) <= 2:  # Solo hay 2 tormentas en toda la serie. No es suficiente

        fst_c, fst_i, lst_c, lst_i = [], [], [], []
    else:
        t = (umbral-df.loc[fst_c[1:], var_ciclos]).values*(fst_c[1:]-fst_ca)/(df.loc[fst_c[1:], var_ciclos].values-
                                                                     df.loc[fst_ca, var_ciclos].values)
        fst_c[1:] += t

        # last row is a True followed by a False
        lst_c = np.asarray(df.index[df['tag'] & ~ df['tag'].shift(-1).fillna(False)])
        lst_cd = np.asarray(df.index[~ df['tag'] & df['tag'].shift(1).fillna(False)])

        t = (df.loc[lst_c, var_ciclos] - umbral).values*(lst_cd - lst_c)/(df.loc[lst_c, var_ciclos].values-
                                                                          df.loc[lst_cd, var_ciclos].values)
        lst_c += t

        # DURACION CALMA
        fst_i, lst_i = lst_c[:-1], fst_c[1:]
        fst_c, lst_c = fst_c[:-1], lst_c[:-1]

    return fst_c, fst_i, lst_c, lst_i


def identificacion_ciclos_i_DI(df_i, var_ciclos, umbral):

    """Funcion que genera informacion de los ciclos de solicitación del registro inicial

           Args:
               df: dataframe con las variables climaticas
               var_ciclos: variable que define los ciclos de solicitacion
               umbral: umbral incial para el corte de los ciclos
               min_tD: numero mínimo de horas para que sea considerado ciclo
               min_tI: numero mínimo de horas para que sea considerado calma

           Returns:
               lista con información de los ciclos del registro original
               :param min_t:

           """
    df = pd.DataFrame(df_i.loc[:, var_ciclos].copy())

    fst_c, fst_i, lst_c, lst_i = duracion_ciclos(df, umbral, var_ciclos)

    if isinstance(fst_c, np.ndarray):
        hs_max_c, index_max = (np.zeros([len(fst_c)]), np.zeros([len(fst_c)], dtype='datetime64[ms]'))
        for i, j in enumerate(fst_c):
            hs_max_c[i] = df.loc[j:lst_c[i], var_ciclos].max()
            index_max[i] = df.loc[j:lst_c[i], var_ciclos].idxmax()

        dur_c = lst_c - fst_c
        dur_i = lst_i - fst_i
    else:
        hs_max_c, dur_c, dur_i, index_max = None, None, None, None

    return hs_max_c, dur_c, dur_i, index_max


def identificacion_ciclos_i(df_i, var_ciclos, umbral, min_tD):

    """Funcion que genera informacion de los ciclos de solicitación del registro inicial

           Args:
               df: dataframe con las variables climaticas
               var_ciclos: variable que define los ciclos de solicitacion
               umbral: umbral incial para el corte de los ciclos 

           Returns:
               lista con información de los ciclos del registro original
               :param min_t: 

           """

    df = pd.DataFrame(df_i.loc[:, var_ciclos].copy())

    fst_c, fst_i, lst_c, lst_i = duracion_ciclos(df, umbral, var_ciclos)

    if isinstance(fst_c, np.ndarray):
        mag_c, ciclo_i = list()
        hs_max_c, index_max = (np.zeros([len(fst_c)]), np.zeros([len(fst_c)], dtype='datetime64[ms]'))
        for i, j in enumerate(fst_c):
            hs_max_c[i] = df.loc[j:lst_c[i], var_ciclos].max()
            index_max[i] = df.loc[j:lst_c[i], var_ciclos].idxmax()
            mag_c.append(
                np.trapz(df.loc[j:lst_c[i], var_ciclos].values - umbral,
                         df.loc[j:lst_c[i], var_ciclos].index) / np.timedelta64(1, 'h'))
            ciclo_i.append(df.loc[j:lst_c[i]])

        dur_c = lst_c - fst_c
        dur_i = lst_i - fst_i

        # Quitar las filas donde el ciclo es menor de 6 horas. Se suma la calma y el ciclo a la calma del anterior
        while (dur_c < np.timedelta64(min_tD, 'h')).any():
            indx_c_no = np.where(dur_c < np.timedelta64(min_tD, 'h'))[0]
            indx_c_no = np.delete(indx_c_no, [np.where(np.diff(indx_c_no) == 1)[0] + 1])

            dur_i[indx_c_no - 1] = dur_i[indx_c_no - 1] + dur_c[indx_c_no] + dur_i[indx_c_no]

            hs_max_c = np.delete(hs_max_c, indx_c_no)
            dur_c = np.delete(dur_c, indx_c_no)
            dur_i = np.delete(dur_i, indx_c_no)
            index_max = np.delete(index_max, indx_c_no)
            mag_c = np.delete(mag_c, indx_c_no)
            ciclo_i[:] = [item for i, item in enumerate(ciclo_i) if i not in indx_c_no]
            index_max_date = np.delete(index_max_date, indx_c_no)

        dur_c_hrs = dur_c / np.timedelta64(1, 'h')
    else:
        hs_max_c, dur_c, dur_i, index_max, mag_c, ciclo_i, dur_c_hrs = None, None, None, None, None, None, None

    return hs_max_c, dur_c, dur_i, index_max, mag_c, ciclo_i, dur_c_hrs


def ciclos_division_estaciones(df_vars_ciclos):
    """Funcion que divide el dataframe de ciclos por estaciones

       Args:
           df_vars_ciclos: dataframe con informacion de los ciclos
           
       Returns:
           Una lista con los dataframes de los ciclos diMvidos por estaciones
           
    """


    df_vars_ciclos_inv = df_vars_ciclos.loc[((df_vars_ciclos.index.month == 12) & (df_vars_ciclos.index.day >= 21)) |
                                            (df_vars_ciclos.index.month == 1) | (df_vars_ciclos.index.month == 2) |
                                            ((df_vars_ciclos.index.month == 3) & (df_vars_ciclos.index.day < 21))]

    df_vars_ciclos_prim = df_vars_ciclos.loc[((df_vars_ciclos.index.month == 3) & (df_vars_ciclos.index.day >= 21)) |
                                             (df_vars_ciclos.index.month == 4) | (df_vars_ciclos.index.month == 5) |
                                             ((df_vars_ciclos.index.month == 6) & (df_vars_ciclos.index.day < 21))]

    df_vars_ciclos_ver = df_vars_ciclos.loc[((df_vars_ciclos.index.month == 6) & (df_vars_ciclos.index.day >= 21)) |
                                            (df_vars_ciclos.index.month == 7) | (df_vars_ciclos.index.month == 8) |
                                            ((df_vars_ciclos.index.month == 9) & (df_vars_ciclos.index.day < 21))]

    df_vars_ciclos_oto = df_vars_ciclos.loc[((df_vars_ciclos.index.month == 9) & (df_vars_ciclos.index.day >= 21)) |
                                            (df_vars_ciclos.index.month == 10) | (df_vars_ciclos.index.month == 11) |
                                            ((df_vars_ciclos.index.month == 12) & (df_vars_ciclos.index.day < 21))]

    return df_vars_ciclos_inv, df_vars_ciclos_prim, df_vars_ciclos_ver, df_vars_ciclos_oto


def info_ciclos(df_vars_ciclos):
    """Funcion que proporciona informacion de los ciclos de solicitacion

       Args:
           df_vars_ciclos: dataframe con informacion de los ciclos

       Returns:
           ciclos_mm_yy: numero total de ciclos por mes y por año
           ciclos_yy = numero total de ciclos por año
           ciclos_mm_tot = numero total de ciclos por mes
           ciclos_mm_mean = media de ciclos por mes
           ciclos_mm_std = desviacion tipica de ciclos por mes

    """

    ciclos_mm_yy = df_vars_ciclos.resample('M').size()  # numero de ciclos por mes por año
    ciclos_yy = df_vars_ciclos.resample('A').size()  # numero de ciclos por año
    ciclos_mm_tot = df_vars_ciclos.groupby(df_vars_ciclos.index.month).size()  # total de ciclos por mes
    ciclos_mm_mean = ciclos_mm_yy.groupby(ciclos_mm_yy.index.month).mean()  # media de ciclos por mes
    ciclos_mm_std = ciclos_mm_yy.groupby(ciclos_mm_yy.index.month).std()  # std de ciclos por mes

    return ciclos_mm_yy, ciclos_yy, ciclos_mm_tot, ciclos_mm_mean, ciclos_mm_std


def ajuste_marginal_dtemporal_eventos(df_eventos, tipo_ajuste, no_ord, fun, no_norm):
    """Funcion que proporciona los ajustes marginales de la serie por encima de un umbral original

       Args:
           df_eventos: dataframe con las variables climaticas
           tipo_ajuste: seleccion del tipo de ajuste para cada variable
                - 0: LO CHEQUEAS DURANTE EL PROCESO
                - 1: ESTACIONARIO
                - 2: NO ESTACIONARIO
            no_ord: orden de fourier para el analisis no estacionario y para cada variable
            fun: funcion utilizada para cada variable
            no_norm: numero de los normales para los casos direccionales (False en el caso lineal)
                       
       Returns:
           Un dataframe con información de los ciclos del registro original

       """

    par, mod, f_mix, data_graph = list(), list(), list(), list()
    for i, j in enumerate(df_eventos.columns):
        analisis_ = analisis.analisis(df_eventos[j], fun[i], tipo_ajuste[i], ordg=no_ord[i], nnorm=no_norm[i])
        par.append(analisis_[0])
        mod.append(analisis_[1])
        f_mix.append(analisis_[2])
        data_graph.append(analisis_[3])

    df_dt = analisis.dependencia_temporal(df_eventos, par, mod, no_norm, f_mix, fun)

    return par, mod, f_mix, df_dt


def simulacion_tiempos_ciclos_copulas(df_pri, df_ver, df_oto, df_inv, var_x, var_y, vars, ciclo, duraciones, Ny):
    """Funcion que proporciona una serie temporal de ciclos simulada

          Args:
              df_pri: dataframe con los datos de los ciclos del registro inicial en primavera
              df_ver: dataframe con los datos de los ciclos del registro inicial en verano
              df_oto: dataframe con los datos de los ciclos del registro inicial en otoño
              df_inv: dataframe con los datos de los ciclos del registro inicial en invierno
              var_x: variable x de analisis por copulas
              var_y: variable y de analisis por copulas
              vars: variables climaticas para analizar
              ciclo: dataframe con los datos de la lista de ciclos simulados
              duracion: duraciones de los ciclos individuales de la lista de ciclos
              Ny: numero de años a simular
              
          Returns:
              lista con informacion de la serie temporal de ciclos simulada
          """


    # SIMULAMOS LAS DURACIONES DE CALMA Y LAS VAMOS AÑADIENDO AL VECTOR DE CICLOS

    parFMPri = weibull_min.fit(df_pri[var_x].values[:])
    copClaytonPri = Copula(df_pri[var_x], df_pri[var_y], 'clayton')

    parFMVer = weibull_min.fit(df_ver[var_x].values[:])
    copClaytonVer = Copula(df_ver[var_x], df_ver[var_y], 'clayton')

    parFMOto = weibull_min.fit(df_oto[var_x].values[:])
    copClaytonOto = Copula(df_oto[var_x], df_oto[var_y], 'clayton')

    parFMInv = weibull_min.fit(df_inv[var_x].values[:])
    copClaytonInv = Copula(df_inv[var_x], df_inv[var_y], 'clayton')

    j, k, calma = 0, datetime(2000, 1, 1), list()
    ke = datetime(2000 + Ny, 1, 1)
    dcs = pd.DataFrame(columns=vars)
    times = pd.DataFrame(columns=['fi', 'ff', 'Dc', 'I'])

    xd_pri, yd_pri, xd_ver, yd_ver, xd_oto, yd_oto, xd_inv, yd_inv = list(), list(), list(), list(), list(), list(), list(), list()
    while k < ke:
        mask_3 = ((ciclo[j].index.hour % 3 == 0) & (ciclo[j].index.minute == 0))
        t_index = ciclo[j].index[mask_3] - ciclo[j].index[0] + k
        if not mask_3[0]:
            t_index = t_index.insert(0, ciclo[j].index[0] - ciclo[j].index[0] + k)
            mask_3[0] = True
        if not mask_3[-1]:
            t_index = t_index.insert(-1, ciclo[j].index[-1] - ciclo[j].index[0] + k)
            mask_3[-1] = True

        if ((k > pd.datetime(k.year, 3, 21)) & (k <= pd.datetime(k.year, 6, 21))):
            copClaytonPri.U = weibull_min.cdf(duraciones[0, j], *parFMPri)
            xd, yd = copClaytonPri.generate_xy(n=1)
            xd_pri.append(xd)
            yd_pri.append(yd)
        elif ((k > pd.datetime(k.year, 6, 21)) & (k <= pd.datetime(k.year, 9, 21))):
            copClaytonVer.U = weibull_min.cdf(duraciones[0, j], *parFMVer)
            xd, yd = copClaytonVer.generate_xy(n=1)
            xd_ver.append(xd)
            yd_ver.append(yd)
        elif ((k > pd.datetime(k.year, 9, 21)) & (k <= pd.datetime(k.year, 12, 21))):
            copClaytonOto.U = weibull_min.cdf(duraciones[0, j], *parFMOto)
            xd, yd = copClaytonOto.generate_xy(n=1)
            xd_oto.append(xd)
            yd_oto.append(yd)
        else:
            copClaytonInv.U = weibull_min.cdf(duraciones[0, j], *parFMInv)
            xd, yd = copClaytonInv.generate_xy(n=1)
            xd_inv.append(xd)
            yd_inv.append(yd)

        dff = pd.DataFrame(ciclo[j][mask_3].values[:, 0:3], index=t_index, columns=vars).sort_index()
        dcs = dcs.append(dff[['hs', 'tp', 'dh']])
        times = times.append({'fi': dff.index[0], 'ff': dff.index[-1], 'Dc': duraciones[0, j],
                              'I': pd.Timedelta(hours=yd[0]), 'hs_max': dff['hs'].max(),
                              'tp_aso': dff['tp']['hs' == dff['hs'].max()],
                              'dh_aso': dff['dh']['hs' == dff['hs'].max()]},
                             ignore_index=True)

        k += pd.Timedelta(hours=yd[0] + duraciones[0, j])
        j += 1
        calma.append(yd[0])

    n_aux = dcs.index.map(dates.date2num)
    n_anio = map(datetime, dcs.index.year, np.ones(len(dcs), dtype=int), np.ones(len(dcs), dtype=int))
    n_anio = np.array(map(dates.date2num, n_anio))
    n_anio1 = map(datetime, dcs.index.year + 1, np.ones(len(dcs), dtype=int), np.ones(len(dcs), dtype=int))
    n_anio1 = np.array(map(dates.date2num, n_anio1))
    n = (n_aux - n_anio) / (n_anio1 - n_anio)

    times['I_hrs'] = times['I'] / pd.Timedelta('1hours')
    rho = 1000
    g = 9.81
    E_hsmax = (rho * g * (times['hs_max'].values ** 2)) / 8  # N/m
    P_hsmax = (g * times['tp_aso'].values * E_hsmax) / (4 * np.pi)  # W/m
    times['P'] = P_hsmax

    times.set_index(times.fi, inplace=True)
    cicloss_mm_yy = times.resample('M').size()
    cicloss_mm_mean = cicloss_mm_yy.groupby(cicloss_mm_yy.index.month).mean()
    cicloss_mm_std = cicloss_mm_yy.groupby(cicloss_mm_yy.index.month).std()

    dcs = dcs.sort_index()

    return n, dcs, times, cicloss_mm_yy, cicloss_mm_mean, cicloss_mm_std, xd_pri, yd_pri, xd_ver, yd_ver, xd_oto, yd_oto, xd_inv, yd_inv
