# -*- coding: utf-8 -*-
from __future__ import division
import numpy as np
import scipy.stats as st
from statsmodels.distributions import empirical_distribution
from clima_maritimo.clima_maritimo.fdist import normenr, normtrun, lognorm_genpar2, lognorm2, weib_genpar2, norm_genpar2
from clima_maritimo.clima_maritimo.fdist import ajuste_simple as ajs
from clima_maritimo.graficas import plot_analisis
import logging
from datetime import datetime
from matplotlib import dates
import pandas as pd
import matplotlib.pyplot as plt
import warnings
logging.basicConfig(filename='debug_info.log', level=logging.INFO)


def var_fit_ols(y, z):
    """Funcion que estima los parametros del modelo VAR siguiendo el metodo de minimos cuadrados descrito en
    Lutkepohl (Ecuaciones 3.2.1 y 3.2.10)

    Args:
        y: Matriz X en Lutkepohl
        z: Matriz Z en Lutkepohl

    Returns:
        Un dataframe que contiene las matrices B, Q, y U en Lutkepohl y el bic

    """

    df = pd.Series()
    m1, m2 = np.dot(y, z.T), np.dot(z, z.T)
    df['B'] = np.dot(m1, np.linalg.inv(m2))
    df['U'] = y - np.dot(df['B'], z)
    df['Q'] = np.cov(df['U'])
    nel, df['dim'] = np.shape(df['U'].T)
    llf = np.sum(np.log(st.multivariate_normal.pdf(df['U'].T, mean=np.zeros(df['dim']), cov=df['Q'])))
    bic = -2*llf + np.log(nel)*np.size(np.hstack((df['B'], df['Q'])))

    return df, bic


def ajusta_var(data, ord_):
    """Funcion que elige el modelo VAR con menor BIC. Acomoda la variable data y calcula dimension del modelo (numero
    de V.A. del vector) y el numero de observaciones

    Args:
        data: Datos a ajustar.
        ord_: Maximo orden a tener en cuenta en el ajuste

    Returns:
        Un dataframe con los parametros del ajuste

    """

    [dim, t] = np.shape(data)
    t = t-ord_
    bic = np.zeros(ord_)
    mask = ((data == np.nan) | (data == -np.inf) | (data == np.inf))
    data[mask] = 0
    par_dt = [list() for i in range(ord_)]
    for p in range(1, ord_+1):
        y = data[:, ord_:]
        z0 = np.zeros([p*dim, t])
        for i in range(1, p+1):
            z0[(i-1)*dim:i*dim, :] = data[:, ord_-i:-i]
        z = np.vstack((np.ones(t), z0))
        par_dt[p-1], bic[p-1] = var_fit_ols(y, z)
        print 'no. elementos: ' + str(i) + ',  BIC: ' + str(bic[p-1])

    id_ = np.argmin(bic)
    par_dt = par_dt[id_]
    par_dt['id-bic'] = int(id_)+1

    if id_+1 == ord_:
        logging.info('¡Cuidado! El modelo de menor BIC coincide con el de mayor orden analizado')

    return par_dt


def mejor_ajuste(bic, ord_, terms):
    """Elige el modelo de menor bic de todos los ajustados

    Args:
        bic: Matriz con el bic calculado de cada modelo
        ord_: Orden maximo del modelo
        terms: Numero de conjuntos sometidos a la expansión de Fourier

    Returns:
        Un vector con los parametros del modelo seleccionado

    """

    mod = np.zeros(terms, dtype=int)
    pos = np.nanargmin(np.abs(bic))
    for i in range(1, terms+1):
        if pos < 0:
            pos = 0
        mod[terms-i] = pos/ord_**(terms - i)
        pos = pos - mod[terms-i]*ord_**(terms - i)

    return mod


def analisis_est(df, fun, f_mix, nnorm):
    """ Funcion que realiza el ajuste estacionario de los datos de entrada a una funcion de probabilidad unimodal y
    mixta dada.

    Args:
        - df: Dataframe con los datos de la variable de partida
        - funp: Variable que contiene la funcion de probabilidad a la que ajustar
        - f_mix: Valor logico (True: Ajuste mixto; False: Ajuste simple)
        - nnorm [opc]: Numero de normales (ajuste direccional)

    Returns:
        Los parametros y el modo del mejor ajuste

    """
    mod = [0, 0, 0]
    nvar = df.columns.values[0]
    if not f_mix:
        par0 = fun.fit(df[nvar])
        mod = np.zeros(len(par0), dtype=int)
    elif ((fun == 'normenr') | (fun == 'normtrun')):
        data = df[nvar]*np.pi/180
        par0 = fun.prepara_param(data, nnorm)
        mod = np.zeros(nnorm, dtype=int)
        par0, _ = fun.fit(par0, data, mod, nnorm)
    else:
        par0 = fun.prepara_param(df[nvar])
        par0, _ = fun.fit(par0, df[nvar])

    return par0, mod


def aj_nest(df, var_, f_mix, ordg, fn, par0, nnorm, fun):
    """ Funcion que se encarga de realizar el ajuste no estacionario y seleccionar el modo de mejor ajuste

    Args:
        df: Dataframe con la variable
        var_: String con el nombre de la variable
        f_pro: Valor logico (False - ajuste simple; True - ajuste mixto)
        ordg: Orden maximo del ajuste
        par0: Parametros iniciales
        nnorm: Numero de normales
        fun: Funcion de probabilidad

    Returns:
        Una lista con:

        * ``par``: Parametros de todos los modos ajustados
        * ``bic``: Criterio de informacion bayesiana
        * ``mod``: Modo del mejor ajuste
        * ``par_ne``: Parametros del mejor ajuste no estacionario

    """
    if not f_mix:
        par, bic = ajs.n_est(ordg, df[var_].values, fn, par0, fun)
        mod = mejor_ajuste(bic, ordg, len(par0)) + np.ones(len(par0), dtype=np.int)
        if len(par0) == 2:
            par_ne = par[mod[0]-1][mod[1]-1]
        else:
            par_ne = par[mod[0]-1][mod[1]-1][mod[2]-1]
    elif (((var_ == 'dh') | (var_ == 'dv')) & f_mix):
        par, bic = fun.n_est(ordg, par0, nnorm, df[var_], fn)
        mod = mejor_ajuste(bic, ordg, nnorm) + np.ones(nnorm, dtype=int)
        if nnorm == 2:
            par_ne = par[mod[0]-1][mod[1]-1]
        elif nnorm == 3:
            par_ne = par[mod[0]-1][mod[1]-1][mod[2]-1]
        else:
            par_ne = par[mod[0]-1][mod[1]-1][mod[2]-1][mod[3]-1]
    else:
        par, bic = fun.n_est(ordg, df[var_], fn, par0)
        mod = mejor_ajuste(bic, ordg, 3) + [1, 1, 1]
        par_ne = par[mod[0]-1][mod[1]-1][mod[2]-1]
    print 'Ajuste de parámetros terminado'
    return par, bic, mod, par_ne


def analisis(df, fun, cnt, ordg=1, nnorm=False, par0=0):
    """Funcion que analiza los datos de un punto WANA/SIMAR (Puertos del Estado) o fichero . Utiliza modelos
    estacionarios y no  estacionarios para describir las funciones de distribucion de las principales variables tal y
    como se describe en Solari y Losada (2011).

    Args:
        - df: Dataframe del estado
        - fun: Modelo de probabilidad para ajustar
        - cnt: Tipo de analisis (0: a chequear; 1: estacionario, 2: no estacionario)
        - ordg[opc]: Orden maximo del ajuste no estacionario
        - nnorm[opc]: numero de normales para el analisis direccional
        - par0: parametros iniciales para el ajuste no estacionario

    Returns:
        Un diccionario con los resultados del analisis
          * ``par``: Parametros del ajuste
          * ``mod``: Modo del ajuste
          * ``f_pro``: Valor logico (False - ajuste simple; True - ajuste mixto)
          * info para las funciones graficas

    """
    # LISTADO DE MODELOS DE PROBABILIDAD MIXTOS ADMITIDOS
    list_fun = [normenr, normtrun, lognorm_genpar2, lognorm2, weib_genpar2, norm_genpar2]
    
    df = pd.DataFrame(df)

    n_aux = df.index.map(dates.date2num)
    n_anio = map(datetime, df.index.year, np.ones(len(df), dtype=int), np.ones(len(df), dtype=int))
    n_anio = np.array(map(dates.date2num, n_anio), dtype=np.float64)
    n_anio1 = map(datetime, df.index.year + 1, np.ones(len(df), dtype=int), np.ones(len(df), dtype=int))
    n_anio1 = np.array(map(dates.date2num, n_anio1), dtype=np.float64)
    df.ix[:, 'n'] = ((n_aux - n_anio)/(n_anio1 - n_anio))

    var_ = df.columns.values[0]

    # FORMA MAS SENCILLA Y RAPIDA DE CALCULARLO PERO NECESITA UNA VERSION SUPERIOR DE PANDAS
#    df['n'] = ((df.index.dayofyear + df.index.hour/24. - 1)/pd.to_datetime(
#            {'year': df.index.year, 'month': 12, 'day': 31, 'hour': 23}).dt.dayofyear).values

    if fun in list_fun:
        f_mix = True
    else:
        f_mix = False

    # AJUSTE ESTACIONARIO O NO ESTACIONARIO
    cheqaj = 0
    if not(cnt):
        plot_analisis.cuantiles_ne(df['n'], df[var_], var_)
        plt.pause(0.5)
        cheqaj = raw_input('¿Que tipo de ajuste desea?\n        - 1: Estacionario\n'
                           '          2: No estacionario\n')
        cheqaj = int(cheqaj)
    if ((cheqaj == 1) | (cnt == 1)):
        if not f_mix:
            print 'Ajuste estacionario de ' + var_ + ' con el modelo de probabilidad ' + fun.name
        else:
            print 'Ajuste estacionario de ' + var_ + ' con el modelo de probabilidad ' + fun.__name__
        logging.info('')
        logging.info('Se ha seleccionado ajuste estacionario para ' + var_)
        logging.info('------------------------------------------------------------------')
        logging.info('')

        par, mod = analisis_est(df, fun, f_mix, nnorm)
        cnt = 1
    elif (((cheqaj == 2) | (cnt == 2)) & isinstance(par0, int)):
        if not f_mix:
            print 'Ajuste no estacionario de ' + var_ + ' con el modelo de probabilidad ' + fun.name
        else:
            print 'Ajuste no estacionario de ' + var_ + ' con el modelo de probabilidad ' + fun.__name__
        logging.info('')
        logging.info('Se ha seleccionado ajuste no estacionario para ' + var_)
        logging.info('------------------------------------------------------------------')
        logging.info('')

        par0, mod = analisis_est(df, fun, f_mix, nnorm)
        par0, bic, mod, par = aj_nest(df, var_, f_mix, ordg, df['n'], par0, nnorm, fun)
        cnt = 2
    elif (((cheqaj == 2) | (cnt == 2)) & (isinstance(par0, np.ndarray))):
        if not f_mix:
            print 'Ajuste no estacionario de ' + var_ + ' con el modelo de probabilidad ' + fun.name
            print ' y los parametros dados'
        else:
            print 'Ajuste no estacionario de ' + var_ + ' con el modelo de probabilidad ' + fun.__name__
            print ' y los parametros dados'
        logging.info('')
        logging.info('Se ha seleccionado ajuste no estacionario para ' + var_)
        logging.info('------------------------------------------------------------------')
        logging.info('')

        par0, bic, mod, par = aj_nest(df, var_, f_mix, ordg, df['n'], par0, nnorm, fun)
        cnt = 2
    else:
        logging.info('La opción de ajuste seleccionada no es válida')
        warnings.warn('La opción de ajuste seleccionada no es válida', RuntimeError)

    # TAMBIEN SE DEVUELVE LA INFO PARA GRAFICAS
    data_graph = df['n'], df[var_], var_, par, mod, fun, nnorm

    return par, mod, f_mix, data_graph


def dependencia_temporal(df, par, mod, nnorm, f_mix, f_pro):
    """ Funcion que calcula los parametros de dependencia temporal entre las series mediante un modelo VAR (Solari y
    van Gelder, 2011).

    Args:
        - df: Dataframe con todas las variables
        - par: Lista con los parametros de los ajustes
        - mod: Lista con los modos de los ajustes
        - nnorm: Numero de normales para el analisis direccional
        - f_mix: Modelos de probabilidad de cada variable
        - f_pro: Lista con valores logicos (True: ajuste mixto; False: ajuste simple)

    Returns:
        Un dataframe con los parametros del modelo VAR

    """

    # Paso las fechas a intervalo [0 - 1] y me quedo con el analisis donde tengo info
    n_aux = df.index.map(dates.date2num)
    n_anio = map(datetime, df.index.year, np.ones(len(df), dtype=int), np.ones(len(df), dtype=int))
    n_anio = np.array(map(dates.date2num, n_anio), dtype=np.float64)
    n_anio1 = map(datetime, df.index.year + 1, np.ones(len(df), dtype=int), np.ones(len(df), dtype=int))
    n_anio1 = np.array(map(dates.date2num, n_anio1), dtype=np.float64)
    df['n'] = (n_aux - n_anio)/(n_anio1 - n_anio)

    cdf_ = pd.DataFrame(index=df['n'], columns=df.columns[:-1])
    for i, j in enumerate(df.columns[:-1]):
        if not f_mix[i]:
            cdf_[j] = ajs.cdf(df['n'], df[j], par[i], mod[i], f_pro[i])
        elif ((f_pro[i] == 'normenr') | (f_pro[i] == 'normtrun')):
            cdf_[j] = f_pro[i].cdf(df['n'], df[j], par[i], mod[i], nnorm[i])
        else:
            cdf_[j] = f_pro[i].cdf(df['n'], df[j], par[i], mod[i])

    z = np.zeros(np.shape(cdf_))
    for i, j in enumerate(cdf_):
        z[:, i] = st.norm.ppf(cdf_[j])

    df_dt = ajusta_var(z.T, 10)
    return df_dt
