# -*- coding: utf-8 -*-
from __future__ import division
import pandas as pd
import numpy as np
from operator import itemgetter
from itertools import groupby
from scipy.optimize import fsolve
 
 
 
def interseccion_series(df_oleaje, df_viento, df_marea, cadencia, informe, agentes=None):
    """Funcion para obtener la serie temporal de datos simultaneos de todos los agentes
 
    Args:
        df_oleaje: dataframe de datos de oleaje
        df_viento: dataframe de datos de viento
        df_marea: dataframe de datos de nivel de mar
        cadencia: cadencia en segundos que se establece a todos los agentes
        informe: nombre del archivo del informe
        agentes: nombre de los agentes. Predefinido a "OLEAJE", "VIENTO", "MAREA"
 
    Returns:
        df_clima: dataframe con los datos simultaneos de todos los agentes
        Se escriben resultados de calidad en el informe.
 
    """
 
    if agentes is None:
        agentes = ['OLEAJE', 'VIENTO', 'MAREA']
 
    dfs = [df_oleaje, df_viento, df_marea]
    nvars = [3, 2, 1]
    for j, df in enumerate(dfs):
        analisis_calidad_series(df, informe, agentes[j])
 
    df_inter_1 = pd.merge(df_oleaje, df_viento, how='inner', left_index=True, right_index=True)
    df_clima = pd.merge(df_inter_1, df_marea, how='inner', left_index=True, right_index=True)
 
    rng = pd.date_range(df_clima.index.min(), df_clima.index.max(), freq=pd.Timedelta(str(cadencia) + 'seconds'))
 
    df_clima = df_clima.reindex(rng, copy=False)
    analisis_calidad_series(df_clima, file, 'CLIMA')
 
    df_oleaje = df_oleaje.reindex(rng, copy=False)
    df_viento = df_viento.reindex(rng, copy=False)
    df_marea = df_marea.reindex(rng, copy=False)
 
    return df_clima, df_oleaje, df_viento, df_marea
 
 
def analisis_calidad_series(df_in, informe, agente):
    """Funcion que analiza las características de longitud y continuidad (huecos) de las series temporales de un agente
    en relación a todas sus variables
 
    Args:
        df: dataframe del agente
        informe: nombre del archivo del informe
        agente: nombre del agente.
 
    Returns:
        Se escriben resultados de calidad en el informe.
    """
    df = df_in.copy(deep=True)
    df['td'] = df.index.to_series() - df.index.to_series().shift()
 
    saltos = df['td'].value_counts()
    nodatos = len(df.index)
    saltos = saltos[saltos < 0.01 * nodatos]
    index_nan = df.loc[df['td'].isin(saltos.index)]
 
    for j in range(len(index_nan)):
        df_temp = df.loc[:index_nan.index[j]][-2:]
        rng = pd.date_range(df_temp.index.min(), df_temp.index.max(), freq=df_temp.td[0])
        df_temp = df_temp.reindex(rng)
        df = df.combine_first(df_temp)
 
    df.drop('td', 1, inplace=True)
    df['td'] = df.index.to_series() - df.index.to_series().shift()
    cadencias = df['td'].value_counts()
    cadencias = cadencias[cadencias > 0.01 * nodatos]
    # nodatos = len(df.index)
 
    lontiempo = df.index.max() - df.index.min()  # Intervalo de tiempo de la serie
    lontiempo_yy = np.floor(lontiempo.days / 365)
    lontiempo_dd = (lontiempo.days / 365. - lontiempo_yy) * 365
 
    informe.write(agente + '\n')
    str_1 = 'La serie temporal tiene una longitud de ' + str(lontiempo_yy) + ' años y ' + str(lontiempo_dd) + ' días'
    print(str_1)
    informe.write(str_1 + '\n')
 
    str_3 = 'La serie temporal tiene las siguientes cadencias:'
    print(str_3)
    informe.write(str_3 + '\n')
 
    str_4 = ''
    for j, cad in enumerate(cadencias):
        str_4 += str(cad) + ' datos con una cadencia de ' + str(cadencias.index[j]) + '\n'
    print(str_4)
    informe.write(str_4 + '\n')
 
    df.drop('td', 1, inplace=True)
    huecos_variables_series(df, informe)
 
    return df
 
 
def huecos_variables_series(df_in, informe):
    """Funcion que analiza las características de longitud y continuidad (huecos) de las series temporales de un agente
    en relación a todas sus variables
 
    Args:
        df_in: dataframe del agente
        informe: nombre del archivo del informe
 
    Returns:
        Se escriben resultados de calidad en el informe.
    """
    df = df_in.copy(deep=True)
 
    nvars = len(df.columns)
 
    lontiempo = df.index.max() - df.index.min()  # Intervalo de tiempo de la serie
    nodatos = len(df.index)
    max_dtgaps_vars = pd.Series([pd.Timedelta(days=0) for i in range(nvars)])
    pgaps_vars = np.zeros(nvars)
    nombre_vars = np.chararray(nvars, itemsize=5)
    for j in range(nvars):
        # Porcentaje de huecos
        nans = df.iloc[:, j].isnull()
        pgaps = nans.sum() / nodatos
        pgaps_vars[j] = pgaps
        nombrevar = df.iloc[:, j].name
        nombre_vars[j] = nombrevar
 
        # Continuidad de los huecos
        if pgaps != 0:
            gps_index = find_gap(df, j)
            columns = ['ini', 'fin']
            df_gps_index = pd.DataFrame(columns=columns)
            df_gps_index.ini = df.index[gps_index.index]
            df_gps_index.fin = df.index[gps_index.values]
            dt_gaps = df_gps_index.fin - df_gps_index.ini
            max_dtgaps_vars[j] = dt_gaps.max()
 
    pgaps = max(pgaps_vars)  # Mayor porcentaje (sobre 1) de huecos
    lontiempo_gaps = pgaps * lontiempo.total_seconds()  # Intervalo de tiempo de gaps de oleaje
 
    lontiempo_gaps_yy_t = lontiempo_gaps / (3600 * 24 * 365)
    lontiempo_gaps_yy = np.floor(lontiempo_gaps_yy_t)
    lontiempo_gaps_mm = np.floor((lontiempo_gaps_yy_t - lontiempo_gaps_yy) * 365)
 
    str_2 = 'La serie temporal tiene un total de ' + str(lontiempo_gaps_yy) + ' años y ' + str(lontiempo_gaps_mm) + \
            ' días sin datos (huecos) correspondiente a un {:.2f}' .format(pgaps * 100) + '% de la serie'
 
    print(str_2)
    informe.write(str_2 + '\n')
 
    str_3 = 'La duración máxima de huecos para la variable es de:'
    print(str_3)
    informe.write(str_3 + '\n')
 
    str_4 = ''
    for j, nombre in enumerate(nombre_vars):
        str_4 += str(max_dtgaps_vars[j]) + '\n'
        # str_4 += nombre + ': ' + str(max_dtgaps_vars[j]) + '\n'
 
    print(str_4)
    informe.write(str_4 + '\n')
 
 
def establecer_cadencia_series(df_in, cadencia, maxinterp):
    """Función que establece la misma cadencia a lo largo de toda la serie
 
    Args:
        cadencia: Cadencia en segundos que se establece a todos los agentes
        informe: nombre del archivo del informe
        df: dataframe de serie temporal
        maxinterp: máximo número de elemntos consecutivos a interpolar
 
 
    Returns:
        df_mod: dataframe con la misma cadencia en toda la serie
 
    """
 
    # dt = df.index.to_series() - df.index.to_series().shift()
    df = df_in.copy(deep=True)
    if cadencia == 10800:
        rng3 = [0, 3, 6, 9, 12, 15, 18, 21]
        posrng3 = [i for i, j in enumerate(rng3) if j >= df.index.min().hour]
        minrng = df.index.min().replace(hour=rng3[posrng3[0]])
    else:
        minrng = df.index.min()
    rng = pd.date_range(minrng, df.index.max(), freq=pd.Timedelta(str(cadencia) + 'seconds'))
    df = df.reindex(rng, copy=False, method='nearest', limit=maxinterp)
    firstd = df.first_valid_index()
    lastd = df.last_valid_index()
    df = df[firstd:lastd]
 
    # interpolación de direcciones
    if 'dh' in df.columns:
        complement360 = np.rad2deg(np.unwrap(np.deg2rad(df[df.dh.notnull()].dh.values)))
        df.dh.loc[df.dh.notnull()] = complement360
    elif 'dv' in df.columns:
        complement360 = np.rad2deg(np.unwrap(np.deg2rad(df[df.dv.notnull()].dv.values)))
        df.dv.loc[df.dv.notnull()] = complement360
 
    # se interpolan todas las variable
    df.interpolate(method='linear', inplace=True, limit=maxinterp)
 
    # direcciones
    if 'dh' in df.columns:
        df.dh = df.dh % 360
    elif 'dv' in df.columns:
        df.dv = df.dv % 360
 
    str_1 = 'Se ha interpolado a la cadencia de ' + str(pd.Timedelta(str(cadencia)
                                                                     + 'seconds'))
#    print(str_1)
#    informe.write(str_1 + '\n')
 
    return df
 
 
def recomend_alcance_series(alcanceproyecto, vidautil, df, informe):
    """Funcion que analiza la longitud de las series climáticas en relación al alcance del proyecto y la vida útil de la
     obra
 
    Args:
        alcanceproyecto:
 
            * 1 Estudio Previo
            * 2 Seleccion de Alternativas
            * 3 Anteproyecto
            * 4 Proyecto de Inversión
 
        vidautil: Años de vida útil de la obra
 
        df: dataframe del agente
 
 
 
 
    Returns:
        Informe de los datos.
        """
    nvars = len(df.columns)
 
    por_longitud_min_fases = [0.2, 0.4, 0.7, 1]  # porcentaje mínimo respecto a vidautil
    por_huecos_max_fases = [0.2, 0.15, 0.1, 0.05]  # porcentaje máximo respecto a huecos
    cadencia_min_fases = [1, 1, 1, 1]  # cadencia mínima en horas
 
    # Longitud de la series respecto a la vida util y fase de proyecto
    days_vidautil = vidautil * 365
    vidautil_td = pd.Timedelta(str(days_vidautil) + 'days')
 
    longitud_min = por_longitud_min_fases[alcanceproyecto - 1] * vidautil_td
    longitud_df = df.index.max() - df.index.min()  # Intervalo de tiempo de la serie
 
    if longitud_df >= longitud_min:
        str_1 = 'La serie cumple las recomendaciones de calidad de longitud según la fase del proyecto'
        print(str_1)
        informe.write(str_1 + '\n')
    else:
        str_1 = 'La serie no cumple las recomendaciones de calidad de longitud según la fase del proyecto'
        print(str_1)
        informe.write(str_1 + '\n')
 
    # Cadencia de la serie
    cadencia_min = pd.Timedelta(str(cadencia_min_fases[alcanceproyecto - 1]) + 'hours')
 
    dt = df.index.to_series() - df.index.to_series().shift()
    cadencia_serie = dt.mode()[0]
 
    if cadencia_serie <= cadencia_min:
        str_1 = 'La serie cumple las recomendaciones de calidad de cadencia según la fase del proyecto'
        print(str_1)
        informe.write(str_1 + '\n')
    else:
        str_1 = 'La serie no cumple las recomendaciones de calidad de cadencia según la fase del proyecto'
        print(str_1)
        informe.write(str_1 + '\n')
 
    # Huecos de la serie
    longitud_gaps_max = por_huecos_max_fases[alcanceproyecto - 1] * longitud_df
 
    rng = pd.date_range(df.index.min(), df.index.max(), freq=cadencia_serie)
    df_gps = df.reindex(rng, copy=False)
    nodatos = len(df_gps.index)
 
    pgaps_vars = np.zeros(nvars)
    for j in range(nvars):
        nans = df_gps.iloc[:, j].isnull()
        pgaps = nans.sum() / nodatos
        pgaps_vars[j] = pgaps
 
    pgaps = max(pgaps_vars)  # Mayor porcentaje (sobre 1) de huecos
    lontiempo_gaps_serie = pgaps * longitud_df.total_seconds()  # Intervalo de tiempo de gaps
 
    if lontiempo_gaps_serie <= longitud_gaps_max.total_seconds():
        str_1 = 'La serie cumple las recomendaciones de calidad de huecos según la fase del proyecto'
        print(str_1)
        informe.write(str_1 + '\n')
    else:
        str_1 = 'La serie no cumple las recomendaciones de calidad de huecos según la fase del proyecto'
        print(str_1)
        informe.write(str_1 + '\n')
 
 
def find_gap(serie, col):
    """ just treat it as a list
    """
    nullindex = np.where(serie.iloc[:, col].isnull())[0]
    # nullindex = np.where(s.isnull())[0]
    ranges = []
    for k, g in groupby(enumerate(nullindex), lambda (i, x): i - x):
        group = map(itemgetter(1), g)
        ranges.append((group[0], group[-1]))
    startgap, endgap = zip(*ranges)
    return pd.Series(endgap, index=startgap)
 
 
def peralte(df_in, h=-99.0):
 
    df = df_in.copy(deep=True)
    hs = df.hs.values
    tm = df.tm.values
    per = list()
 
    for hsi, tmi in zip(hs,tm):
        if h == -99.0:
            li = (9.81*(tmi**2)) / (2*np.pi)
            peri = hsi / li
        else:
            ki = wavnum(tmi, h)
            li = 2*np.pi / ki
            peri = hsi / li
 
        per.append(peri)
 
    df['per'] = per
 
    return df
 
 
def wavnum(t, h):
    g = 9.81
    gamma = (2 * np.pi / t) ** 2. * h / g
    sigma2 = (2 * np.pi / t) ** 2
 
    def func(k):
        return sigma2 / g - k * np.tanh(k * h)
 
    if gamma > np.pi ** 2.:
        ko = np.sqrt(gamma / h)
        k = fsolve(func, ko)
    else:
        ko = gamma / h
        k = fsolve(func, ko)
 
        k = abs(k)
 
    return k[0]
 
 
def cadencia_df(df):
    """Función que devuelve las cadencias y la moda de estas
 
        Args:
            df: serie temporal
 
        Returns:
            cad: moda
 
        """
    cad = df.index.to_series() - df.index.to_series().shift()
    cad_m = cad.mode().iloc[0].total_seconds()
 
    return cad, cad_m
