# -*- coding: utf-8 -*-
"""
Created on Mon Jun 20 11:58:52 2016
"""
import numpy as np
import logging
import utils
from scipy.optimize import minimize
logging.basicConfig(filename='debug_info.log', level=logging.INFO)


def cdf(n, data, par, mod, f_pro):
    """Probabilidad acumulada de la distribucion

    Args:
        n: Fechas en escala anual [0 1]
        data: Valores de la variable
        par: Parametros del modelo:

            * par[0:mod[0]*2+1]: terminos independiente y de Fourier del parametro de forma
            * par[mod[0]*2+1:np.sum(mod[0:2])*2+2]: terminos independiente y de Fourier del parametro de localizacion
            * par[np.sum(mod[0:2])*2+2:np.sum(mod)*2+3]: terminos independiente y de Fourier del parametro de escala

        mod: Orden de aproximacion del modelo para los tres parametros
        f_pro: Funcion de probabilidad simple

    Returns:
        Una serie con la probabilidad de no excedencia

    """

    cossen = utils.cossen_n(mod, n)

    if np.max(mod) == 0:
        s, l = par[0:2]
        if len(mod) > 2:
            e = par[2]
    else:
        s = par[0] + np.dot(par[1:mod[0]*2+1], cossen[0:mod[0]*2, :])
        l = par[mod[0]*2+1] + np.dot(par[mod[0]*2+2:np.sum(mod[0:2])*2+2], cossen[0:mod[1]*2, :])
        if len(mod) > 2:
            e = par[np.sum(mod[0:2])*2+2] + np.dot(par[np.sum(mod[0:2])*2+3:np.sum(mod)*2+3], cossen[0:mod[2]*2, :])

    if len(mod) == 2:
        p = f_pro.cdf(data, s, l)
    else:
        p = f_pro.cdf(data, s, l, e)

    return p


def inv(n, p, par, mod, f_pro):
    """Inversa de la funcion de probabilidad

    Args:
       n: Fechas en escala anual [0 1]
       p: Probabilidad para la que se desea recuperar el valor de la variable
       par: Parametros del modelo:

            * par[0:mod[0]*2+1]: terminos independiente y de Fourier del parametro de forma
            * par[mod[0]*2+1:np.sum(mod[0:2])*2+2]: terminos independiente y de Fourier del parametro de localizacion
            * par[np.sum(mod[0:2])*2+2:np.sum(mod)*2+3]: terminos independiente y de Fourier del parametro de escala

       mod: Orden de aproximacion del modelo para los tres parametros
       f_pro: Funcion de probabilidad simple

    Returns:
        Una serie con la variable

    """

    cossen = utils.cossen_n(mod, n)
    
    if np.max(mod) == 0:
        s, l = par[0:2]
        if len(mod) > 2:
            e = par[2]
    else:
        s = par[0] + np.dot(par[1:mod[0]*2+1], cossen[0:mod[0]*2, :])
        l = par[mod[0]*2+1] + np.dot(par[mod[0]*2+2:np.sum(mod[0:2])*2+2], cossen[0:mod[1]*2, :])
        if len(mod) > 2:
            e = par[np.sum(mod[0:2])*2+2] + np.dot(par[np.sum(mod[0:2])*2+3:np.sum(mod)*2+3], cossen[0:mod[2]*2, :])

    if len(mod) == 2:
        dat = f_pro.ppf(p, s, l)
    else:
        dat = f_pro.ppf(p, s, l, e)

    return dat


def nllf(par, data, cossen, mod, f_pro):
    """Negative Log Likelihood Function

    Args:
        par: parametros del modelo:

            * par[0:mod[0]*2+1]: terminos independiente y de Fourier del parametro de forma
            * par[mod[0]*2+1:np.sum(mod[0:2])*2+2]: terminos independiente y de Fourier del parametro de localizacion
            * par[np.sum(mod[0:2])*2+2:np.sum(mod)*2+3]: terminos independiente y de Fourier del parametro de escala

        data: Valores de la variable
        cossen: Variable auxiliar para calcular los terminos de Fourier
        mod: Orden de aproximacion que se quiere utilizar en cada parametro. mod tiene que tener 3 elementos
        f_pro: Funcion de probabilidad simple

    Returns:
        El valor de la Negative Log Likelihood Function

    """

    s = par[0] + np.dot(par[1:mod[0]*2+1], cossen[0:mod[0]*2, :])
    l = par[mod[0]*2+1] + np.dot(par[mod[0]*2+2:np.sum(mod[0:2])*2+2], cossen[0:mod[1]*2, :])
    if len(mod) > 2:
        e = par[np.sum(mod[0:2])*2+2] + np.dot(par[np.sum(mod[0:2])*2+3:np.sum(mod)*2+3], cossen[0:mod[2]*2, :])

    if len(mod) == 2:
        pdf = f_pro.pdf(data, s, l)
    else:
        pdf = f_pro.pdf(data, s, l, e)

    nllf = -np.sum(np.log(pdf))

    return nllf


def fit(par0, data, f_pro, n, mod):
    """ Funcion que ajusta los datos a una funcion de probabilidad simple

    Args:
        - par0: Parametros del modelo:

            * par[0:mod[0]*2+1]: terminos independiente y de Fourier del parametro de forma
            * par[mod[0]*2+1:np.sum(mod[0:2])*2+2]: terminos independiente y de Fourier del parametro de localizacion
            * par[np.sum(mod[0:2])*2+2:np.sum(mod)*2+3]: terminos independiente y de Fourier del parametro de escala

        - data: Valores de la variable
        - f_pro: Funcion de probabilidad a ajustar
        - n: Fecha en escala anual [0 1]
        - mod: Orden de aproximacion que se quiere utilizar en cada parametro.

    Returns:
        Una tupla con: par y llf

    """
#    comptot = np.sum(mod)*2 + 3
#    if len(par0) != comptot:
#        logging.info('par0 no tiene la dimension adecuada')

    cossen = utils.cossen_n(mod, n)

    parBREAK = par0
    err, it = 1, 1
    bnds = [[] for i in range(len(par0))]
    for i in range(0, len(par0)):
        bnds[i] = -10, 10
    while err:
        res = minimize(nllf, par0, args=(data, cossen, mod, f_pro), method='SLSQP', tol=1e-6,
                       options={'maxiter': 1e2, 'disp': True})
        if ((res['fun'] != 0) & (res['fun'] < 1e8) & (res['x'] != par0).any()):
            err = 0
        elif ((it > 2e1) & (np.max(mod) != 1)):
            err = 0
            logging.info(res)
            res['x'], res['fun'] = parBREAK, np.nan
        else:
            it += 1
            logging.info('Solucion con parametros no operacionales')
            logging.info('Minimizando de nuevo con diferentes parametros de entrada')
            logging.info('Iteracion ' + str(it))
            par0 = parBREAK*np.random.rand(len(parBREAK))

    return res['x'], res['fun']


def n_est(ord_, data, n, par0, funp):
    """ Funcion que ajusta los datos a la funcion de probabilidad no estacionaria dada

    Args:
        - ord_: Orde maximo en el desarrollo en serie de Fourier
        - data: variable a analizar
        - n: Fecha en formato [0 - 1] por anio
        - par0: Parametros del modelo:

            * par[0:mod[0]*2+1]: terminos independiente y de Fourier del parametro de forma
            * par[mod[0]*2+1:np.sum(mod[0:2])*2+2]: terminos independiente y de Fourier del parametro de localizacion
            * par[np.sum(mod[0:2])*2+2:np.sum(mod)*2+3]: terminos independiente y de Fourier del parametro de escala

        - funp: Funcion de probabilidad del modelo

    Returns:
        Los parametros del ajuste y el bic

    """

    logndat = np.log(len(data))
    if len(par0) == 2:
        par = [[[] for i in xrange(ord_)] for i in xrange(ord_)]
        llf, bic = np.zeros([ord_, ord_]), np.zeros([ord_, ord_])
        par0 = np.hstack((par0[0], 0, 0, par0[1], 0, 0))
        mod = [1, 1]
        par[0][0], llf[0, 0] = fit(par0, data, funp, n, mod)
        bic[0, 0] = -2*llf[0, 0]+logndat*(3*2)
        nk = ord_**2
        modos = np.zeros(nk)+100
        modos[ord_:nk:ord_] = 10
    else:
        par = [[[[] for i in xrange(ord_)] for i in xrange(ord_)] for i in xrange(ord_)]
        llf, bic = np.zeros([ord_, ord_, ord_]), np.zeros([ord_, ord_, ord_])
        par0 = np.hstack((par0[0], 0, 0, par0[1], 0, 0, par0[2], 0, 0))
        mod = [1, 1, 1]
        par[0][0][0], llf[0, 0, 0] = fit(par0, data, funp, n, mod)
        bic[0, 0, 0] = -2*llf[0, 0, 0]+logndat*(3*2 + 3)
        nk = ord_**3
        modos = np.zeros(nk)+1000
        modos[ord_:nk:ord_] = 100
        modos[ord_**2:nk:ord_**2] = 10

    cont = -1
    if len(mod) == 2:
        for j in range(1, ord_+1):
            for i in range(1, ord_+1):
                cont += 1
                if i+j == 2:
                    continue

                elif modos[cont] == 100:
                    logging.info(str(i) + str(j) + ':I')
                    par0 = par[i-2][j-1]
                    par0 = np.concatenate((par0[0:(i-1)*2+1], [0.], [0.], par0[(i-1)*2+1:]))
                    (par[i-1][j-1], llf[i-1, j-1]) = fit(par0, data, funp, n, [i, j])
                    bic[i-1, j-1] = -2*llf[i-1, j-1] + logndat*(2*(i+j)+3)

                elif modos[cont] == 10:
                    logging.info(str(i) + str(j) + ':J')
                    par0 = par[i-1][j-2]
                    par0 = np.concatenate((par0[0:2*i+2*(j-1)+2], [0.], [0.], par0[2*i+2*(j-1)+2:]))
                    par[i-1][j-1], llf[i-1, j-1] = fit(par0, data, funp, n, [i, j])
                    bic[i-1, j-1] = -2*llf[i-1, j-1] + logndat*(2*(i+j)+3)

    else:
        for k in range(1, ord_+1):
            for j in range(1, ord_+1):
                for i in range(1, ord_+1):
                    cont += 1
                    if i+j+k == 3:
                        continue

                    elif modos[cont] == 1000:
                        logging.info(str(i) + str(j) + str(k) + ':I')
                        par0 = par[i-2][j-1][k-1]
                        par0 = np.concatenate((par0[0:(i-1)*2+1], [0.], [0.], par0[(i-1)*2+1:]))
                        (par[i-1][j-1][k-1], llf[i-1, j-1, k-1]) = fit(par0, data, funp, n, [i, j, k])
                        bic[i-1, j-1, k-1] = -2*llf[i-1, j-1, k-1] + logndat*(2*(i+j+k)+3)

                    elif modos[cont] == 100:
                        logging.info(str(i) + str(j) + str(k) + ':J')
                        par0 = par[i-1][j-2][k-1]
                        par0 = np.concatenate((par0[0:2*i+2*(j-1)+2], [0.], [0.], par0[2*i+2*(j-1)+2:]))
                        par[i-1][j-1][k-1], llf[i-1, j-1, k-1] = fit(par0, data, funp, n, [i, j, k])
                        bic[i-1, j-1, k-1] = -2*llf[i-1, j-1, k-1] + logndat*(2*(i+j+k)+3)

                    elif modos[cont] == 10:
                        logging.info(str(i) + str(j) + str(k) + ':K')
                        par0 = par[i-1][j-1][k-2]
                        par0 = np.concatenate((par0, [0.], [0.]))
                        par[i-1][j-1][k-1], llf[i-1, j-1, k-1] = fit(par0, data, funp, n, [i, j, k])
                        bic[i-1, j-1, k-1] = -2*llf[i-1, j-1, k-1] + logndat*(2*(i+j+k)+3)

    return par, bic
