#! /usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import division
from scipy.stats import kendalltau, pearsonr, spearmanr
import numpy as np
from scipy.integrate import quad
from scipy.optimize import fmin
import sys
from sklearn.neighbors import KernelDensity
from scipy.interpolate import interp1d
from scipy.stats import scoreatpercentile


class Copula():
    """
    Esta clase estima el parametro de la copula generada para una variable aleatoria conjunta por los parametros.
    Esta clase contiene las copulas de Clayton, Frank y Gumbel.
    'El archivo base pertenece al modulo ambhas 0.4.0 de Sat Kumar Tomer, http://civil.iisc.ernet.in/~satkumar/,
    satkumartomer@gmail.com'

    Las modificaciones son:
        1. Incluidas las copulas condicionales de las familias clayton, frank y gumbel
        2. Se utiliza el paquete sklearn.neighbors.KernelDensity para generar el interpolador en vez del paquete
        no tan comun 'statistics'.

    """

    def __init__(self, X, Y, family, *args):
        """ Se inicializa la clase con las variables X e Y
        Args:
            - X: vector de una dimension
            - Y: vector de una dimension
            - familia: clayton, frank o gumbel

            Nota: el tamaño de X e Y debe ser el mismo
        """
        # se chequean las dimensiones
        if not ((X.ndim == 1) and (Y.ndim == 1)):
            raise ValueError('La dimension del vector debe ser uno.')

        # se chequean las longitudes de ambos vectores
        if X.size != Y.size:
            raise ValueError('El tamaño de ambos vectores debe ser el mismo.')

        # se chequean que la familia es correcta
        copula_family = ['clayton', 'frank', 'gumbel']
        if family not in copula_family:
            raise ValueError('La familia debe ser clayton, frank o gumbel.')

        self.X = X
        self.Y = Y
        self.family = family

        # si se tienen marginales teoricas
        if args:
            self.F1, self.F2 = args[0][0], args[0][2]
            self.p1, self.p2 = args[0][1], args[0][3]

        # se estiman el coeficiente de correlación de Kendall
        tau = kendalltau(self.X, self.Y)[0]
        self.tau = tau

        # se estima el R de pearson y R de spearman
        self.pr = pearsonr(self.X, self.Y)[0]
        self.sr = spearmanr(self.X, self.Y)[0]

        # se estima el parametro de la copula
        self._get_parameter()

        # se establece U y V a None
        self.U = None
        self.V = None

    def _get_parameter(self):
        """ Se estima el parametro (theta) de la copula
        """

        if self.family == 'clayton':
            self.theta = 2*self.tau/(1-self.tau)

        elif self.family == 'frank':
            self.theta = -fmin(self._frank_fun, -5, disp=False)[0]

        elif self.family == 'gumbel':
            self.theta = 1/(1-self.tau)

    def generate_uv(self, n=1000):
        """ Se generan la variables aleatorias (u, v)

        Args:
            n: nuimero de elementos de la copula

        Returns:
            U y V de la copula generada

        """
        # copula de clayton
        if self.family == 'clayton':
            U = np.random.uniform(size=n)
            W = np.random.uniform(size=n)

            if self.theta <= -1:
                raise ValueError('the parameter for clayton copula should be more than -1')
            elif self.theta == 0:
                raise ValueError('The parameter for clayton copula should not be 0')

            if self.theta < sys.float_info.epsilon:
                V = W
            else:
                V = U*(W**(-self.theta/(1 + self.theta)) - 1 + U**self.theta)**(-1/self.theta)

        # copula de frank
        elif self.family == 'frank':
            U = np.random.uniform(size=n)
            W = np.random.uniform(size=n)

            if self.theta == 0:
                raise ValueError('The parameter for frank copula should not be 0')
            if abs(self.theta) > np.log(sys.float_info.max):
                V = (U < 0) + np.sign(self.theta)*U
            elif abs(self.theta) > np.sqrt(sys.float_info.epsilon):
                V = -np.log((np.exp(-self.theta*U)*(1-W)/W + np.exp(-self.theta))/(
                        1 + np.exp(-self.theta*U)*(1-W)/W))/self.theta
            else:
                V = W

        # coupla de gumbel
        elif self.family == 'gumbel':
            if self.theta <= 1:
                raise ValueError('El parametro de la copula de gumbel debe ser mayor que uno.')
            if self.theta < 1 + sys.float_info.epsilon:
                U = np.random.uniform(size=n)
                V = np.random.uniform(size=n)
            else:
                u = np.random.uniform(size=n)
                w = np.random.uniform(size=n)
                w1 = np.random.uniform(size=n)
                w2 = np.random.uniform(size=n)

                u = (u - 0.5) * np.pi
                u2 = u + np.pi/2
                e = -np.log(w)
                t = np.cos(u - u2/self.theta)/e
                gamma = (np.sin(u2/self.theta)/t)**(1/self.theta)*t/np.cos(u)
                s1 = (-np.log(w1))**(1/self.theta)/gamma
                s2 = (-np.log(w2))**(1/self.theta)/gamma
                U = np.array(np.exp(-s1))
                V = np.array(np.exp(-s2))

        self.U = U
        self.V = V
        return U, V

    def generate_cond(self):
        """ Genera valores de V dado U a partir de la copula condicional

        Args:

        Return:
            V de la coupla

        """

        if isinstance(self.U, float):
            n = 1
        else:
            n = len(self.U)

        W = np.random.uniform(size=n)

        # copula de clayton
        if self.family == 'clayton':
            if self.theta <= -1:
                raise ValueError('the parameter for clayton copula should be more than -1')
            elif self.theta == 0:
                raise ValueError('The parameter for clayton copula should not be 0')

            if self.theta < sys.float_info.epsilon:
                V = W
            else:
                V = (self.U**-self.theta*(self.U**-self.theta+W**-self.theta-1)**(-1/self.theta))/(self.U*(
                        self.U**-self.theta+W**-self.theta-1))

        # copula de frank
        elif self.family == 'frank':
            if self.theta == 0:
                raise ValueError('The parameter for frank copula should not be 0')

            if abs(self.theta) > np.log(sys.float_info.max):
                V = (self.U < 0) + np.sign(self.theta)*self.U
            elif abs(self.theta) > np.sqrt(sys.float_info.epsilon):
                tU, tW = (self.theta+0j)**self.U, (self.theta+0j)**W
                V = -tU*(tW-1)/((1+(tU-1)*(tW-1)/(self.theta-1))*(self.theta-1))
            else:
                V = W

        # copula de gumbel
        elif self.family == 'gumbel':
            if self.theta <= 1:
                raise ValueError('El parametro de la copula gumbel debe ser mayor que 1.')
            if self.theta < 1 + sys.float_info.epsilon:
                V = np.random.uniform(size=len(self.U))
            else:
                V = -np.log(self.U)**self.theta*((-np.log(self.U))**self.theta+(-np.log(W))**self.theta)**(
                        1/self.theta)*(np.exp(-((-np.log(self.U))**self.theta+(-np.log(W))*self.theta)**(
                                1/self.theta)))/(self.U*np.log(self.U)*((-np.log(self.U))**self.theta+(
                                        -np.log(W))**self.theta))

        self.V = np.absolute(V)
        return np.absolute(V)

    def generate_xy(self, n=0):
        """ Genera un vector aleatorio de variables x e y, y longitud n para el caso en el que U no este dado y
        un vector de variables V condicionados al valor de U en el caso de que U se conozca.

        Args:
            n: Numero de elementos a generar

        Return:
            X1 and Y1

        """

        # estimate inverse cdf of x and y
        self._inverse_cdf()

        # if U and V are not already generated
        if self.U is None:
            self.generate_uv(n)
        elif n != 0:
            self.generate_cond()

        if hasattr(self, 'p1'):
            X1 = self.F1.ppf(self.U, *self.p1)
            Y1 = self.F2.ppf(self.V, *self.p2)
        else:
            X1 = self._inv_cdf_x(self.U)
            Y1 = self._inv_cdf_y(self.V)
        self.X1 = X1
        self.Y1 = Y1

        # se establece U y V a None
        self.U = None
        self.V = None

        return X1, Y1

    def generate_C(self, u, v):
        """ Genera un vector aleatorio de variables x e y, y longitud n para el caso en el que U no este dado y
        un vector de variables V condicionados al valor de U en el caso de que U se conozca.

        Args:
            n: Numero de elementos a generar

        Return:
            X1 and Y1

        """
        uq, vq = np.meshgrid(u, v)

        if self.family == 'clayton':
            self.C = np.zeros(np.shape(uq))
            mask = (uq**-self.theta+vq**-self.theta-1)**-(1/self.theta) > 0
            self.C[mask] = (uq[mask]**-self.theta+vq[mask]**-self.theta-1)**-(1/self.theta)
        elif self.family == 'frank':
            self.C = -1/np.log(self.theta)*np.log(1+(self.theta**uq+1)*(self.theta**vq-1)(self.theta-1))
        elif self.family == 'gumbel':
            self.C = np.exp(-((-np.log(uq)**self.theta)+(-np.log(vq)**self.theta))**(1/self.theta))

        return self.C

    def estimate(self, data=None):
        """ Estima mean, std e iqr para el
        this function estimates the mean, std, iqr for the generated
        ensemble

        Output:
            Y1_mean = mean of the simulated ensemble
            Y1_std = std of the simulated ensemble
            Y1_ll = lower limit of the simulated ensemble
            Y1_ul = upper limit of the simulated ensemble
        """
        nbin = 50
        # se chequea si ya se ha llamado a generate_xy, si no, se hace ahora
        try:
            self.X1
            copula_ens = len(self.X1)
        except:
            copula_ens = 10000
            self.generate_xy(copula_ens)

        if data is None:
            data = self.X

        n_ens = copula_ens/nbin  # numero promedio de bins en cada clase
        ind_sort = self.X1.argsort()
        x_mean = np.zeros((nbin,))
        y_mean = np.zeros((nbin,))
        y_ul = np.zeros((nbin,))
        y_ll = np.zeros((nbin,))
        y_std = np.zeros((nbin,))

        for ii in range(nbin):
            x_mean[ii] = self.X1[ind_sort[n_ens*ii:n_ens*(ii+1)]].mean()
            y_mean[ii] = self.Y1[ind_sort[n_ens*ii:n_ens*(ii+1)]].mean()
            y_std[ii] = self.Y1[ind_sort[n_ens*ii:n_ens*(ii+1)]].std()
            y_ll[ii] = scoreatpercentile(self.Y1[ind_sort[n_ens*ii:n_ens*(ii+1)]], 25)
            y_ul[ii] = scoreatpercentile(self.Y1[ind_sort[n_ens*ii:n_ens*(ii+1)]], 75)

        foo_mean = interp1d(x_mean, y_mean, bounds_error=False)
        foo_std = interp1d(x_mean, y_std, bounds_error=False)
        foo_ll = interp1d(x_mean, y_ll, bounds_error=False)
        foo_ul = interp1d(x_mean, y_ul, bounds_error=False)

        Y1_mean = foo_mean(data)
        Y1_std = foo_std(data)
        Y1_ll = foo_ll(data)
        Y1_ul = foo_ul(data)

        return Y1_mean, Y1_std, Y1_ll, Y1_ul

    def _inverse_cdf(self):
        """ Este modulo calcula la inversa de la cdf obtenida a partir de la conjunta de X e Y para U y V
        Para estimar la inversa se utilizado el kernel de la familia epanechikov y para datos fuera de los datos de
        partida se extrapola con la función lineal. Este modulo devuelve el interpolador.
        """

        x1 = np.linspace(self.X.min(), self.X.max(), 1000)
        x2 = np.cumsum(np.exp(KernelDensity(kernel='epanechnikov').fit(self.X[:, np.newaxis]).score_samples(
                x1[:, np.newaxis])))
        x2 = x2/np.max(x2)

        self._inv_cdf_x = interp1d(x2, x1, bounds_error=False, fill_value='extrapolate', kind='nearest')

        y1 = np.linspace(self.Y.min(), self.Y.max(), 1000)
        y2 = np.cumsum(np.exp(KernelDensity(kernel='epanechnikov').fit(self.Y[:, np.newaxis]).score_samples(
                y1[:, np.newaxis])))
        y2 = y2/np.max(y2)
        self._inv_cdf_y = interp1d(y2, y1, bounds_error=False, fill_value='extrapolate', kind='nearest')

    def _integrand_debye(self, t):
        """  Integrados de primer orden de la funcion de debye
        """
        return t/(np.exp(t)-1)

    def _debye(self, alpha):
        """ Deby funcion de primer orden
        """
        return quad(self._integrand_debye, sys.float_info.epsilon, alpha)[0]/alpha

    def _frank_fun(self, alpha):
        """ Optimizacion de la funcion para obtener el parametro de la copula de frank
        """
        diff = (1-self.tau)/4.0 - (self._debye(-alpha)-1)/alpha
        return diff**2
