# -*- coding: utf-8 -*-
from __future__ import division
import numpy as np
from scipy.stats import lognorm
import utils
from scipy.optimize import minimize
import logging
import pandas as pd
from datetime import datetime
logging.basicConfig(filename='debug_info.log', level=logging.INFO)


def prepara_param(data):
    """ Funcion que calcula los parametros iniciales de busqueda en la minimizacion

    Args:
        - data: Valores de la variable

    Returns:
        Devuelve los parametros inciales
    """
    xlim = np.median(data)
    x1, x2 = np.log(data[data <= xlim]), np.log(data[data > xlim])
    alpha = float(len(x1))/len(data)
    mu1, mu2, sig1, sig2 = np.mean(x1), np.mean(x2), np.std(x1), np.std(x2)
    par0 = np.array([alpha, mu1, sig1, mu2, sig2])
    return par0


def cdf(n, data, par, mod):
    """Probabilidad acumulada de la distribucion compuesta por 2 Log-Normales

    Args:
        n: Fechas en escala anual [0 1]
        data: Valores de la variable
        par: Parametros del modelo:

            * par[0:mod[0]*2+1]: terminos independientes y de Fourier de esc1
            * par[mod[0]*2+1:np.sum(mod[0:2])*2+2]: terminos independientes y de Fourier de mu1
            * par[np.sum(mod[0:2])*2+2:(np.sum(mod[0:2])+mod[1])*2+3]: terminos independientes y de Fourier de sig1
            * par[(np.sum(mod[0:2])+mod[1])*2+3:(np.sum(mod[0:2])+ np.sum(mod[1:3]))*2+4]: terminos independientes y
            de Fourier de mu2
            * par[(np.sum(mod[0:2])+np.sum(mod[1:3]))*2+4:]: terminos independientes y de Fourier de sig2

        mod: Orden de aproximacion del modelo para esc1, mu1, sig1, mu2 y sig2; se asume que mu1 y sig1 tienen el mismo
        orden de aproximacion, asi como mu2 y sig2

    Returns:
        Un vector con la probabilidad de no excedencia

    """

    cossen = utils.cossen_n(mod, n)

    esc1 = par[0] + np.dot(par[1:mod[0]*2+1], cossen[0:mod[0]*2, :])
    mu1 = par[mod[0]*2+1] + np.dot(par[mod[0]*2+2:np.sum(mod[0:2])*2+2], cossen[0:mod[1]*2, :])
    sig1 = (par[np.sum(mod[0:2])*2+2] + np.dot(par[np.sum(mod[0:2])*2+3:(np.sum(
        mod[0:2])+mod[1])*2+3], cossen[0:mod[1]*2, :]))
    mu2 = (par[(np.sum(mod[0:2])+mod[1])*2+3] + np.dot(par[(np.sum(mod[0:2])+mod[1])*2+4:(np.sum(
        mod[0:2]) + np.sum(mod[1:3]))*2+4], cossen[0:mod[2]*2, :]))
    sig2 = (par[(np.sum(mod[0:2])+np.sum(mod[1:3]))*2+4] + np.dot(par[(np.sum(mod[0:2])+np.sum(
        mod[1:3]))*2+5:], cossen[0:mod[2]*2, :]))

    prob = (esc1*lognorm.cdf(data, sig1, scale=np.exp(mu1)) + (1-esc1)*lognorm.cdf(data, sig2, scale=np.exp(mu2)))

    return prob


def inv(n, prob, par, mod):
    """Inversa de la funcion de distribucion mixta compuesta por 2 Log-Normales

    Args:
        n: Fechas en escala anual [0 1]
        prob: Probabilidad de no excedencia
        par: Parametros del modelo:

            * par[0:mod[0]*2+1]: terminos independiente y de Fourier de esc1
            * par[mod[0]*2+1:np.sum(mod[0:2])*2+2]: terminos independiente y de Fourier de mu1
            * par[np.sum(mod[0:2])*2+2:(np.sum(mod[0:2])+mod[1])*2+3]: terminos independiente y de Fourier de sig1
            * par[(np.sum(mod[0:2])+mod[1])*2+3:(np.sum(mod[0:2])+ np.sum(mod[1:3]))*2+4]: terminos independiente y de
                Fourier de mu2
            * par[(np.sum(mod[0:2])+np.sum(mod[1:3]))*2+4:]: terminos independientes y de Fourier de sig2

        mod: Orden de aproximacion del modelo para esc, mu1, sig1, mu2 y sig2; se asume que mu1 y sig1 tienen el mismo
            orden de aproximacion, asi como mu2 y sig2

    Returns:
        Un vector con los valores de la variable

    """

    n = n - np.floor(n)
    dt, dtt = 0.005, 0.1
    t, tt = np.arange(dt/2, 1-dt/2, dt), np.arange(dtt, 35, dtt)
    nt, ntt = len(t), len(tt)

    cossen = utils.cossen_n(mod, t)

    esc = par[0] + np.dot(par[1:mod[0]*2+1], cossen[0:mod[0]*2, :])
    mu1 = par[mod[0]*2+1] + np.dot(par[mod[0]*2+2:np.sum(mod[0:2])*2+2], cossen[0:mod[1]*2, :])
    sig1 = (par[np.sum(mod[0:2])*2+2] + np.dot(par[np.sum(mod[0:2])*2+3:(np.sum(
        mod[0:2])+mod[1])*2+3], cossen[0:mod[1]*2, :]))
    mu2 = (par[(np.sum(mod[0:2])+mod[1])*2+3] + np.dot(par[(np.sum(mod[0:2])+mod[1])*2+4:(np.sum(mod[0:2]) + np.sum(
        mod[1:3]))*2+4], cossen[0:mod[2]*2, :]))
    sig2 = (par[(np.sum(mod[0:2])+np.sum(mod[1:3]))*2+4] + np.dot(par[(np.sum(mod[0:2])+np.sum(
        mod[1:3]))*2+5:], cossen[0:mod[2]*2, :]))

    ppasa = np.zeros([nt, ntt])
    for j in range(0, nt):
        ppasa[j, :] = (esc[j]*lognorm.cdf(tt, sig1[j], scale=np.exp(mu1[j])) + (1 - esc[j])*lognorm.cdf(
            tt, sig2[j], scale=np.exp(mu2[j])))

    nprob = len(prob)
    x = np.zeros(nprob)
    for i in range(0, nprob):
        err = np.abs(n[i]-t)
        j = np.where(err == np.min(err))[0]
        err = np.abs(prob[i]-ppasa[j[0], :])
        posi = np.where(err == np.min(err))[0]
        x[i] = tt[posi[0]]

    return x


def nllf(par, data, cossen, mod):
    """Negative Log Likelihood Function

    Args:
        par: Parametros de la distribucion:

            * par[0:mod[0]*2+1]: terminos independiente y de Fourier de esc1
            * par[mod[0]*2+1:np.sum(mod[0:2])*2+2]: terminos independiente y de Fourier de mu1
            * par[np.sum(mod[0:2])*2+2:(np.sum(mod[0:2])+mod[1])*2+3]: terminos independiente y de Fourier de sig1
            * par[(np.sum(mod[0:2])+mod[1])*2+3:(np.sum(mod[0:2])+ np.sum(mod[1:3]))*2+4]: terminos independiente y de
                Fourier de mu2
            * par[(np.sum(mod[0:2])+np.sum(mod[1:3]))*2+4:]: terminos independientes y de Fourier de sig2

        data: Valores de la variable
        cossen: Variable auxiliar para calcular terminos de Fourier
        mod: Orden de aproximacion que se quiere utilizar en cada parametro. mod tiene que tener 3 elementos

    Returns:
        El valor de la Negative Log Likelihood Function

    """

    df = pd.DataFrame(index=data.index)
    df['esc1'] = par[0] + np.dot(par[1:mod[0]*2+1], cossen[0:mod[0]*2, :])
    df['mu1'] = par[mod[0]*2+1] + np.dot(par[mod[0]*2+2:np.sum(mod[0:2])*2+2], cossen[0:mod[1]*2, :])
    df['sig1'] = par[np.sum(mod[0:2])*2+2] + np.dot(par[np.sum(mod[0:2])*2+3:(np.sum(mod[0:2])+mod[
        1])*2+3], cossen[0:mod[1]*2, :])
    df['mu2'] = par[(np.sum(mod[0:2])+mod[1])*2+3] + np.dot(par[(np.sum(mod[0:2])+mod[1])*2+4:(
        np.sum(mod[0:2]) + np.sum(mod[1:3]))*2+4], cossen[0:mod[2]*2, :])
    df['sig2'] = par[(np.sum(mod[0:2])+np.sum(mod[1:3]))*2+4] + np.dot(par[(np.sum(mod[0:2])+np.sum(
        mod[1:3]))*2+5:], cossen[0:mod[2]*2, :])

    if ((np.min(df['esc1']) <= 0) | (np.max(df['esc1']) >= 1) | (np.min(df['sig1']) <= 0) | (
            np.min(df['sig2']) <= 0) | (np.min(df['mu1']) <= 0) | (np.min(df['mu2']) <= 0)):
        nllf = 1e10
    else:
        pdfln1 = lognorm.pdf(data, df['sig1'], scale=np.exp(df['mu1']))
        pdfln2 = lognorm.pdf(data, df['sig2'], scale=np.exp(df['mu2']))
        PDFLN = df['esc1']*pdfln1 + (1.-df['esc1'])*pdfln2

        nllf = -np.sum(np.log(PDFLN))

    return nllf


def fit(par0, data, *args):
    """Ajuste de la distribucion mixta compuesta por 2 Log-Normal

    Args:
        par0: parametros del modelo. Los parametros son:

            * par[0:mod[0]*2+1]: terminos independiente y de Fourier de ESC1
            * par[mod[0]*2+1:np.sum(mod[0:2])*2+2]: terminos independiente y de Fourier de MU1
            * par[np.sum(mod[0:2])*2+2:(np.sum(mod[0:2])+mod[1])*2+3]: terminos independiente y de Fourier de SiG1
            * par[(np.sum(mod[0:2])+mod[1])*2+3:(np.sum(mod[0:2])+np.sum(mod[1:3]))*2+4]: terminos independiente y de
                Fourier de MU2
            * par[(np.sum(mod[0:2])+np.sum(mod[1:3]))*2+5:]: terminos independiente y de Fourier de SiG2

        - data: Valores de la variable
        - args [opt]: argumento opcional que para el ajuste no estacionario que contiene:

            * n: Fechas en escala anual [0 1]
            * mod: mod da el orden de aproximacion que se quiere utilizar en cada parametro. mod tiene tres componentes
                identificando el nivel de aproximacion de los parametros

    Returns:
        Una tupla de la forma (par,llf):

            * ``par[0:mod[0]*2+1]``: terminos independiente y de Fourier de esc1
            * ``par[mod[0]*2+1:np.sum(mod[0:2])*2+2]``: terminos independiente y de Fourier de mu1
            * ``par[np.sum(mod[0:2])*2+2:(np.sum(mod[0:2])+mod[1])*2+3]``: terminos independiente y de Fourier de sig1
            * ``par[(np.sum(mod[0:2])+mod[1])*2+3:(np.sum(mod[0:2])+np.sum(mod[1:3]))*2+4]``: terminos independiente
                y de Fourier de mu2
            * ``par[(np.sum(mod[0:2])+np.sum(mod[1:3]))*2+5:]``: terminos independiente y de Fourier de sig2

       ``llf``: Log Likelihood Function evaluada en los parametros

    """

    if args:
        n, mod = args
        cossen = utils.cossen_n(mod, n)
    else:
        cossen, mod = np.zeros([1, len(data)]), [0, 0, 0]

    comptot = np.sum(mod)+np.sum(mod[1:3])
    if len(par0) != comptot*2+5:
        logging.info('par0 no tiene la dimension adecuada')

    bnds = [[] for i in range(len(par0))]
    for i in range(0, len(par0)):
        bnds[i] = -5, 5

    parBREAk = par0
    err, it = 1, 1
    while err:
        logging.info('Comienzo del ajuste a las: ' + str(datetime.now()))
        res = minimize(nllf, par0, args=(data, cossen, mod), method='SLSQP', bounds=bnds, tol=1e-6, options={
            'maxiter': 1e3, 'disp': True})
        if (res['fun'] != 0) & (res['fun'] < 1e8):
            err = 0
            logging.info(res)
        elif ((it > 2e1) & (np.max(mod) != 1)):
            err = 0
            logging.info(res)
            res['x'], res['fun'] = parBREAk, np.nan
        else:
            it += 1
            logging.info('Solucion con parametros no operacionales')
            logging.info('Minimizando de nuevo con diferentes parametros de entrada')
            logging.info('iteracion ' + str(it))
            par0 = parBREAk*np.random.rand(len(parBREAk))*2
    logging.info('Final del analisis a las: ' + str(datetime.now()))
    return res['x'], res['fun']


def n_est(ord_, data, n, par0=0):
    """Ajusta los datos a una distribucion mixta formada por dos lognormales. Aumenta el orden de la aproximacion de
    Fourier de forma progresiva hasta alcanzar la aproximacion de orden ord_ en los tres parametros que se modelan.

    Args:
        ord_: Orden maximo de los modelos
        data: Dataframe con los valores a ajustar
        n: Fechas en escala anual [0 1]
        par0 [opc]: parametros inciales

    Returns:
        Una tupla de la forma (par, bic)

        * ``par``: Matriz con los parametros del ajuste
        * ``bic``: Criterios de informacion Bayesiana calculado del modelo

    """

    logndat = np.log(len(data))
    par = [[[[] for i in xrange(ord_)] for i in xrange(ord_)] for i in xrange(ord_)]
    llf, bic = np.zeros([ord_, ord_, ord_]), np.zeros([ord_, ord_, ord_])
    if any(par0):
        par0 = np.array([par0[0], 0., 0., par0[1], 0., 0., par0[2], 0., 0., par0[3], 0., 0., par0[4], 0., 0.])
    else:
        xlim = np.median(data)
        x1, x2 = np.log(data[data <= xlim]), np.log(data[data > xlim])
        alfa = float(len(x1))/len(data)
        mu1, mu2, sig1, sig2 = np.mean(x1), np.mean(x2), np.std(x1), np.std(x2)
        par0 = np.array([alfa, mu1, sig1, mu2, sig2])
        par0 = np.array([par0[0], 0., 0., par0[1], 0., 0., par0[2], 0., 0., par0[3], 0., 0., par0[4], 0., 0.])

    par[0][0][0], llf[0, 0, 0] = fit(par0, data, n, [1, 1, 1])
    bic[0, 0, 0] = -2*llf[0, 0, 0]+logndat*((1+2+2)*2+5)
    for ordact in range(1, ord_+1):
        logging.info('ORDACT =' + str(ordact))
        for k in range(1, ordact):
            i = ordact
            for j in range(1, ordact+1):
                logging.info(str(i) + str(j) + str(k) + ':i')
                if i+j+k == 3:
                    continue
                elif (j == 1) & (k == 1) & (not any(par[i-1][j-1][k-1])):
                    par0 = par[i-2][j-1][k-1]
                    NPA = (i-1)*2+1
                    par0 = np.concatenate((par0[0:NPA], [0.], [0.], par0[NPA:]))
                    (par[i-1][j-1][k-1], llf[i-1, j-1, k-1]) = fit(par0, data, n, [i, j, k])
                    bic[i-1, j-1, k-1] = -2*llf[i-1, j-1, k-1] + logndat*((i+2*k+2*j)*2+5)

                elif (j == 1) & (not any(par[i-1][j-1][k-1])):
                    par0 = par[i-1][j-1][k-2]
                    NPA = i*2 + j*4 + (k-1)*2 + 4
                    par0 = np.concatenate((par0[0:NPA], [0.], [0.], par0[NPA:], [0.], [0.]))
                    (par[i-1][j-1][k-1], llf[i-1, j-1, k-1]) = fit(par0, data, n, [i, j, k])
                    bic[i-1, j-1, k-1] = -2*llf[i-1, j-1, k-1]+logndat*((i+2*k+2*j)*2+5)
                elif (not any(par[i-1][j-1][k-1])):
                    par0 = par[i-1][j-2][k-1]
                    NPA1 = i*2 + (j-1)*2 + 2
                    NPA2 = i*2 + (j-1)*4 + 3
                    par0 = np.concatenate((par0[0:NPA1], [0.], [0.], par0[NPA1:NPA2], [0.], [0.], par0[NPA2:]))
                    (par[i-1][j-1][k-1], llf[i-1, j-1, k-1]) = fit(par0, data, n, [i, j, k])
                    bic[i-1, j-1, k-1] = -2*llf[i-1, j-1, k-1]+logndat*((i+2*k+2*j)*2+5)

            for i in range(ordact, 0, -1):
                logging.info(str(i) + str(j) + str(k) + ':j')
                if (not any(par[i-1][j-1][k-1])):
                    par0 = par[i-1][j-2][k-1]
                    NPA1 = i*2 + (j-1)*2 + 2
                    NPA2 = i*2 + (j-1)*4 + 3
                    par0 = np.concatenate((par0[0:NPA1], [0.], [0.], par0[NPA1:NPA2], [0.], [0.], par0[NPA2:]))
                    (par[i-1][j-1][k-1], llf[i-1, j-1, k-1]) = fit(par0, data, n, [i, j, k])
                    bic[i-1, j-1, k-1] = -2*llf[i-1, j-1, k-1]+logndat*((i+2*k+2*j)*2+5)

        k = ordact
        for i in range(1, ordact+1):
            for j in range(1, ordact+1):
                if (k > 1) & (not any(par[i-1][j-1][k-1])):
                    logging.info(str(i) + str(j) + str(k) + ':k')
                    par0 = par[i-1][j-1][k-2]
                    NPA = i*2 + j*4 + (k-1)*2 + 4
                    par0 = np.concatenate((par0[0:NPA], [0.], [0.], par0[NPA:], [0.], [0.]))
                    par[i-1][j-1][k-1], llf[i-1, j-1, k-1] = fit(par0, data, n, [i, j, k])
                    bic[i-1, j-1, k-1] = -2*llf[i-1, j-1, k-1]+logndat*((i+2*k+2*j)*2+5)

    return par, bic
