# -*- coding: utf-8 -*-
from __future__ import division
import numpy as np
from scipy.stats import lognorm
from scipy.stats import norm
from scipy.optimize import minimize
import utils
import logging
import pandas as pd
from datetime import datetime
from statsmodels.distributions import empirical_distribution
logging.basicConfig(filename='debug_info.log', level=logging.INFO)


def prepara_param(data):
    """ Funcion que calcula los parametros iniciales de busqueda en la minimizacion

    Args:
        - data: Valores de la variable

    Returns:
        Devuelve los parametros inciales
    """

    par0 = np.array([np.mean(np.log(data)), np.std(np.log(data)), 0.2, -1., 3.])
    return par0


def cdf(n, data, par, mod):
    """Probabilidad acumulada de la distribucion log-normal-GPD no estacionaria

    Args:
        n: Fechas en escala anual [0 1]
        data: Valores de la variable
        par: parametros del modelo:

            * par[0:mod[0]*2+1]: terminos independiente y de Fourier de mu
            * par[mod[0]*2+1:np.sum(mod[0:2])*2+2]: terminos independiente y de Fourier de sig
            * par[np.sum(mod[0:2])*2+2:np.sum(mod)*2+3]: terminos independiente y de Fourier de xi2
            * par[-2:]: terminos z1 y z2

        mod: orden de aproximacion del modelo para mu, sigma y xi

    Returns:
        Una serie con la probabilidad de no excedencia

    """

    cossen = utils.cossen_n(mod, n)

    z1, z2 = par[-2], par[-1]
    mu = par[0] + np.dot(par[1:mod[0]*2+1], cossen[0:mod[0]*2, :])
    sig = par[mod[0]*2+1] + np.dot(par[mod[0]*2+2:np.sum(mod[0:2])*2+2], cossen[0:mod[1]*2, :])
    xi2 = (par[np.sum(mod[0:2])*2+2] + np.dot(par[np.sum(mod[0:2])*2+3:np.sum(mod)*2+3], cossen[0:mod[2]*2, :]))
    u1 = np.exp(np.sum(np.vstack((z1*sig, mu)), axis=0))
    u2 = np.exp(np.sum(np.vstack((z2*sig, mu)), axis=0))
    esc1, esc2 = norm.cdf(z1), 1-norm.cdf(z2)
    siggp1 = np.exp(-0.5*((np.log(u1)-mu)/sig)**2.)/(u1*np.sqrt(2.*np.pi)*sig)
    siggp1 = esc1/siggp1
    xi1 = -siggp1/u1
    siggp2 = np.exp(-0.5*((np.log(u2)-mu)/sig)**2.)/(u2*np.sqrt(2.*np.pi)*sig)
    siggp2 = esc2/siggp2

    nh = len(data)
    prob = np.zeros(nh)

    for i in range(0, nh):
        if data[i] < u1[i]:
            prob[i] = esc1*(1-xi1[i]/siggp1[i]*(data[i]-u1[i]))**(-1./xi1[i])
        elif data[i] > u2[i]:
            prob[i] = 1-esc2 + esc2*(1-(1+xi2[i]/siggp2[i]*(data[i]-u2[i]))**(-1./xi2[i]))
        else:
            prob[i] = lognorm.cdf(data[i], sig[i], scale=np.exp(mu[i]))

    return prob


def inv(n, prob, par, mod):
    """Inversa de la funcion de distribucion log-normal-GPD no estacionaria

    Args:
        n: Fechas en escala anual [0 1]
        prob: Probabilidad de no excedencia
        par: Parametros del modelo:

            * par[0:mod[0]*2+1]: terminos independiente y de Fourier de mu
            * par[mod[0]*2+1:np.sum(mod[0:2])*2+2]: terminos independiente y de Fourier de sig
            * par[np.sum(mod[0:2])*2+2:np.sum(mod)*2+3]: terminos independiente y de Fourier de xi2
            * par[-2:]: terminos z1 y z2

        mod: orden de aproximacion del modelo para mu, sigma y xi

    Returns:
        Una serie con los valores de la variable

    """

    lsim = len(prob)
    cossen = utils.cossen_n(mod, n)

    mu = par[0] + np.dot(par[1:mod[0]*2+1], cossen[0:mod[0]*2, :])
    sig = par[mod[0]*2+1] + np.dot(par[mod[0]*2+2:np.sum(mod[0:2])*2+2], cossen[0:mod[1]*2, :])
    xi2 = par[np.sum(mod[0:2])*2+2] + np.dot(par[np.sum(mod[0:2])*2+3:np.sum(mod)*2+3], cossen[0:mod[2]*2, :])
    u1 = np.exp(np.sum(np.vstack((par[-2]*sig, mu)), axis=0))
    u2 = np.exp(np.sum(np.vstack((par[-1]*sig, mu)), axis=0))
    esc1, esc2 = norm.cdf(par[-2]), 1-norm.cdf(par[-1])
    siggp1 = np.exp(-0.5*((np.log(u1)-mu)/sig)**2.)/(u1*np.sqrt(2.*np.pi)*sig)
    siggp1 = esc1/siggp1
    xi1 = -siggp1/u1
    siggp2 = np.exp(-0.5*((np.log(u2)-mu)/sig)**2.)/(u2*np.sqrt(2.*np.pi)*sig)
    siggp2 = esc2/siggp2

    data = np.zeros(lsim)

    for i in range(0, lsim):
        if prob[i] < esc1:
            data[i] = u1[i]-siggp1[i]*((prob[i]/esc1)**(-xi1[i])-1)/xi1[i]
        elif prob[i] > (1-esc2):
            data[i] = u2[i]-siggp2[i]*(1-(1-(prob[i]-1+esc2)/esc2)**(-xi2[i]))/xi2[i]
        else:
            data[i] = lognorm.ppf(prob[i], sig[i], scale=np.exp(mu[i]))

    return data


def nllf(par, data, cossen, mod):
    """Negative Log Likelihood Function

    Args:
        par: Parametros de la distribucion:

            * par[0:mod[0]*2+1]: terminos independiente y de Fourier de mu
            * par[mod[0]*2+1:np.sum(mod[0:2])*2+2]: terminos independiente y de Fourier de sig
            * par[np.sum(mod[0:2])*2+2:np.sum(mod)*2+3]: terminos independiente y de Fourier de xi2
            * par[-2:]: terminos z1 y z2

        data: Valores de la variable
        cossen: Variable auxiliar para calcular terminos de Fourier
        mod: Orden de aproximacion que se quiere utilizar en cada parametro [3 elementos]

    Returns:
        El valor de la Negative Log Likelihood Function

    """

    df = pd.DataFrame(index=data.index)
    df['mu'] = par[0] + np.dot(par[1:mod[0]*2+1], cossen[0:mod[0]*2, :])
    df['sig'] = par[mod[0]*2+1] + np.dot(par[mod[0]*2+2:np.sum(mod[0:2])*2+2], cossen[0:mod[1]*2, :])
    df['xi2'] = par[np.sum(mod[0:2])*2+2] + np.dot(par[np.sum(mod[0:2])*2+3:np.sum(mod)*2+3], cossen[0:mod[2]*2, :])
    df['u1'] = np.exp(np.sum(np.array([par[-2]*df['sig'], df['mu']]), axis=0))
    df['u2'] = np.exp(np.sum(np.array([par[-1]*df['sig'], df['mu']]), axis=0))
    esc1, esc2 = norm.cdf(par[-2]), 1-norm.cdf(par[-1])
    df['siggp1'] = esc1/np.exp(-0.5*((np.log(df['u1'])-df['mu'])/df['sig'])**2.)/(df['u1']*np.sqrt(2.*np.pi)*df['sig'])
    df['xi1'] = -df['siggp1']/df['u1']
    df['siggp2'] = esc2/np.exp(-0.5*((np.log(df['u2'])-df['mu'])/df['sig'])**2.)/(df['u2']*np.sqrt(2*np.pi)*df['sig'])

    if ((np.min(df['u1']) <= 0) | (np.min(df['u2']) <= 0) | (np.min(df['sig']) <= 0)):
        nllf = 1e10
    else:
        idln = ((data <= df['u2']) & (data >= df['u1']))
        idgp1 = data < df['u1']
        idgp2 = data > df['u2']
        if (np.any(df['xi1'][idgp1]*(data[idgp1]-df['u1'][idgp1])/df['siggp1'][idgp1] >= 1) or
                np.any(df['xi2'][idgp2]*(data[idgp2]-df['u2'][idgp2])/df['siggp2'][idgp2] <= -1
                       )):
            nllf = 1e10
        else:
            lPDF1 = -np.log(np.sqrt(2*np.pi))
            lPDF2 = -np.log(data[idln]*df['sig'][idln])
            lPDF3 = -(np.log(data[idln])-df['mu'][idln])**2./(2.*df['sig'][idln]**2.)
            lPDFLN = np.sum(lPDF1 + lPDF2 + lPDF3)
            if not any(idgp1):
                lPDFGP1 = 0
            else:
                lPDFGP1_1 = np.log(esc1)-np.log(df['siggp1'][idgp1])
                lPDFGP1_2a = 1+1./df['xi1'][idgp1]
                lPDFGP1_2b = np.log(1-df['xi1'][idgp1]/df['siggp1'][idgp1]*(data[idgp1] - df['u1'][idgp1]))
                lPDFGP1 = np.sum(lPDFGP1_1-lPDFGP1_2a*lPDFGP1_2b)

            if not any(idgp2):
                lPDFGP2 = 0
            else:
                lPDFGP2_1 = np.log(esc2)-np.log(df['siggp2'][idgp2])
                lPDFGP2_2a = 1+1./df['xi2'][idgp2]
                lPDFGP2_2b = np.log(1+df['xi2'][idgp2]/df['siggp2'][idgp2]*(data[idgp2] - df['u2'][idgp2]))
                lPDFGP2 = np.sum(lPDFGP2_1-lPDFGP2_2a*lPDFGP2_2b)

            nllf = -lPDFLN-lPDFGP1-lPDFGP2

    return nllf


def fit(par0, data, *args):
    """Ajuste del modelo Log-Normal-GPD estacionario o no estacionario

    Args:
        - par0: parametros del modelo. Los parametros son:

            * par[0:mod[0]*2+1]: terminos independiente y de Fourier de mu
            * par[mod[0]*2+1:np.sum(mod[0:2])*2+2]: terminos independiente y de Fourier de sig
            * par[np.sum(mod[0:2])*2+2:np.sum(mod)*2+3]: terminos independiente y de Fourier de xi2
            * par[-2:]: terminos z1 y z2

        - data: Valores de la variable
        - args [opt]: argumento opcional que para el ajuste no estacionario que contiene:

            * n: Fechas en escala anual [0 1]
            * mod: mod da el orden de aproximacion que se quiere utilizar en cada parametro. mod tiene tres componentes
                identificando el nivel de aproximacion de los parametros

    Returns:
        Una tupla con los parametros y el valor de llf.

    """

    if args:
        n, mod = args
        cossen = utils.cossen_n(mod, n)
    else:
        cossen, mod = np.zeros([1, len(data)]), [0, 0, 0]

    comptot = np.sum(mod)
    if len(par0) != comptot*2+5:
        logging.info('par0 no tiene la dimension adecuada')

    parBREAk = par0
    err, it = 1, 1
#    bnds = [[] for i in range(len(par0))]
#    for i in range(0, len(par0)):
#        bnds[i] = -8, 8

    while err:
        logging.info('Comienzo del ajuste a las: ' + str(datetime.now()))
        res = minimize(nllf, par0, args=(data, cossen, mod), method='SLSQP', tol=1e-6, options={
            'maxiter': 1e3, 'disp': True})
        if ((res['fun'] != 0) & (res['fun'] < 1e8) & (res['x'] != par0).any()):
            err = 0
            logging.info(res)
        elif ((it > 2e1) & (np.max(mod) != 1)):
            err = 0
            logging.info(res)
            res['x'], res['fun'] = parBREAk, np.nan
        else:
            it += 1
            logging.info('Solucion con parametros no operacionales')
            logging.info('Minimizando de nuevo con diferentes parametros de entrada')
            logging.info('iteracion ' + str(it))
            par0 = parBREAk*np.random.rand(len(parBREAk))*2
    logging.info('Final del analisis a las: ' + str(datetime.now()))
    return res['x'], res['fun']


def n_est(ord_, data, n, par0=0):
    """Ajusta los datos a una distribucion mixta formada por una lognormal en  el centro y dos paretos generalizadas
    aumentando el ord_en de la aproximacion de Fourier de forma progresiva hasta alcanzar aproximacion de orden
    los tres parametros que se modelan con la aproximacion de Fourier.

    Args:
        - ord_: Orden maximo de los modelos
        - data: Serie de valores
        - n: Dias desde el inicio del ano de la serie
        - par0 [opc] : Parametros iniciales de ajuste

    Returns:
        Una tupla de la forma (par, bic):

        *  ``par``: Matriz con parametros del ajuste
        *  ``bic``: Criterio de informacion Bayesiana calculado del modelo

    """

    logndat = np.log(len(data))
    par = [[[[] for i in xrange(ord_)] for i in xrange(ord_)] for i in xrange(ord_)]
    llf, bic = np.zeros([ord_, ord_, ord_]), np.zeros([ord_, ord_, ord_])
    if any(par0):
        par0 = np.array([par0[0], 0., 0., par0[1], 0., 0., par0[2], 0., 0., par0[3], par0[4]])
    else:
        par0 = np.array([np.mean(np.log(data)), 0., 0., np.std(np.log(data)), 0., 0., -.02, 0., 0., -2., 2.])
    par[0][0][0], llf[0, 0, 0] = fit(par0, data, n, [1, 1, 1])
    bic[0, 0, 0] = -2.*llf[0, 0, 0] + logndat*(3*2 + 5)
    for ord_ACT in range(1, ord_ + 1):
        logging.info('ord_ACT = ' + str(ord_ACT))
        for k in range(1, ord_ACT):
            i = ord_ACT
            for j in range(1, ord_ACT+1):
                logging.info(str(i) + str(j) + str(k) + ':i')
                if i+j+k == 3:
                    continue
                elif (j == 1) & (k == 1) & (not any(par[i-1][j-1][k-1])):
                    par0 = par[i-2][j-1][k-1]
                    NPA = (i-1)*2 + 1
                    par0 = np.concatenate((par0[0:NPA], [0.], [0.], par0[NPA:]))
                    par[i-1][j-1][k-1], llf[i-1, j-1, k-1] = fit(par0, data, n, [i, j, k])
                    bic[i-1, j-1, k-1] = -2*llf[i-1, j-1, k-1]+logndat*((i+k+j)*2 + 5)

                elif (j == 1) & (not any(par[i-1][j-1][k-1])):
                    par0 = par[i-1][j-1][k-2]
                    NPA = (i+j+k-1)*2+3
                    par0 = np.concatenate((par0[0:NPA], [0.], [0.], par0[NPA:]))
                    par[i-1][j-1][k-1], llf[i-1, j-1, k-1] = fit(par0, data, n, [i, j, k])
                    bic[i-1, j-1, k-1] = -2*llf[i-1, j-1, k-1]+logndat*((i+k+j)*2+5)

                elif not any(par[i-1][j-1][k-1]):
                    par0 = par[i-1][j-2][k-1]
                    NPA = (i+j-1)*2+2
                    par0 = np.concatenate((par0[0:NPA], [0.], [0.], par0[NPA:]))
                    par[i-1][j-1][k-1], llf[i-1, j-1, k-1] = fit(par0, data, n, [i, j, k])
                    bic[i-1, j-1, k-1] = -2*llf[i-1, j-1, k-1]+logndat*((i+k+j)*2+5)

            for i in range(ord_ACT, 0, -1):
                logging.info(str(i) + str(j) + str(k) + ':j')
                if not any(par[i-1][j-1][k-1]):
                    par0 = par[i-1][j-2][k-1]
                    NPA = (i+j-1)*2+2
                    par0 = np.concatenate((par0[0:NPA], [0.], [0.], par0[NPA:]))
                    par[i-1][j-1][k-1], llf[i-1, j-1, k-1] = fit(par0, data, n, [i, j, k])
                    bic[i-1, j-1, k-1] = -2*llf[i-1, j-1, k-1]+logndat*((i+k+j)*2+5)

        k = ord_ACT
        for i in range(1, ord_ACT+1):
            for j in range(1, ord_ACT+1):
                logging.info(str(i) + str(j) + str(k) + ':k')
                if (k > 1) & (not any(par[i-1][j-1][k-1])):
                    par0 = par[i-1][j-1][k-2]
                    NPA = (i+j+k-1)*2+3
                    par0 = np.concatenate((par0[0:NPA], [0.], [0.], par0[NPA:]))
                    par[i-1][j-1][k-1], llf[i-1, j-1, k-1] = fit(par0, data, n, [i, j, k])
                    bic[i-1, j-1, k-1] = -2*llf[i-1, j-1, k-1]+logndat*((i+k+j)*2+5)

    return par, bic
