# -*- coding: utf-8 -*-
"""
Created on Wed May 04 10:40:42 2016

@author: m_cob
"""
from __future__ import division
import numpy as np
from scipy.optimize import minimize
from scipy.signal import argrelmax
from scipy.stats import norm
from itertools import product
import utils
import logging
from datetime import datetime
logging.basicConfig(filename='debug_info.log', level=logging.INFO)


def prepara_param(data, nnorm):
    """ Funcion que calcula los parametros iniciales de busqueda en la minimizacion

    Args:
        - data: serie de direcciones
        - nnorm: numero de normales enrolladas

    Returns:
        Devuelve los parametros inciales
    """
    if len(data) > 2500:
        nbins, nord = 120, 3  # cada 3 grados
    else:
        nbins, nord = 40, 2  # cada 9 grados
    dat = data.copy()
    pdf, ext = np.histogram(dat, bins=nbins, normed=True)
    cdf = np.cumsum(pdf)/np.sum(pdf)
    ind_ = argrelmax(pdf, order=nord)[0]
    ind_s = np.sort(ind_[np.argsort(pdf[ind_])[-nnorm:]])
    par0 = np.zeros(2*nnorm)
    weight = np.append(0, cdf[ind_s])

    for i in range(nnorm):
        mask = ((dat <= ext[ind_s[i]]+np.pi/(3*nnorm)) & (dat > ext[ind_s[i]]-np.pi/(3*nnorm)))
        par0[2*i], par0[2*i+1] = np.mean(dat[mask]), np.std(dat[mask])
    
    par0 = np.append(par0, np.diff(weight)[:nnorm-1])

    return par0


def cdf(n, x, par, mod, nnorm):
    """Probabilidad acumulada de la distribucion formada por nnorm normales enrolladas

    Args:
        n: Fechas en escala anual [0 1]
        x: Valores de la variable
        par: Parametros del modelo:

            * par[0:mod[0]*2+1]: terminos independientes y de Fourier de MU1
            * par[2*mod[0]+1:4*mod[0]+2]: terminos independientes y de Fourier de SIG1
            * par[4*mod[0]+2:4*mod[0]+2*mod[1]+3]: terminos independientes y de Fourier de MU2
            * par[4*mod[0]+2*mod[1]+4:4*np.sum(mod[0:2])+4]: terminos independientes y de Fourier de SIG2
            * par[4*np.sum(mod[0:2])+4:4*np.sum(mod[0:2])+2*mod[2]+5]: terminos independientes y de Fourier de MU3
            * par[4*np.sum(mod[0:2])+2*mod[2]56:4*np.sum(mod[0:3])+6]: terminos independientes y de Fourier de SIG3
            * par[4*np.sum(mod[0:3])+6:4*np.sum(mod[0:3])+2*mod[3]+7]: terminos independientes y de Fourier de MU4
            * par[4*np.sum(mod[0:3])+2*mod[3]+7:4*np.sum(mod)+8]: terminos independientes y de Fourier de SIG4

        mod: Orden de aproximacion del modelo
        nnorm: Numero de normales enrolladas

    Returns:
        Una serie con la probabilidad de no excedencia

    """

    cossen = utils.cossen_n(mod, n)

    mu, sig = np.zeros([len(x), nnorm]), np.zeros([len(x), nnorm])
    mu[:, 0] = par[0] + np.dot(par[1:2*mod[0]+1], cossen[0:mod[0]*2, :])
    sig[:, 0] = par[2*mod[0]+1] + np.dot(par[2*mod[0]+2:4*mod[0]+2], cossen[0:mod[0]*2, :])
    mu[:, 1] = par[4*mod[0]+2] + np.dot(par[4*mod[0]+3:4*mod[0]+2*mod[1]+3], cossen[0:mod[1]*2, :])
    sig[:, 1] = par[4*mod[0]+2*mod[1]+3] + np.dot(par[4*mod[0]+2*mod[1]+4:4*np.sum(
        mod[0:2])+4], cossen[0:mod[1]*2, :])
    if nnorm >= 3:
        mu[:, 2] = par[4*np.sum(mod[0:2])+4] + np.dot(par[4*np.sum(mod[0:2])+5:4*np.sum(mod[0:2])+2*mod[2]+5], cossen[
            0:mod[2]*2, :])
        sig[:, 2] = par[4*np.sum(mod[0:2])+2*mod[2]+5] + np.dot(par[4*np.sum(mod[0:2])+2*mod[2]+6:4*np.sum(
            mod[0:3])+6], cossen[0:mod[2]*2, :])
    elif nnorm == 4:
        mu[:, 3] = par[4*np.sum(mod[0:3])+6] + np.dot(par[4*np.sum(mod[0:3])+7:4*np.sum(mod[0:3])+2*mod[3]+7], cossen[
            0:mod[3]*2, :])
        sig[:, 3] = par[4*np.sum(mod[0:3])+2*mod[3]+7] + np.dot(par[4*np.sum(mod[0:3])+2*mod[3]+8:4*np.sum(
            mod)+8], cossen[0:mod[3]*2, :])
    esc = par[-nnorm+1:]
    esc = np.append(esc, 1 - np.sum(esc))
    x = x/180.*np.pi

    p = 0
    for i in range(0, nnorm):
        for k in range(-100, 100):
            p = p + esc[i]*(norm.cdf(x+2*np.pi*k, mu[:, i], sig[:, i]) - norm.cdf(2*np.pi*k, mu[:, i], sig[:, i]))

    return p


def inv(n, prob, par, mod, nnorm, *cdf_):
    """Inversa de la funcion de distribucion mixta compuesta por 4 normales enrolladas no estacionarias

    Args:
        n: Fechas en escala anual [0 1]
        prob: Probabilidad de no excedencia
        par: Parametros del modelo:

            * par[0:mod[0]*2+1]: terminos independientes y de Fourier de MU1
            * par[2*mod[0]+1:4*mod[0]+2]: terminos independientes y de Fourier de SIG1
            * par[4*mod[0]+2:4*mod[0]+2*mod[1]+3]: terminos independientes y de Fourier de MU2
            * par[4*mod[0]+2*mod[1]+4:4*np.sum(mod[0:2])+4]: terminos independientes y de Fourier de SIG2
            * par[4*np.sum(mod[0:2])+4:4*np.sum(mod[0:2])+2*mod[2]+5]: terminos independientes y de Fourier de MU3
            * par[4*np.sum(mod[0:2])+2*mod[2]56:4*np.sum(mod[0:3])+6]: terminos independientes y de Fourier de SIG3
            * par[4*np.sum(mod[0:3])+6:4*np.sum(mod[0:3])+2*mod[3]+7]: terminos independientes y de Fourier de MU4
            * par[4*np.sum(mod[0:3])+2*mod[3]+7:4*np.sum(mod)+8]: terminos independientes y de Fourier de SIG4

        mod: Orden de aproximacion del modelo
        nnorm: Numero de normales enrolladas

    Returns:
        Una serie con los valores de la variable

    """

    n = n - np.floor(n)
    dt, dtt = 0.01, 2./180*np.pi
    t, tt = np.arange(dt/2, 1-dt/2, dt), np.arange(dtt, 2*np.pi, dtt)
    nt, ntt = len(t), len(tt)

    cossen = utils.cossen_n(mod, n)

    mu, sig = np.zeros([len(n), nnorm]), np.zeros([len(n), nnorm])
    mu[:, 0] = par[0] + np.dot(par[1:2*mod[0]+1], cossen[0:mod[0]*2, :])
    sig[:, 0] = par[2*mod[0]+1] + np.dot(par[2*mod[0]+2:4*mod[0]+2], cossen[0:mod[0]*2, :])
    mu[:, 1] = par[4*mod[0]+2] + np.dot(par[4*mod[0]+3:4*mod[0]+2*mod[1]+3], cossen[0:mod[1]*2, :])
    sig[:, 1] = par[4*mod[0]+2*mod[1]+3] + np.dot(par[4*mod[0]+2*mod[1]+4:4*np.sum(
        mod[0:2])+4], cossen[0:mod[1]*2, :])
    if nnorm >= 3:
        mu[:, 2] = par[4*np.sum(mod[0:2])+4] + np.dot(par[4*np.sum(mod[0:2])+5:4*np.sum(mod[0:2])+2*mod[2]+5], cossen[
            0:mod[2]*2, :])
        sig[:, 2] = par[4*np.sum(mod[0:2])+2*mod[2]+5] + np.dot(par[4*np.sum(mod[0:2])+2*mod[2]+6:4*np.sum(
            mod[0:3])+6], cossen[0:mod[2]*2, :])
    elif nnorm == 4:
        mu[:, 3] = par[4*np.sum(mod[0:3])+6] + np.dot(par[4*np.sum(mod[0:3])+7:4*np.sum(mod[0:3])+2*mod[3]+7], cossen[
            0:mod[3]*2, :])
        sig[:, 3] = par[4*np.sum(mod[0:3])+2*mod[3]+7] + np.dot(par[4*np.sum(mod[0:3])+2*mod[3]+8:4*np.sum(
            mod)+8], cossen[0:mod[3]*2, :])
    esc = par[-nnorm+1:]
    esc = np.append(esc, 1 - np.sum(esc))

    if not hasattr(cdf_, 'attr_name'):
        ppasa = np.zeros([nt, ntt])
        for j, i, k in product(np.arange(0, nt), np.arange(0, nnorm), np.arange(-100, 100)):
            ppasa[j, :] = ppasa[j, :] + esc[i]*(
                    norm.cdf(tt+2*np.pi*k, mu[j, i], sig[j, i]) - norm.cdf(2*np.pi*k, mu[j, i], sig[j, i]))
    else:
        ppasa = cdf_[0]        

    nprob = len(prob)
    tinv = np.zeros(nprob)
    for i in range(0, nprob):
        err = np.abs(n[i]-t)
        j = np.where(err == np.min(err))[0]
        err = np.abs(prob[i]-ppasa[j[0], :])
        posi = np.where(err == np.min(err))[0]
        if isinstance(posi, np.ndarray):
            posi = posi[0]
        tinv[i] = tt[posi]

    tinv = tinv*180/np.pi
    return tinv, ppasa


def nllf(par, data, cossen, mod, nnorm):
    """Negative Log Likelihood Function

    Args:
        par: parametros de la distribucion:

            * par[0:mod[0]*2+1]: terminos independientes y de Fourier de MU1
            * par[2*mod[0]+1:4*mod[0]+2]: terminos independientes y de Fourier de SIG1
            * par[4*mod[0]+2:4*mod[0]+2*mod[1]+3]: terminos independientes y de Fourier de MU2
            * par[4*mod[0]+2*mod[1]+4:4*np.sum(mod[0:2])+4]: terminos independientes y de Fourier de SIG2
            * par[4*np.sum(mod[0:2])+4:4*np.sum(mod[0:2])+2*mod[2]+5]: terminos independientes y de Fourier de MU3
            * par[4*np.sum(mod[0:2])+2*mod[2]56:4*np.sum(mod[0:3])+6]: terminos independientes y de Fourier de SIG3
            * par[4*np.sum(mod[0:3])+6:4*np.sum(mod[0:3])+2*mod[3]+7]: terminos independientes y de Fourier de MU4
            * par[4*np.sum(mod[0:3])+2*mod[3]+7:4*np.sum(mod)+8]: terminos independientes y de Fourier de SIG4

        data: Valores de la variable
        cossen: Variable auxiliar para calcular terminos de Fourier
        mod: orden de aproximacion que se quiere utilizar en cada parametro. mod tiene que tener 4 elementos

    Returns:
        El valor de la Negative Log Likelihood Function

    """
#    print par, mod, nnorm
    data = data.values
    mu, sig = np.zeros([len(data), nnorm]), np.zeros([len(data), nnorm])
    mu[:, 0] = par[0] + np.dot(par[1:2*mod[0]+1], cossen[0:mod[0]*2, :])
    sig[:, 0] = par[2*mod[0]+1] + np.dot(par[2*mod[0]+2:4*mod[0]+2], cossen[0:mod[0]*2, :])
    mu[:, 1] = par[4*mod[0]+2] + np.dot(par[4*mod[0]+3:4*mod[0]+2*mod[1]+3], cossen[0:mod[1]*2, :])
    sig[:, 1] = par[4*mod[0]+2*mod[1]+3] + np.dot(par[4*mod[0]+2*mod[1]+4:4*np.sum(
        mod[0:2])+4], cossen[0:mod[1]*2, :])
    if nnorm >= 3:
        mu[:, 2] = par[4*np.sum(mod[0:2])+4] + np.dot(par[4*np.sum(mod[0:2])+5:4*np.sum(mod[0:2])+2*mod[2]+5], cossen[
            0:mod[2]*2, :])
        sig[:, 2] = par[4*np.sum(mod[0:2])+2*mod[2]+5] + np.dot(par[4*np.sum(mod[0:2])+2*mod[2]+6:4*np.sum(
            mod[0:3])+6], cossen[0:mod[2]*2, :])
    elif nnorm == 4:
        mu[:, 3] = par[4*np.sum(mod[0:3])+6] + np.dot(par[4*np.sum(mod[0:3])+7:4*np.sum(mod[0:3])+2*mod[3]+7], cossen[
            0:mod[3]*2, :])
        sig[:, 3] = par[4*np.sum(mod[0:3])+2*mod[3]+7] + np.dot(par[4*np.sum(mod[0:3])+2*mod[3]+8:4*np.sum(
            mod)+8], cossen[0:mod[3]*2, :])
    esc = par[-nnorm+1:]
    esc = np.append(esc, 1 - np.sum(esc))
    
    if ((any(esc < 0)) | (any(esc >= 1)) | (np.min(sig) <= 0) | any(par == np.nan)):
        nllf = 1e10
    else:
        pdf = np.zeros((len(data), nnorm), dtype=np.complex128)
        for i in range(0, nnorm):
            p = 0
#            max_new = 1
#            rho = np.exp(-sig[:, i]**2/2)
            q, z = np.exp(-sig[:, i]**2), np.exp(1j*(data-mu[:, i]))
            if any((np.abs(q) >= 1) | (z == 0)):
                nllf = 1e10
            else:
#                while max_new > 1e-3:
                while p < 100:
                    p += 1
#                    new = rho**(p**2+1j*p*(data-mu[:, i]))
                    # new = np.real(np.exp(-sig[:, i]**2/2*p**2 + 1j*p*(data-mu[:, i])))
                    
                    new = (1-q**p)*(1+q**(p-0.5)*z)*(1+q**(p-0.5)/z)
#                    max_new = np.max(new)
#                    pdf[:, i] += new            
                    pdf[:, i] = pdf[:, i]*new      

            pdf[:, i] = esc[i]/(2*np.pi)*(1+2*pdf[:, i])

        pdf1 = np.sum(np.real(pdf), axis=1)
        nllf = -np.sum(np.log(pdf1))
        if np.isnan(nllf):
            nllf = 1e10

    return nllf


def fit(par0, data, *args):
    """Ajuste de la distribucion mixta compuesta por 4 normales enrolladas no estacionarias

    Args:
        par0: parametros del modelo. Los parametros son:

            * par[0:mod[0]*2+1]: terminos independientes y de Fourier de MU1
            * par[2*mod[0]+1:4*mod[0]+2]: terminos independientes y de Fourier de SIG1
            * par[4*mod[0]+2:4*mod[0]+2*mod[1]+3]: terminos independientes y de Fourier de MU2
            * par[4*mod[0]+2*mod[1]+4:4*np.sum(mod[0:2])+4]: terminos independientes y de Fourier de SIG2
            * par[4*np.sum(mod[0:2])+4:4*np.sum(mod[0:2])+2*mod[2]+5]: terminos independientes y de Fourier de MU3
            * par[4*np.sum(mod[0:2])+2*mod[2]56:4*np.sum(mod[0:3])+6]: terminos independientes y de Fourier de SIG3
            * par[4*np.sum(mod[0:3])+6:4*np.sum(mod[0:3])+2*mod[3]+7]: terminos independientes y de Fourier de MU4
            * par[4*np.sum(mod[0:3])+2*mod[3]+7:4*np.sum(mod)+8]: terminos independientes y de Fourier de SIG4

        data: valores de la variable
        pes: pesos de cada una de los picos direccionales
        args: [opt] argumento valido para los ajustes no estacionarios que contiene:

            * N: fecha en escala anual [0 1]
            * mod: orden de aproximacion que se quiere utilizar en cada parametro. Tiene cuatro componentes,
            identificando el nivel de aproximacion de los parametros
            * nnorm: numero de normales enrolladas

    Returns:
        Una tupla de la forma (par,llf):

    """

    if len(args) == 3:
        mod, nnorm, n = args
        cossen = utils.cossen_n(mod, n)
    else:
        mod, nnorm = args
        cossen = np.zeros([1, len(data)])

    comptot = np.sum(mod)+np.sum(mod[1:3])
    if len(par0) != comptot*2+5:
        logging.info('par0 no tiene la dimension adecuada')

    parBREAK = par0
    bnds = [[] for i in range(len(par0))]
    for i in range(0, len(par0)):
        bnds[i] = 0, 2*np.pi
    err, it = 1, 1
    while err:
        logging.info('Comienzo del ajuste a las: ' + str(datetime.now()))
        res = minimize(nllf, par0, args=(data, cossen, mod, nnorm), method='SLSQP', bounds=bnds, tol=1e-6, options={
            'maxiter': 1e3, 'disp': True})
        if ((res['fun'] != 0) & (res['fun'] < 1e8)):
            err = 0
            logging.info(res)
        elif ((it > 2e1) & (np.max(mod) != 1)):
            err = 0
            logging.info(res)
            res['x'], res['fun'] = parBREAK, np.nan
        else:
            it += 1
            logging.info('Solucion con parametros no operacionales')
            logging.info('Minimizando de nuevo con diferentes parametros de entrada')
            logging.info('Iteracion ' + str(it))
        par0 = parBREAK*np.random.rand(len(parBREAK))*2
    logging.info('Final del analisis a las: ' + str(datetime.now()))

    return res['x'], res['fun']


def n_est(ord_, par0, nnorm, data, n):
    """Ajusta los datos a una distribucion mixta formada por 'nnrom' normales enrolladas en el que se aumenta el orden
    de la aproximacion de Fourier de forma progresiva hasta alcanzar aproximacion de orden ord en los tres parametros
    que se modelan con la aproximacion de Fourier.

    Args:
        ord_: orden maximo de los modelos
        par0: parametros iniciales del ajuste
        nnorm: numero de normales enrolladas
        data: valores de la variables
        n: fecha en formato anual [0 - 1]

    Returns:
        Una tupla de la forma (par, bic)

        * ``par``: Matriz con parametros del ajuste
        * ``bic``: Criterios de Informacion Bayesiana calculado del modelo

    """

    logndat = np.log(len(data))
    par, bic, llf = ini_var(nnorm, ord_)

    data = data*np.pi/180

    par1 = np.zeros(6*nnorm + nnorm - 1)
    for i in range(0, nnorm):
        par1[6*i], par1[6*i+3] = par0[2*i], par0[2*i+1]

    par1[-nnorm+1:] = par0[-nnorm+1:]
    nk = ord_**4
    modos = np.zeros(nk)+1000
    modos[ord_:nk:ord_] = 100
    modos[ord_**2:nk:ord_**2] = 10
    modos[ord_**3:nk:ord_**3] = 1

    par, bic = bucles_nnorm(data, n, ord_, nnorm, modos, par, bic, llf, logndat, par1)

    return par, bic


def bucles_nnorm(x, n, ord_, nnorm, modos, par, bic, llf, logndat, par0):
    """ Funcion que realiza la busqueda de los parametros optimos para cada uno de los terminos del desarrollo de
    Fourier

    Args:
        - x: serie con los datos de direcciones
        - n: fecha en formato [0 - 1]
        - ord_: orden maximo de la expansion
        - nnorm: numero de normales enrolladas
        - modos: forma en la que trasladan los parametros iniciales del desarrollo desde el modo actual al siguiente
        - par: lista donde se guardan los parametros
        - bic: matriz donde se guardan los resultados del bic
        - llf: matriz donde se guardan los resultados del llf
        - logndat: longitud de la serie
        - par0: parametros inciales de busqueda

    Returns:
        Devuelve los parametros y el bic para cada modo

    """

    cont = -1
    if nnorm == 2:
        par[0][0], llf[0, 0] = fit(par0, x, np.ones(nnorm, dtype=np.int), nnorm, n)
        bic[0, 0] = -2*llf[0, 0]+logndat*(2*2 + 2)
        ol, ok, oj, oi = 0, 0, ord_, ord_
        for j in range(1, oj+1):
            for i in range(1, oi+1):
                cont += 1
                if i+j == nnorm:
                    continue

                elif modos[cont] == 1000:
                    logging.info(str(i) + str(j) + ':I')
                    par0 = par[i-2][j-1]
                    par0 = np.concatenate(
                        (par0[0:2*(i-1)+1], [0.], [0.],  par0[2*(i-1)+1:4*(i-1)+2], [0.], [0.], par0[4*(i-1)+2:]))
                    (par[i-1][j-1], llf[i-1, j-1]) = fit(par0, x, [i, j], nnorm, n)
                    bic[i-1, j-1] = -2*llf[i-1, j-1] + logndat*(2*(i+j)+4)

                elif modos[cont] == 100:
                    logging.info(str(i) + str(j) + ':J')
                    par0 = par[i-1][j-2]
                    par0 = np.concatenate(
                        (par0[0:4*i+2*(j-1)+3], [0.], [0.], par0[4*i+2*(j-1)+3:4*(i+(j-1))+4],
                         [0.], [0.], par0[4*(i+(j-1))+4:]))
                    (par[i-1][j-1], llf[i-1, j-1]) = fit(par0, x, [i, j], nnorm, n)
                    bic[i-1, j-1] = -2*llf[i-1, j-1] + logndat*(2*(i+j)+4)

    elif nnorm == 3:
        par[0][0][0], llf[0, 0, 0] = fit(par0, x, np.ones(nnorm, dtype=np.int), nnorm, n)
        bic[0, 0, 0] = -2*llf[0, 0, 0]+logndat*(4*2 + 4)
        ol, ok, oj, oi = 0, ord_, ord_, ord_
        for k in range(1, ok+1):
            for j in range(1, oj+1):
                for i in range(1, oi+1):
                    cont += 1
                    if i+j+k == nnorm:
                        continue

                    elif modos[cont] == 1000:
                        logging.info(str(i) + str(j) + str(k) + ':I')
                        par0 = par[i-2][j-1][k-1]
                        par0 = np.concatenate(
                            (par0[0:2*(i-1)+1], [0.], [0.],  par0[2*(i-1)+1:4*(i-1)+2], [0.], [0.], par0[4*(i-1)+2:]))
                        (par[i-1][j-1][k-1], llf[i-1, j-1, k-1]) = fit(par0, x, [i, j, k], nnorm, n)
                        bic[i-1, j-1, k-1] = -2*llf[i-1, j-1, k-1] + logndat*(2*(i+j+k)+4)

                    elif modos[cont] == 100:
                            logging.info(str(i) + str(j) + str(k) + ':J')
                            par0 = par[i-1][j-2][k-1]
                            par0 = np.concatenate(
                                (par0[0:4*i+2*(j-1)+3], [0.], [0.], par0[4*i+2*(j-1)+3:4*(i+(j-1))+4],
                                 [0.], [0.], par0[4*(i+(j-1))+4:]))
                            (par[i-1][j-1][k-1], llf[i-1, j-1, k-1]) = fit(par0, x, [i, j, k], nnorm, n)
                            bic[i-1, j-1, k-1] = -2*llf[i-1, j-1, k-1] + logndat*(2*(i+j+k)+4)

                    elif modos[cont] == 10:
                        logging.info(str(i) + str(j) + str(k) + ':K')
                        par0 = par[i-1][j-1][k-2]
                        par0 = np.concatenate(
                            (par0[0:4*(i+j)+2*(k-1)+5], [0.], [0.], par0[4*(i+j)+2*(k-1)+5:4*(
                             i+j+(k-1))+6], [0.], [0.], par0[4*(i+j+(k-1))+6:]))
                        (par[i-1][j-1][k-1], llf[i-1, j-1, k-1]) = fit(par0, x, [i, j, k], nnorm, n)
                        bic[i-1, j-1, k-1] = -2*llf[i-1, j-1, k-1] + logndat*(2*(i+j+k)+4)

    else:
        ol, ok, oj, oi = ord_, ord_, ord_, ord_
        par[0][0][0][0], llf[0, 0, 0, 0] = fit(par0, x, np.ones(nnorm, dtype=np.int), nnorm, n)
        bic[0, 0, 0, 0] = -2*llf[0, 0, 0, 0]+logndat*(4*2 + 4)
        for l in range(1, ol+1):
            for k in range(1, ok+1):
                for j in range(1, oj+1):
                    for i in range(1, oi+1):
                        cont += 1
                        if i+j+k+l == nnorm:
                            continue

                        elif modos[cont] == 1000:
                            logging.info(str(i) + str(j) + str(k) + str(l) + ':I')
                            par0 = par[i-2][j-1][k-1][l-1]
                            par0 = np.concatenate(
                                (par0[0:2*(i-1)+1], [0.], [0.],  par0[2*(i-1)+1:4*(i-1)+2], [0.], [0.],
                                 par0[4*(i-1)+2:]))
                            (par[i-1][j-1][k-1][l-1], llf[i-1, j-1, k-1, l-1]) = fit(par0, x, [i, j, k, l], nnorm, n)
                            bic[i-1, j-1, k-1, l-1] = -2*llf[i-1, j-1, k-1, l-1] + logndat*(2*(i+j+k+l)+4)

                        elif modos[cont] == 100:
                            logging.info(str(i) + str(j) + str(k) + str(l) + ':J')
                            par0 = par[i-1][j-2][k-1][l-1]
                            par0 = np.concatenate(
                                (par0[0:4*i+2*(j-1)+3], [0.], [0.], par0[4*i+2*(j-1)+3:4*(i+(j-1))+4],
                                 [0.], [0.], par0[4*(i+(j-1))+4:]))
                            (par[i-1][j-1][k-1][l-1], llf[i-1, j-1, k-1, l-1]) = fit(par0, x, [i, j, k, l], nnorm, n)
                            bic[i-1, j-1, k-1, l-1] = -2*llf[i-1, j-1, k-1, l-1] + logndat*(2*(i+j+k+l)+4)

                        elif modos[cont] == 10:
                            logging.info(str(i) + str(j) + str(k) + str(l) + ':K')
                            par0 = par[i-1][j-1][k-2][l-1]
                            par0 = np.concatenate(
                                (par0[0:4*(i+j)+2*(k-1)+5], [0.], [0.], par0[4*(i+j)+2*(k-1)+5:4*(
                                 i+j+(k-1))+6], [0.], [0.], par0[4*(i+j+(k-1))+6:]))
                            (par[i-1][j-1][k-1][l-1], llf[i-1, j-1, k-1, l-1]) = fit(par0, x, [i, j, k, l], nnorm, n)
                            bic[i-1, j-1, k-1, l-1] = -2*llf[i-1, j-1, k-1, l-1] + logndat*(2*(i+j+k+l)+4)

                        elif modos[cont] == 1:
                            logging.info(str(i) + str(j) + str(k) + str(l) + ':L')
                            par0 = par[i-1][j-1][k-1][l-2]
                            par0 = np.concatenate(
                                (par0[0:4*(i+j+k)+2*(l-1)+7], [0.], [0.],
                                 par0[4*(i+j+k)+2*(l-1)+7:4*((i+j+k)+(l-1))+8], [0.], [0.],
                                 par0[4*((i+j+k)+(l-1))+8:]))
                            (par[i-1][j-1][k-1][l-1], llf[i-1, j-1, k-1, l-1]) = fit(par0, x, [i, j, k, l], nnorm, n)
                            bic[i-1, j-1, k-1, l-1] = -2*llf[i-1, j-1, k-1, l-1] + logndat*(2*(i+j+k+l)+4)

    return par, bic


def ini_var(nnorm, ord_):
    """ Funcion que inicializa las variables par, bic y llf a partir del numero de normales enrolladas y del orden de
    desarrollo de Fourier definidos

    Args:
        - nnorm: numero de normales enrolladas
        - ord_: orden del desarrollo de Fourier

    Returns:
        Las listas con las variables vacias

    """

    if nnorm == 2:
        par = [[[] for i in xrange(ord_)] for i in xrange(ord_)]
        llf, bic = np.zeros([ord_, ord_]), np.zeros([ord_, ord_])
    elif nnorm == 3:
        par = [[[[] for i in xrange(ord_)] for i in xrange(ord_)] for i in xrange(ord_)]
        llf, bic = np.zeros([ord_, ord_, ord_]), np.zeros([ord_, ord_, ord_])
    elif nnorm == 4:
        par = [[[[[] for i in xrange(ord_)] for i in xrange(ord_)] for i in xrange(ord_)] for i in xrange(ord_)]
        llf, bic = np.zeros([ord_, ord_, ord_, ord_]), np.zeros([ord_, ord_, ord_, ord_])

    return par, bic, llf
