# -*- coding: utf-8 -*-

from __future__ import division
import numpy as np
import matplotlib.pyplot as plt
import scipy.stats as st
import pandas as pd
import utils as ut
from scipy import interpolate
import scipy.stats as st
from statsmodels.distributions import empirical_distribution
from scipy.optimize import curve_fit
from clima_maritimo.clima_maritimo.fdist.regimen_extremal import lmom
#from clima_maritimo.graficas.plot_conjunto import plot_copula
from clima_maritimo.clima_maritimo.fdist.copula import Copula as Copula
from sklearn.neighbors import KernelDensity
from scipy.interpolate import interp1d

# from clima_maritimo.graficas import plot_conjunto
#import clima_maritimo.graficas.plot_conjunto as plc


def ajuste_sobre_umbral(h, t, alpha, nv):
    """ Funcion que devuelve los parametros del ajuste de la pareja (H_s-T_z) sobre el umbral previamente establecido

    Args:
        - h: serie de altura de ola significante
        - t: serie con el periodo medio
        - alpha: nivel de significancia
        - nv: numero de años

    Returns:
        Los parametros y las graficas del ajuste
    """

    pf = 1-(1-alpha)**nv

    ind = np.argsort(h)
    nc = int(len(h)*pf)
    hh = h[ind][nc:]
    tt = t[ind][nc:]

    plt.figure()
    plt.plot(h, t, '.', color='gray')
    slope, intercept, r_value, p_value, std_err = st.linregress(hh, tt)
    popts = np.polyfit(hh, tt, 2)
    plt.plot(hh, tt, '.k')
    hq = np.linspace(np.min(hh), np.max(hh), 25)
    plt.plot(hq, slope*hq + intercept, 'k', lw=2, label='Ajuste')
    plt.annotate(r'$R^2=$'+str(np.round_(r_value**2, 4)), xy=(4, 2))
    plt.legend(loc='best')
    plt.xlabel(r'$H_s$')
    plt.ylabel(r'$T_z$')

    return popts


def ajuste_copula_fourier(x, y, M):
    """ Funcion que ajusta las series x e y a una copula de fourier bivariada

    Args:
        - x e y: series de las variables
        - nombrex y nombre y: nompbre

    Returns:
        Devuelve las matrices
    """

    xord = np.sort(x)
    XP = empirical_distribution.ECDF(xord)
    F1 = XP(x)

    yord = np.sort(y)
    YP = empirical_distribution.ECDF(yord)
    F2 = YP(y)

    # Copula Empírica ---------------------------------------------------------
    ulim = np.linspace(0, 1, M)
    vlim = np.linspace(0, 1, M)

    du = ulim[1]-ulim[0]
    dv = vlim[1]-vlim[0]

    frecs = np.histogram2d(F1, F2, (ulim, vlim))[0]

    Nu, Nv = np.shape(frecs)
    Ntot = np.sum(np.sum(frecs))
    Cop = np.zeros([Nu+1, Nv+1])
    Cop_pi = np.zeros([Nu+1, Nv+1])

    for j in range(1, Nu+1):
        for k in range(1, Nv+1):
            Cop[j, k] = np.sum(np.sum(frecs[0:j, 0:k]))/Ntot
            Cop_pi[j, k] = ulim[j-1]*vlim[k-1]

    cop = frecs/Ntot/du/dv
    coefs = np.fft.fft2(cop)
    coefs[0, 0] = 0
    coefs[0, :] = 0*coefs[0, :]
    coefs[:, 0] = 0*coefs[:, 0]

    c_ap = 1+np.fft.ifft2(coefs)

    p_ = M/(M+1)
    ulim_ap = p_*ulim
    vlim_ap = p_*vlim
    duap = ulim_ap[1]-ulim_ap[0]
    dvap = vlim_ap[1]-vlim_ap[0]
    Cop_ap = np.ones([Nu+1, Nv+1])
    crec = np.zeros([M, M])
    crec[1:, 1:] = np.real(c_ap)

    for j in range(1, Nu+1):
        for k in range(1, Nv+1):
            Cop_ap[j-1, k-1] = np.sum(np.sum(crec[0:j, 0:k]))*duap*dvap

    Cop_ap = Cop_ap*p_*p_
    P_ = Cop_ap[-1, -1]

    Cop_ap = Cop_ap/P_
    Cop_ap[-1, :] = ulim
    Cop_ap[:, -1] = vlim.T

    ulim, vlim = np.meshgrid(ulim_ap, vlim_ap)
    xord, yord = np.interp(ulim_ap, XP(xord), xord), np.interp(vlim_ap, YP(yord), yord)
    xord, yord = np.meshgrid(xord, yord)

    return ulim, vlim, Cop, ulim_ap, vlim_ap, Cop_ap, xord, yord


def condicionada(x, y, xlim, ylim, ndat, dist, reg, peso, distx):
    """ Funcion que calcula la funcion de distribucion conjunta a partir de la condicionada de la variable principal
    'x' con 'y'.

    Args:
        - x: variable principal
        - y: variable secundaria
        - xlim: limites a considerar para la variable x
        - ylim: limites a considerar para la variable y
        - ndat: numero de datos de la ventana movil usada para estimar la distribución dist
        - dist: distribucion usada para f(y|x)
        - reg: lista con los nombres de las regresiones a usar para cada uno de los parametros de dist.
            Las cuatro opciones posibles por el momento son: poli1, poli2, pote1 y pote2.
        - peso: indica si pondera los datos para ajustar las regresiones o no. Las opciones son:
            * ``0``: no pondera
            * ``1``: pondera con x
            * ``2``: pondera con x**2
        - distx: informacion para construir la distribución marginal de  x, necesaria por la distribución bivariada.
        Debe contener: distx[0]: gpd o empirico-gpd; (solo para el caso empirico) distx[1], el umbral.

    Returns:
        Parametros del ajuste, el intervalo de confianza y la funcion de distribucion conjunta

    """

    alpha = 0.1
    xaux = np.linspace(xlim[0], xlim[1], 1000)
    yaux = np.linspace(ylim[0], ylim[1], 1000)

    x, y = x.values, y.values
    x, id_ = np.sort(x), np.argsort(x)
    y = y[id_]
    ndat = np.round(ndat/2)
    nest = int(np.size(x)-2*ndat)

    xparaux = list()
    xpar = np.zeros([nest, 5])
    for i in range(0, nest):
        # OBTENGO LOS PARÁMETROS DE LA CONDICIONAL PARA CADA VENTANA
        paraux = dist.fit(y[i:i+2*ndat])
        # PROBABILIDAD DE NO EXCEDENCIA DE LOS DATOS DE LA VENTANA AL PUNTO MEDIO DE LA VENTANA
        proby = np.sum(y[i:i+2*ndat] <= y[i+ndat])/(2*ndat+2)
 
        hist, binedg = np.histogram(y[i:i+2*ndat], normed=True)
        chisq, pval = st.chisquare(np.cumsum(hist), dist.cdf((binedg[0:-1]+binedg[1:])/2, *paraux))
        xpar[i, :] = (x[i+ndat], y[i+ndat], proby, pval, chisq)
        xparaux.append(paraux)
    d_pesos = dict()
    d_pesos['0'], d_pesos['1'], d_pesos['2'] = xpar[:, 0]*0+1, xpar[:, 0], xpar[:, 0]**2.

#    if reg[-1] == '1':
#        p1 = (3, 0.5)
#    elif reg[-1] == '2':
#        p1 = (0.5, -2, 0.5)

    fit = list()
    for i in range(len(xparaux[0])):
        fit.append(curve_fit(reg[i], xpar[:, 0], np.asarray(xparaux)[:, i], sigma=d_pesos[peso]))
        print fit[i]

#    param = np.zeros([nest, len(xparaux[0])])
    param = list()
    for i in range(len(xparaux[0])):
#        param[:, i] = reg[i](xpar[:, 0], *fit[i][0])
        param.append(reg[i](xpar[:, 0], *fit[i][0]))

#    yest = dist.ppf(xpar[:, 2][:, np.newaxis], *np.split(param, len(xparaux[0]), axis=1))
    yest = dist.ppf(xpar[:, 2], *param)

    if hasattr(distx, 'attr_name'):
        u0 = distx[1]
        k, mu, sig = lmom(x[x > u0], 'estandar')
        p = np.sort(x)
        xp = empirical_distribution.ECDF(p)
        xxaux, pdfaux = xp(p), p
        pdfxaux = np.interp(xaux, xxaux, pdfaux)
        pdfxaux[xaux > mu] = np.sum(x > mu)/(np.size(x)+1)*st.pareto(xaux[xaux > mu], k, mu, sig)
    else:
        paraux2 = distx.fit(x)
        pdfxaux = distx.pdf(xaux, *paraux2)

    xg, yg = np.meshgrid(xaux, yaux)
    pdfg = np.ones(np.shape(xg))
#    param2 = np.zeros([np.size(xaux), len(xparaux[0])])
    param2 = list()
    for i in range(len(xparaux[0])):
#        param2[:, i] = reg[i](xaux, *fit[i][0])
        param2.append(reg[i](xaux, *fit[i][0]))

#    for i in range(np.size(pdfg[1, :])):
    pdfg = dist.pdf(yaux[:, np.newaxis],  *param2)*pdfxaux

#    alp = np.ones([2, np.size(xaux)])
    alp = np.array([alpha/2, 1-alpha/2])
    ci = np.zeros([2, np.size(xaux)])
    for j in range(0, 2):
        ci[j, :] = dist.ppf(alp[j], *param2)

    valmed = dist.ppf(0.5, *param2)

    return xpar, xparaux, param2, yest, xg, yg, pdfg, ci, valmed, xaux


def regresion(event, df, umb, comb, fun, sup, *dir_us):
    """ Funcion que ajusta a los datos una funcion de regresion

    Args:
        - event: dataframes de los picos para los distintos umbrales
        - df: dataframe con todos los valores que se quieran utilizar para los ajustes
        - umb: umbrales
        - comb: lista con los nombres de las series a las que se le quiere ajustar la funcion de regresion
        - fun: lista con las funciones de regresion que se quieren utilizar (poli1, poli2, pote1, pote2)
        - dir_us: diccionario con direcciones principales a considerar (opcional)

    Returns:
        Devuelve los resultados de la regresion con el intervalo de confianza

    """

    df.replace([np.inf, -np.inf], np.nan)
    df = df.dropna()
    dicc = dict()
    dicc['poli1'], dicc['poli2'], dicc['pote1'], dicc['pote2'] = (poli1, poli2, pote1, pote2)
    dicc['exp1'], dicc['exp2'], dicc['pab1'], dicc['pab2'] = (exp1, exp2, pab1, pab2)
    df = df.values
    alpha = 0.05
    numb = len(umb)

    reg = pd.DataFrame()
    lcomb = len(comb)

    # # 8 direcciones
    # dir_bins = [0, 22.5, 67.5, 112.5, 157.5, 202.5, 247.5, 292.5, 338.5, 360]
    # dir_lab = ['N', 'NE', 'E', 'SE', 'S', 'SW', 'W', 'NW', 'N2']
    #
    # dir_dic = {'N': [338.5, 22.5], 'NE': [22.5, 67.5], 'E': [67.5, 112.5], 'SE': [112.5, 157.5],
    #            'S': [157.5, 202.5], 'SW': [202.5, 247.5], 'W': [247.5, 292.5], 'NW': [292.5, 338.5]}

    # 4 direcciones
    dir_bins = [0, 45, 135, 225, 315, 360]
    dir_lab = ['N', 'E', 'S', 'W', 'N2']

    dir_dic = {'N': [315, 45], 'E': [45, 135], 'S': [135, 225], 'W': [225, 315]}

    for k in range(1, lcomb+1):
        if ((comb[k-1] == 'dh') | (comb[k-1] == 'dv')):

            for i in range(0, len(fun)):

                if fun[i][-1] == '1':
                    p1 = (3, 0.5)
                elif fun[i][-1] == '2':
                    p1 = (-10, 5, 5)

                for j in range(numb):
                    xaux = np.linspace(np.max((np.min(df[:, 0], 0))), 2 * np.max(df[:, 0]),
                                       1000)  # TODO: no se podria meter df.min()[0] y quitar la linea df=df.values
                    eventi = event[j].values
                    if sup:
                        id_ = eventi[:, 0] >= umb[j]
                    else:
                        id_ = eventi[:, 0] <= umb[j]

                    x = eventi[id_, 0]
                    y = eventi[id_, k]

                    if any(dir_us):
                        dir_2pr = pd.DataFrame.from_dict(dir_us[0]).all()
                        dir_dic = dir_us[0]


                    else:
                        df_prov = event[j].loc[id_, :]
                        df_prov['d_bins'] = pd.cut(df_prov[comb[k - 1]], bins=dir_bins, labels=dir_lab)

                        df_prov.loc[df_prov['d_bins'] == 'N2', 'd_bins'] = 'N'

                        dir_pr_all = df_prov['d_bins'].value_counts()
                        dir_pr_all.drop(['N2'], inplace=True)
                        dir_pr_all = dir_pr_all[dir_pr_all > 0]
                        dir_pr_all = dir_pr_all[dir_pr_all > 0.1 * dir_pr_all.sum()]
                        dir_2pr = dir_pr_all.nlargest(2)

                        if len(dir_2pr)==2:
                            dir1 = dir_dic[dir_2pr.index[0]]
                            dir2 = dir_dic[dir_2pr.index[1]]

                            inter_dir = set(dir1).intersection(dir2)

                            if inter_dir:
                                dirt = np.append(dir1, dir2)

                                index_d = np.argwhere(dirt == list(inter_dir)[0])

                                if ((dir_2pr.index == 'N') | (dir_2pr.index == 'E')).all():
                                    dir_n = np.sort(np.delete(dirt, index_d))[::-1]
                                    dir_2pr.drop(['E'], inplace=True)
                                    dir_dic['N'] = dir_n

                                elif ((dir_2pr.index == 'N') | (dir_2pr.index == 'W')).all():
                                    dir_n = np.sort(np.delete(dirt, index_d))[::-1]
                                    dir_2pr.drop(['W'], inplace=True)
                                    dir_dic['N'] = dir_n
                                else:
                                    dir_n = np.sort(np.delete(dirt, index_d))
                                    dir_2pr.drop([dir_2pr.index[1]], inplace=True)
                                    dir_dic[dir_2pr.index[0]] = dir_n

                    for kk in range(len(dir_2pr)):
                        dirpi = dir_dic[dir_2pr.index[kk]]
                        if dirpi[1]-dirpi[0] < 0:
                            id_dir = (((y>dirpi[0]) & (y<=360)) | ((y>=0) & (y<dirpi[1])))
                            x2 = x[id_dir]
                            y2 = y[id_dir]
                            y2 = np.rad2deg(np.unwrap(np.deg2rad(y2)))
                            reg['dirN' + comb[k - 1] + fun[i] + str(umb[j])] = True

                        else:
                            id_dir = (y>dirpi[0]) & (y<dirpi[1])
                            x2 = x[id_dir]
                            y2 = y[id_dir]
                            reg['dirN' + comb[k - 1] + fun[i] + str(umb[j])] = False

                        par = curve_fit(dicc[fun[i]], x2, y2, p0=p1, maxfev=100000)
                        jac = jacob_fun(xaux, fun[i], par[0])
                        yaux = dicc[fun[i]](xaux, *par[0])
                        ss_res = np.mean((y2 - dicc[fun[i]](x2, *par[0])) ** 2)
                        predvar = ss_res + (jac * np.dot(par[1], jac.T).T).sum(1)
                        predstd = st.t.cdf(1 - alpha, len(xaux) - len(par[0])) * np.sqrt(predvar)
                        reg['x' + comb[k - 1] + fun[i] + str(umb[j]) + 'dir' + str(kk)] = xaux
                        reg['ysup' + comb[k - 1] + fun[i] + str(umb[j]) + 'dir' + str(kk)] = yaux + predstd
                        reg['y' + comb[k - 1] + fun[i] + str(umb[j]) + 'dir' + str(kk)] = yaux
                        reg['yinf' + comb[k - 1] + fun[i] + str(umb[j]) + 'dir' + str(kk)] = yaux - predstd
                    reg['ndirp' + comb[k - 1] + fun[i] + str(umb[j])] = len(dir_2pr)

        else:
            for i in range(0, len(fun)):
                if fun[i][-1] == '1':
                    p1 = (3, 0.5)
                elif fun[i][-1] == '2':
                    p1 = (-10, 5, 5)

                for j in range(numb):
                    xaux = np.linspace(np.max((np.min(df[:, 0], 0))), 2*np.max(df[:, 0]), 1000)  # TODO: no se podria meter df.min()[0] y quitar la linea df=df.values
                    eventi = event[j].values
                    if sup:
                        id_ = eventi[:, 0] >= umb[j]
                    else:
                        id_ = eventi[:, 0] <= umb[j]

                    x = eventi[id_, 0]
                    y = eventi[id_, k]

                    par = curve_fit(dicc[fun[i]], x, y, p0=p1, maxfev=100000)
                    jac = jacob_fun(xaux, fun[i], par[0])
                    yaux = dicc[fun[i]](xaux, *par[0])
                    ss_res = np.mean((y-dicc[fun[i]](x, *par[0]))**2)
                    predvar = ss_res + (jac * np.dot(par[1], jac.T).T).sum(1)
                    predstd = st.t.cdf(1-alpha, len(xaux)-len(par[0]))*np.sqrt(predvar)
                    reg['x'+comb[k-1]+fun[i]+str(umb[j])] = xaux
                    reg['ysup'+comb[k-1]+fun[i]+str(umb[j])] = yaux + predstd
                    reg['y'+comb[k-1]+fun[i]+str(umb[j])] = yaux
                    reg['yinf'+comb[k-1]+fun[i]+str(umb[j])] = yaux - predstd

    return reg


def regresion_adim(event, df, fun):
    """ Funcion que ajusta a los datos de hs, tp y v, a una funcion de regresion considerando como valores limite los
    dados por los monomios adimensionales para oleaje totalmente desarrollado sin limitacion de profundidad.

    Args:
        - event:
        - df: dataframe con todos las series
        - fun: nombre de la funcion de regresion que se quieren utilizar (poli1, poli2, pote1, pote2)

    Returns:
        Devuelve los resultados de la regresion con el intervalo de confianza

    """

    df.replace([np.inf, -np.inf], np.nan)
    df = df.dropna()
    dicc = dict()
    dicc['poli1'], dicc['poli2'], dicc['pote1'], dicc['pote2'] = (poli1, poli2, pote1, pote2)
    df = df.values
    alpha = 0.05
    nomb = ['tp', 'vv', 'adim']
    reg = pd.DataFrame()
    par = list()
    eventi = event[0].values

    for k in range(0, 3):
        if fun[-1] == '1':
            p1 = (3, 0.5)
        elif fun[-1] == '2':
            p1 = (-10, 5, 5)

        xaux = np.linspace(np.max((np.min(df[:, 0], 0))), 1.5*np.max(df[:, 0]), 1000)

        if k == 2:
            x = eventi[:, 0]*9.81/eventi[:, 2]**2
            y = eventi[:, 1]*9.81/eventi[:, 2]
            xaux = np.linspace(0,  0.24, 1000)
        else:
            x = eventi[:, 0]
            y = eventi[:, k+1]

        par.append(curve_fit(dicc[fun], x, y, p0=p1, maxfev=100000))
        jac = jacob_fun(xaux, fun, par[k][0])
        yaux = dicc[fun](xaux, *par[k][0])
        ss_res = np.mean((y-dicc[fun](x, *par[k][0]))**2)
        predvar = ss_res + (jac * np.dot(par[k][1], jac.T).T).sum(1)
        predstd = t.cdf(1-alpha, len(xaux)-len(par[k][0]))*np.sqrt(predvar)
        reg['x'+nomb[k]+str(fun)] = xaux
        reg['ysup'+nomb[k]+str(fun)] = yaux + predstd
        reg['y'+nomb[k]+str(fun)] = yaux
        reg['yinf'+nomb[k]+str(fun)] = yaux - predstd

    hs_aux = np.linspace(np.min(df[:, 0], 0), 2*np.max(df[:, 0]), 1000)
    v_aux = dicc[fun](hs_aux, *par[1][0])
    hst_aux = hs_aux*9.81/(v_aux**2.)
    hst_aux[hst_aux > 0.2541] = 0.2541
    t_aux = dicc[fun](hst_aux, *par[2][0])
    t_aux[t_aux > 7.944] = 7.944
    t_aux = t_aux*v_aux/9.81
    reg['hs_aux'] = hs_aux
    reg['t_aux'] = t_aux

    return reg


def tr(x, y, pr, tipo, nanio=False, nevumb=False, *args):
    """ Funcion que calcula los periodos de retorno a partir de la probabilidad

    Args:
        - x: serie de valores de la variable 1
        - y: serie de valores de la variable 2
        - pr: periodo de retorno seleccionado
        - tipo: variable que indica si las variables son maximos anuales ``max_``, picos sobre umbral ``pot_`` o
            conjuntas ``con``.
        - nanio: numero de años
        - nevumb: numero de temporales
        - familia: de la copula

    Returns:
        Devuelve la serie (x1, x2) cuyo valor de periodo de retorno es el dado

    """
#    lvls = np.array([2, 5, 10, 20, 50, 100, 200, 500, 1000])
    nel = 200

#    if pr in lvls:
#        ind = np.where(pr == lvls)[0]
#    else:
#        lvls = np.append(lvls, pr)
#        lvls = lvls[np.argsort(lvls)]
#        ind = np.where(pr == lvls)[0]

    x1, y1 = list(), list()
    cont = 0
    if args:
        cop_ = Copula(x, y, args[0][0], args[0][1:])
    else:
        cop_ = Copula(x, y, 'clayton')

    x_cdf = np.arange(0, nel)
    cdf = 1-np.exp(-x_cdf/20)
    print cdf
#    cdf = np.linspace(0, 1, nel)
    Cop_ap = cop_.generate_C(cdf, cdf)
    cop_.U, cop_.V = cdf, cdf
    xord, yord = cop_.generate_xy()
    if args:
        Fx, Fy = args[0][1].cdf(xord, *args[0][2]), args[0][3].cdf(yord, *args[0][4])

    else:
        x1 = np.linspace(x.min(), x.max(), 1000)
        x2 = np.cumsum(
            np.exp(KernelDensity(kernel='epanechnikov').fit(x[:, np.newaxis]).score_samples(x1[:, np.newaxis])))
        x2 = x2 / np.max(x2)

        Fx = interp1d(x2, x1, bounds_error=False, fill_value='extrapolate', kind='nearest')

        y1 = np.linspace(y.min(), y.max(), 1000)
        y2 = np.cumsum(
            np.exp(KernelDensity(kernel='epanechnikov').fit(y[:, np.newaxis]).score_samples(y1[:, np.newaxis])))
        y2 = y2 / np.max(y2)
        Fy = interp1d(y2, y1, bounds_error=False, fill_value='extrapolate', kind='nearest')

    Fx, Fy = np.meshgrid(Fx, Fy)
    if tipo == 'max_':
        p_ret = 1/(1+Cop_ap-Fx-Fy)
    elif tipo == 'pot_':
        p_ret = 1/((1+Cop_ap-Fx-Fy)*(len(x)/nanio))
    elif tipo == 'con_':
        p_ret = 1/((1+Cop_ap-Fx-Fy)*nevumb/nanio)

    for i in pr:
        cs = plt.contour(xord, yord, p_ret, levels=[i])
        plt.close()
        for collection in cs.collections:
            paths = collection.get_paths()
            for path in paths:
                mask = ((path.vertices[:, 0] == np.max(xord)) | (path.vertices[:, 1] == np.max(yord)))
                x1.append(path.vertices[~mask, 0])
                y1.append(path.vertices[~mask, 1])
        cont += 1

    return x1, y1, xord, yord, Cop_ap


def poli1(x, a, b):
    y = a*x + b
    return y


def poli2(x, a, b, c):
    y = a*x**2 + b*x + c
    return y


def pote1(x, a, b):
    y = a*x**b
    return y


def pote2(x, a, b, c):
    y = a*x**b + c
    return y


def exp1(x, a, b):
    y = a*np.exp(b*x)
    return y


def exp2(x, a, b, c):
    y = a*np.exp(b*x) + c
    return y


def pab1(x, a, b):
    y = a/(b*x)
    return y


def pab2(x, a, b, c):
    y = a/(b*x) + c
    return y


def jacob_fun(xaux, fun, *popts):
    if fun == 'poli1':
        jac = np.ones([len(xaux), 2])
        jac[:, 0] = xaux
    elif fun == 'poli2':
        jac = np.ones([len(xaux), 3])
        jac[:, 0], jac[:, 1] = xaux**2, xaux
    elif fun == 'pote1':
        jac = np.zeros([len(xaux), 2])
        jac[:, 0], jac[:, 1] = xaux**popts[0][1], popts[0][0]*xaux**popts[0][1]*np.log(xaux)
    elif fun == 'pote2':
        jac = np.zeros([len(xaux), 3])
        jac[:, 0], jac[:, 1] = xaux**popts[0][1], popts[0][0]*xaux**popts[0][1]*np.log(xaux)
    elif fun == 'exp1':
        jac = np.ones([len(xaux), 2])
        jac[:, 0], jac[:, 1] = np.exp(popts[0][1]*xaux), popts[0][0]*xaux*np.exp(popts[0][1]*xaux)
    elif fun == 'exp2':
        jac = np.ones([len(xaux), 3])
        jac[:, 0], jac[:, 1] = np.exp(popts[0][1]*xaux), popts[0][0]*xaux*np.exp(popts[0][1]*xaux)
    elif fun == 'pab1':
        jac = np.ones([len(xaux), 2])
        jac[:, 0], jac[:, 1] = -1/(popts[0][0]+popts[0][1]*xaux)**2, -xaux/(popts[0][0]+popts[0][1]*xaux)**2
    elif fun == 'pab2':
        jac = np.ones([len(xaux), 3])
        jac[:, 0], jac[:, 1] = -1/(popts[0][0]+popts[0][1]*xaux)**2, -xaux/(popts[0][0]+popts[0][1]*xaux)**2
    return jac


def eventos_sobre_umbral(df, var, umbral, tmin_ciclo, tmin_calma):

    df['tag'] = df[var] > umbral

    # DURACION CICLOS
    # first row is a True preceded by a False
    fst_c = np.asarray(df.index[df['tag'] & ~ df['tag'].shift(1).fillna(False)])
    fst_ca = np.asarray(df.index[~ df['tag'] & df['tag'].shift(-1).fillna(False)])

    for jj, fst_ca_jj in enumerate(fst_ca):
        if df.index[0] == fst_c[0]:
            jj = jj + 1
        df_prov = df.loc[fst_ca_jj:fst_c[jj]].resample('T').interpolate()
        df_prov = df_prov[df_prov[var] > umbral].fillna(True)
        df_prov.drop(fst_c[jj], inplace=True)
        df = pd.concat([df, df_prov]).sort_index()
        if not df_prov.empty:
            fst_c[jj] = df_prov.index[0]

    # last row is a True followed by a False
    lst_c = np.asarray(df.index[df['tag'] & ~ df['tag'].shift(-1).fillna(False)])
    lst_cd = np.asarray(df.index[~ df['tag'] & df['tag'].shift(1).fillna(False)])

    for jj, lst_cd_jj in enumerate(lst_cd):
        df_prov = df.loc[lst_c[jj]:lst_cd_jj].resample('T').interpolate()
        df_prov = df_prov[df_prov[var] > umbral].fillna(True)
        df_prov.drop(lst_c[jj], inplace=True)
        df = pd.concat([df, df_prov]).sort_index()
        if not df_prov.empty:
            lst_c[jj] = df_prov.index[-1]

    # DURACION CALMA
    fst_i = lst_c[:-1]
    lst_i = fst_c[1:]

    hs_max_c, dur_c, dur_i, index_max = list(), list(), list(), list()
    for fst_ci, lst_ci, fst_ii, lst_ii in zip(fst_c, lst_c, fst_i, lst_i):
        hs_max_c.append(df.loc[fst_ci:lst_ci][var].max())
        dur_c.append(lst_ci - fst_ci)
        dur_i.append(lst_ii - fst_ii)
        index_max.append(df.loc[fst_ci:lst_ci][var].idxmax())

    index_max_date = np.asarray(index_max)

    # Quitar las filas donde la calma es menor de 6 horas. Se suma la calma y el ciclo siguiente al ciclo y se coge la calma del siguiente
    while (dur_i < np.timedelta64(tmin_calma, 'h')).any():
        indx_i_no = np.where(dur_i < np.timedelta64(6, 'h'))[0]
        indx_i_no = np.delete(indx_i_no, [np.where(np.diff(indx_i_no) == 1)[0] + 1])

        for indx_i_no_ind in indx_i_no:
            dur_c[indx_i_no_ind] = dur_c[indx_i_no_ind] + dur_i[indx_i_no_ind] + dur_c[indx_i_no_ind + 1]
            dur_i[indx_i_no_ind] = dur_i[indx_i_no_ind + 1]

            index_max_date[indx_i_no_ind] = index_max_date[[indx_i_no_ind, indx_i_no_ind + 1]
            [np.argmax([hs_max_c[indx_i_no_ind], hs_max_c[indx_i_no_ind + 1]])]]
            hs_max_c[indx_i_no_ind] = np.max([hs_max_c[indx_i_no_ind], hs_max_c[indx_i_no_ind + 1]])

        hs_max_c = np.delete(hs_max_c, indx_i_no + 1)
        fst_c = np.delete(fst_c, indx_i_no + 1)
        lst_c = np.delete(lst_c, indx_i_no + 1)
        dur_c = np.delete(dur_c, indx_i_no + 1)
        dur_i = np.delete(dur_i, indx_i_no + 1)
        index_max_date = np.delete(index_max_date, indx_i_no + 1)

    # Quitar las filas donde el ciclo es menor de 6 horas. Se suma la calma y el ciclo a la calma del anterior
    while (dur_c < np.timedelta64(tmin_ciclo, 'h')).any():
        indx_c_no = np.where(dur_c < np.timedelta64(6, 'h'))[0]
        indx_c_no = np.delete(indx_c_no, [np.where(np.diff(indx_c_no) == 1)[0] + 1])

        for indx_c_no_ind in indx_c_no:
            dur_i[indx_c_no_ind - 1] = dur_i[indx_c_no_ind - 1] + dur_c[indx_c_no_ind] + dur_i[indx_c_no_ind]

        hs_max_c = np.delete(hs_max_c, indx_c_no)
        fst_c = np.delete(fst_c, indx_c_no)
        lst_c = np.delete(lst_c, indx_c_no)
        dur_c = np.delete(dur_c, indx_c_no)
        dur_i = np.delete(dur_i, indx_c_no)
        index_max_date = np.delete(index_max_date, indx_c_no)

    df_vars_ciclos = pd.DataFrame(
        data={var + '_max': hs_max_c, 'ini_c': fst_c[:-1], 'fin_c': lst_c[:-1]})
    df_vars_ciclos.set_index(index_max_date, inplace=True)

    return df_vars_ciclos
