# -*- coding: utf-8 -*-


from __future__ import division
import numpy as np
import os
from scipy.stats import genpareto, norm, genextreme
from scipy.special import gamma
from scipy.optimize import fsolve
from scipy.interpolate import RegularGridInterpolator


def calcula_bca(boot, alpha, bca, *args):
    """ Funcion que calcula los intervalos de confianza

    Args:
        - boot: matriz con los resultados del remuestreo
        - alpha: nivel de significancia
        - bca: metodo para calcular los intervalos de confianza (estandar o bca)
        - args[opc]: argumentos opcionales entre los que se encuentran:

            * ``orig``: parametros originales del modelo
            * ``nsimu``: numero de simulaciones
            * ``picos``: serie de picos

    Returns:
        Devuelve los intervalos de confianza (limite inferior y superior) de la serie

    """

    if bca == 'estandar':
        ci = np.percentile(boot, np.array((alpha/2, 1-alpha/2))*100, axis=0)
    elif bca == 'bca':
        orig, nsimu, picos, tri, tipo, nanios = args[0]
        npicos, norig = np.size(picos), np.size(orig)
        pz0 = np.zeros(norig)
        for i in range(0, norig):
            mask = boot[:, i] < orig[i]
            pz0[i] = np.sum(mask)/float(nsimu)
        z0 = norm.ppf(pz0)
        u = np.zeros([npicos, norig])
        id_ = np.arange(npicos)
        for i in range(0, npicos):
            u[i, :] = -ajusta_lmom(picos[id_ != i], tri, tipo, nanios) + orig
        a = np.sum(u**3)/np.sum(u**2)**(3/2)/6
        zalpha = norm.ppf((alpha/2, 1-alpha/2))
        prob = np.zeros([2, norig])
        prob[0, :] = norm.cdf(z0+(z0+zalpha[0])/(1-a*(z0+zalpha[0])))
        prob[1, :] = norm.cdf(z0+(z0+zalpha[1])/(1-a*(z0+zalpha[1])))
        ci = np.zeros([2, norig])
        for i in range(0, norig):
            if not (np.isnan(prob[0, i]) | np.isnan(prob[1, i])):
                ci[:, i] = np.percentile(boot[:, i], prob[:, i]*100, axis=0)
    else:
        print 'No se indico si los ci se calculan de forma estandar o mediante bca'

    return ci


def calcula_boot(picos, tri, tipo, nanios, nsimu, param):
    """ Funcion que calcula los parametros y los valores del ajuste para determinados periodos de retorno y para la
    funcion de probabilidad de extremos dada

    Args:
        - picos: serie con los eventos extremos
        - tri: vector con los periodos de retorno
        - tipo: funcion de ajuste
        - nanios: numero de anios para la simulacion
        - nsimu: numero de simulaciones para la tecnica de remuestreo
        - param: tipo de tecnica de remuestreo

    Returns:
        Devuelve los parametros y valores del periodo de retorno obtenidos de la serie original y mediante la tecnica
        de remuestreo

    """

    npicos = np.size(picos)
    orig = ajusta_lmom(picos, tri, tipo, nanios)
    boot = np.zeros([nsimu, np.size(orig)])
    if param == 'parametrico':
        if ((tipo == 'exp') | (tipo == 'estandar')):
            for j in range(0, nsimu):
                boot[j, :] = ajusta_lmom(genpareto.rvs(orig[0], orig[1], orig[2], npicos), tri, tipo, nanios)
        elif ((tipo == 'gev') | (tipo == 'gumb')):
            for j in range(0, nsimu):
                boot[j, :] = ajusta_lmom(genextreme.rvs(-orig[0], orig[1], orig[2], npicos), tri, tipo)
    elif param == 'no-parametrico':
        for j in range(0, nsimu):
            boot[j, :] = ajusta_lmom(np.random.choice(picos, npicos), tri, tipo, nanios)
    else:
        print 'No se definio si la tecnica de remuestreo es parametrica o no-parametrica'

    return boot, orig


def ajusta_lmom(data, tr, tipo, *nanios):
    """ Funcion que calcula los parametros de la funcion de probabilidad dada

    Args:
        - data: serie con los datos de partida
        - tr: periodos de retorno para los que se desea obtener el valor
        - tipo: metodo de calculo para l-moments.
        - *nanios[opc]: numero de anios de la muestra

    Returns:
        Devuelve los parametros del ajuste, el parametro nu y el valor asociado a los periodos de retorno

    """

    k, sig, mu = lmom(data, tipo)
    if ((tipo == 'exp') | (tipo == 'estandar')):
        nu = np.size(data)/nanios[0]
        pri = 1-1/(tr*nu)
        qtr = genpareto.ppf(pri, k, mu, sig)
    elif ((tipo == 'gev') | (tipo == 'gumb')):
        pri = 1-1./tr
        qtr = genextreme.ppf(pri, -k, mu, sig)
        nu = 1

    return np.hstack((k, mu, sig, nu, qtr))


def lmom(data, tipo):
    """ Funcion que calcula los parametros de forma, localizacion y escala a partir del metodo l-moments

    Args:
        - data: datos de partida
        - tipo: funcion de ajuste

    Returns:
        Devuelve los parametros

    """

    data = np.sort(data)
    n = np.size(data)
    p = (np.arange(1, n+1)-0.35)/n
    lam1, lam2 = np.mean(data), np.mean((2*p-1)*data)
    if tipo == 'estandar':
        lam3 = np.mean((1-6*p+6*p**2)*data)
        tau3 = lam3/lam2
        k = (3*tau3-1)/(1+tau3)
        sig = lam2*(1-k)*(2-k)
        mu = lam1 - sig/(1-k)
    elif tipo == 'exp':
        sig = 2*lam2
        mu = lam1 - sig
        k = 0
    elif tipo == 'gev':
        lam3 = np.mean((1-6*p+6*p**2)*data)
        tau3 = lam3/lam2

        def fun(x, tau3): return 2.*(1-3.**-x)/(1-2.**-x)-3-tau3
        c = 2/(3+tau3)-np.log(2)/np.log(3)
        k = 7.859*c + 2.9554*c**2
        k = fsolve(fun, k, args=(tau3))
        sig = lam2*k/(1-2**-k)/gamma(1+k)
        mu = lam1-sig*(1-gamma(1+k))/k
        k = -k
    elif tipo == 'gumb':
        sig = lam2/np.log(2)
        mu = lam1 - 0.5772*sig
        k = 0

    return k, sig, mu


def evalua_lmom(data, tr, anios):
    """ Funcion que calcula el valor de los parametros de la pareto generalizada a partir del metodo de los l-moments

    Args:
        - data: serie de picos
        - tr: periodos de retorno
        - anios: numero de anios de la serie

    Returns:
        Devuelve todos los parametros del ajuste

    """

    k, sig, mu = lmom(data, 'estandar')
    mrlp = np.mean(data - mu)
    sigmodif = sig - k*mu
    nu = np.size(data)/anios
    pri = 1-1/(tr*nu)
    qtr = genpareto.ppf(pri, k, mu, sig)
    return np.hstack((k, mu, sig, mrlp, sigmodif, nu, qtr))


def analisis_pot_lmon(df_eventos, alpha, nanios):
    """ Funcion que calcula los parametros de la funcion de probabilidad pareto generalizada a partir de la técnica de
    l-moment y sus intervalos de confianza a partir de técnicas de remuestreo.

    Args:
        - df_eventos: dataframe con los picos
        - alpha: nivel de significancia
        - nanios: numero de anios de la muestra

    Returns:
        Devuelve una tupla con los valores del test de Anderson-Darling, test de Anderson-Darling modificado, el valor
        medio, los intervalos de confianza, el umbral definido, periodos de retorno y el numero de anios de la muestra

    """

    pvallmom, au2val, ko, ndata = np.load(os.path.join(os.path.dirname(__file__), 'pval_au2_lmom.npy'))
    interpolador = RegularGridInterpolator((ndata, ko, au2val), pvallmom, method='linear')

    nsimu = 10000
    # Picos sobre el umbral con pwm y ci. bootstrapping

    # Identifica eventos
    event = df_eventos.values[:, 0]

    # Inicializa variables
    treval = np.array([50, 100, 1000])
    umb = np.sort(event)[0:-15]
    umb = umb[np.diff(umb) > 0]

    numb = np.size(umb)
    valmed_lmom, limsup, liminf, au2_lmom, au2pv_lmom = (
        np.zeros([numb, np.size(treval)+6]), np.zeros([numb, np.size(treval)+6]), np.zeros([
            numb, np.size(treval)+6]), np.zeros(numb), np.zeros(numb))

    # Analisis
    for i in range(numb):
        print str(i+1)+' de '+str(numb)
        evi = event[event > umb[i]]
        nevi = np.size(evi)
        valmed_lmom[i, :] = evalua_lmom(evi, treval, nanios)

    # Bootstraping casero
        boot = np.zeros([nsimu, np.size(treval)+6])

        for j in range(0, nsimu):
            boot[j, :] = evalua_lmom(np.random.choice(evi, nevi), treval, nanios)

        limsup[i, :] = np.percentile(boot, (1-alpha/2)*100, axis=0)
        liminf[i, :] = np.percentile(boot, (alpha/2)*100, axis=0)

        # Anderson-Darlin Upper sin intervalo de confianza para L-Moments
        au2_lmom[i] = au2(valmed_lmom[i, 0:3], evi)
        au2pv_lmom[i] = interpolador([np.max((10, np.min((500, nevi)))), np.sign(valmed_lmom[i, 0])*np.min((np.abs(
            valmed_lmom[i, 0]), 0.5)), au2_lmom[i]])

    return au2pv_lmom, au2_lmom, valmed_lmom, limsup, liminf, umb, treval, nanios


def au2(par, data):
    """ Funcion que calcula el valor del estadistico Anderson-Darling

    Args:
        - par: parametros medios de la funcion de distribucion de pareto
        - data: picos sobre umbral

    Returns:
        Devuelve el valor del test

    """

    ndat = np.size(data)
    cdf = np.sort(genpareto.cdf(data, *par))
    cdf -= 1e-6  # TODO: para evitar los 1.0 en la cdf
    estadist = ndat/2-2*np.sum(cdf)-np.sum((2-(2*np.arange(1, ndat+1)-1.)/ndat)*np.log(1-cdf))

    return estadist


def automatico_lmom_boot(df_eventos, alpha, umbral, param, bca, nanios):
    """ Funcion que calcula los parametros de la funcion de Pareto generalizada a partir de tecnicas de remuestreo,
    seleccionado el umbral, la tecnica de remuestreo y calcula de los intervalos de confianza

    Args:
        - df_eventos: serie de datos de entrada
        - alpha: nivel de significancia para los intervalos de confianza
        - umbral: umbral seleccionado para el regimen extremal
        - param: duracion (dias) de la ventana movil para identificar los picos
        - bca: metodo de calculo de los intervalos de confianza
        - nanios: numero de anios de la muestra

    Returns:
        Devuelve una tupla con:

            * ``boot``: matriz con los resultados de la aplicacion de la tecnica de remuestreo
            * ``orig``: parametros originales de la funcion, ci, tr, picos, npicos, eventanu

    """
    umb = list()
    umb.append(umbral)
    tr = np.hstack((np.arange(1, 11), np.arange(2, 11)*10, np.arange(2, 11)*100))
    nsimu = 10000

    #  PICOS SOBRE EL UMBRAL CON PWM Y C.I. BOOTSTRAPPING
    # Identifica eventos
    numb = np.size(umb)
    event = df_eventos.values[:, 0]

    boot, orig, ci = [list() for i in range(2)], [list() for i in range(2)], [list() for i in range(2)]
    for i in range(numb):
        df_pot = df_eventos[df_eventos.values[:, 0] > umb[i]]  # TODO: ver las funciones siguientes para eliminar picos
        # y dejar solo df_pot
        picos = event[event > umb[i]]
        npicos = np.size(picos)
        eventanu = npicos/nanios

        # Valor central y Bootstrapping "casero" con umbral predefinido
        boot_s, orig_s = calcula_boot(picos, tr, 'estandar', nanios, nsimu, param)
        ci_s = calcula_bca(boot_s, alpha, bca, (orig_s, nsimu, picos, tr, 'estandar', nanios))
        boot[0].append(boot_s), orig[0].append(orig_s), ci[0].append(ci_s)

    # Si exponencial es posible, la calcula
        expo = ((ci_s[0, 0] < 0) & (ci_s[1, 0] > 0))
        if expo:
            boot_e, orig_e = calcula_boot(picos, tr, 'exp', nanios, nsimu, param)
            ci_e = calcula_bca(boot_e, alpha, bca, (orig_e, nsimu, picos, tr, 'exp', nanios))
            boot[1].append(boot_e), orig[1].append(orig_e), ci[1].append(ci_e)

    return boot, orig, ci, tr, picos, npicos, eventanu


def maximo_anual_lmom_boot(df, alpha, param, bca):
    """ Funcion que calcula los parametros de la funcion de GEV, los intervalos de confianza y los valores para
    distintos periodos de retorno

    Args:
        - df: serie de maximos anuales
        - alpha: nivel de significancia para los intervalos de confianza
        - param: metodo de remuestreo
        - bca: metodo de calculo de los intervalos de confianza

    Returns:
        Devuelve una tupla con la matriz de los parametros ajustados de la aplicacion de la tecnica de remuestreo, de
        la serie original y sus intervalos de confianza

            * ``boot``: matriz con los resultados de la aplicacion de la tecnica de remuestreo
            * ``orig``: parametros originales de la funcion, ci, tr, picos, npicos, eventanu

    """
    # Maximos anuales para comparar
    nanios = df.index.year.max() - df.index.year.min()
    tr = np.hstack((np.arange(1, 11), np.arange(2, 11) * 10, np.arange(2, 11) * 100))
    tri = np.hstack((np.arange(1.1, 2 + 1e-6, 0.1), tr[tr > 2]))
    maxanu = df.groupby(df.index.year).max().values[:, 0]
    pmaxanu = np.arange(1., nanios + 2.) / (nanios + 2.)

    nsimu = 10000
    boot, orig, ci = [list() for i in range(2)], [list() for i in range(2)], [list() for i in range(2)]
    boot_a, orig_a = calcula_boot(maxanu, tri, 'gev', nanios, nsimu, param)
    ci_a = calcula_bca(boot_a, alpha, bca, (orig_a, nsimu, maxanu, tri, 'gev', nanios))
    boot[0].append(boot_a), orig[0].append(orig_a), ci[0].append(ci_a)
    # Si Gumbel es posible, la calcula
    gumb = ((ci_a[0, 0] < 0) & (ci_a[1, 0] > 0))
    if gumb:
        boot_g, orig_g = calcula_boot(maxanu, tri, 'gumb', nanios, nsimu, param)
        ci_g = calcula_bca(boot_g, alpha, bca, (orig_g, nsimu, maxanu, tri, 'gumb', nanios))
        boot[1].append(boot_g), orig[1].append(orig_g), ci[1].append(ci_g)

    return tri, pmaxanu, maxanu, boot, orig, ci, tr
