# -*- coding: utf-8 -*-
from __future__ import division
import numpy as np
from scipy.optimize import minimize
from scipy.stats import weibull_min, norm, genpareto
from statsmodels.distributions import empirical_distribution
import utils
import logging
import pandas as pd
from datetime import datetime
logging.basicConfig(filename='debug_info.log', level=logging.INFO)


def prepara_param(data):
    """ Funcion que calcula los parametros iniciales de busqueda en la minimizacion

    Args:
        - data: serie de direcciones
        - nnorm: numero de normales enrolladas

    Returns:
        Devuelve los parametros inciales
    """
    data = np.sort(data)
    xp = empirical_distribution.ECDF(data)
    u1, u2 = np.interp(np.array([0.05, 0.95]), xp(data), data)
    par_wb = weibull_min.fit(data)
    a, b = par_wb[-1], par_wb[-3]
    par_gp = genpareto.fit(data[data > u2]-u2)
    xi2 = -par_gp[-3]
    par0 = np.array([a, b, xi2, -1., 3.])

    return par0

def cdf(n, data, par, mod):
    """Probabilidad acumulada de la distribucion WB-GPD

    Args:
        n: Fechas en escala anual [0 1]
        data: Valores de la variable
        par: Parametros del modelo:

            * par[1:mod[0]*2+1]: terminos independientes y de Fourier de mu
            * par[mod[0]*2+1:np.sum(mod[0:2])*2+2]: terminos independientes y de Fourier de sig
            * par[np.sum(mod[0:2])*2+1:np.sum(mod)*2+3]: terminos independientes y de Fourier de xi2
            * par[-2:]: terminos z1 y z2

        mod: Orden de aproximacion del modelo para mu, sig y xi2

    Returns:
        Un vector con la probabilidad de no excedencia

    """

    cossen = utils.cossen_n(mod, n)

    mu = par[0] + np.dot(par[1:mod[0]*2+1], cossen[0:mod[0]*2, :])
    sig = par[mod[0]*2+1] + np.dot(par[mod[0]*2+2:np.sum(mod[0:2])*2+2], cossen[0:mod[1]*2, :])
    xi2 = (par[np.sum(mod[0:2])*2+2] + np.dot(par[np.sum(mod[0:2])*2+3:np.sum(mod)*2+3], cossen[0:mod[2]*2, :]))
    u1 = weibull_min.ppf(norm.cdf(par[-2]), sig, mu)
    u2 = weibull_min.ppf(norm.cdf(par[-1]), sig, mu)

    esc1 = norm.cdf(par[-2])
    esc2 = 1-norm.cdf(par[-1])
    siggp1 = esc1/weibull_min.pdf(u1, sig, mu)
    xi1 = -siggp1/u1
    siggp2 = esc2/weibull_min.pdf(u2, sig, mu)

    prob = np.zeros(len(data))

    for i in range(0, len(data)):
        if data[i] < u1[i]:
            prob[i] = esc1*(1-xi1[i]/siggp1[i]*(data[i]-u1[i]))**(-1/xi1[i])
        elif data[i] > u2[i]:
            prob[i] = 1-esc2 + esc2*(1-(1+xi2[i]/siggp2[i]*(data[i]-u2[i]))**(-1/xi2[i]))
        else:
            prob[i] = weibull_min.cdf(data[i], sig[i], scale=mu[i])

    return prob


def inv(n, prob, par, mod):
    """Inversa de la funcion de distribucion WB-GPD

    Args:
        n: Fechas en escala anual [0 1]
        prob: Probabilidad de no excedencia
        par: Parametros del modelo:

            * par[1:mod[0]*2+1]: terminos independientes y de Fourier de mu
            * par[mod[0]*2+1:np.sum(mod[0:2])*2+2]: terminos independiente y de Fourier de sig
            * par[np.sum(mod[0:2])*2+1:np.sum(mod)*2+3]: terminos independientes y de Fourier de xi2
            * par[-2:]: terminos z1 y z2

        mod: Orden de aproximacion del modelo para mu, sig y xi2

    Returns:
        Un vector con los valores de la variable

    """

    lsim = len(n)
    cossen = utils.cossen_n(mod, n)

    mu = par[0] + np.dot(par[1:mod[0]*2+1], cossen[0:mod[0]*2, :])
    sig = par[mod[0]*2+1] + np.dot(par[mod[0]*2+2:np.sum(mod[0:2])*2+2], cossen[0:mod[1]*2, :])
    xi2 = par[np.sum(mod[0:2])*2+2] + np.dot(par[np.sum(mod[0:2])*2+3:np.sum(mod)*2+3], cossen[0:mod[2]*2, :])
    u1 = weibull_min.ppf(norm.cdf(par[-2]), sig, mu)
    u2 = weibull_min.ppf(norm.cdf(par[-1]), sig, mu)

    esc1 = norm.cdf(par[-2])
    esc2 = 1-norm.cdf(par[-1])

    siggp1 = esc1/weibull_min.pdf(u1, sig, mu)
    xi1 = -siggp1/u1
    siggp2 = esc2/weibull_min.pdf(u2, sig, mu)

    data = np.zeros(lsim)

    for i in range(0, lsim):
        if prob[i] < esc1:
            data[i] = u1[i]-siggp1[i]*((prob[i]/esc1)**(-xi1[i])-1)/xi1[i]
        elif prob[i] > (1-esc2):
            data[i] = u2[i]-siggp2[i]*(1-(1-(prob[i]-1+esc2)/esc2)**(-xi2[i]))/xi2[i]
        else:
            data[i] = weibull_min.ppf(prob[i], sig[i], mu[i])

    return data


def nllf(par, data, cossen, mod):
    """Negative Log Likelihood Function

    Args:
        par: Parametros del modelo:

            * par[1:mod[0]*2+1]: terminos independientes y de Fourier de mu
            * par[mod[0]*2+1:np.sum(mod[0:2])*2+2]: terminos independiente y de Fourier de sig
            * par[np.sum(mod[0:2])*2+1:np.sum(mod)*2+3]: terminos independientes y de Fourier de xi2
            * par[-2:]: terminos z1 y z2

        data: Valores de la variable
        cossen: Variable auxiliar para calcular terminos de Fourier
        mod: Orden de aproximacion que se quiere utilizar

    Returns:
        El valor de la Negative Log Likelihood Function

    """

    df = pd.DataFrame(index=data.index)
    df['mu'] = par[0] + np.dot(par[1:mod[0]*2+1], cossen[0:mod[0]*2, :])
    df['sig'] = par[mod[0]*2+1] + np.dot(par[mod[0]*2+2:np.sum(mod[0:2])*2+2], cossen[0:mod[1]*2, :])
    df['xi2'] = par[np.sum(mod[0:2])*2+2] + np.dot(par[np.sum(mod[0:2])*2+3:np.sum(mod)*2+3], cossen[0:mod[2]*2, :])
    df['u1'] = weibull_min.ppf(norm.cdf(par[-2]), df['sig'], df['mu'])
    df['u2'] = weibull_min.ppf(norm.cdf(par[-1]), df['sig'], df['mu'])
    esc1 = norm.cdf(par[-2])
    esc2 = 1-norm.cdf(par[-1])
    df['siggp1'] = esc1/weibull_min.pdf(df['u1'], df['sig'], df['mu'])
    df['xi1'] = -df['siggp1']/df['u1']
    df['siggp2'] = esc2/weibull_min.pdf(df['u2'], df['sig'], df['mu'])

    if ((np.min(df['u1']) <= 0) | (np.min(df['u2']) <= 0) | (np.min(df['mu']) <= 0) | (np.min(df['sig']) <= 0)):
        nllf = 1e10
    else:
        idwb = (data <= df['u2']) & (data >= df['u1'])
        idgp1 = data < df['u1']
        idgp2 = data > df['u2']

        if (any(df['xi1'][idgp1]*(data[idgp1]-df['u1'][idgp1])/df['siggp1'][idgp1] >= 1) | any(df['xi2'][idgp2]*(
                data[idgp2]-df['u2'][idgp2])/df['siggp2'][idgp2] <= -1)):
            nllf = 1e10
        else:
            lPDFWB = np.sum(np.log(weibull_min.pdf(data[idwb], df['sig'][idwb], df['mu'][idwb])))
            if not any(idgp1):
                lPDFGP1 = 0
            else:
                lPDFGP1_1 = np.log(esc1)-np.log(df['siggp1'][idgp1])
                lPDFGP1_2a = 1+1./df['xi1'][idgp1]
                lPDFGP1_2b = np.log(1-df['xi1'][idgp1]/df['siggp1'][idgp1]*(data[idgp1]-df['u1'][idgp1]))
                lPDFGP1 = lPDFGP1_1 - lPDFGP1_2a*lPDFGP1_2b
                lPDFGP1 = np.sum(lPDFGP1)

            if not any(idgp2):
                lPDFGP2 = 0
            else:
                lPDFGP2_1 = np.log(esc2)-np.log(df['siggp2'][idgp2])
                lPDFGP2_2a = 1+1./df['xi2'][idgp2]
                lPDFGP2_2b = np.log(1+df['xi2'][idgp2]/df['siggp2'][idgp2]*(data[idgp2]-df['u2'][idgp2]))
                lPDFGP2 = lPDFGP2_1 - lPDFGP2_2a*lPDFGP2_2b
                lPDFGP2 = np.sum(lPDFGP2)

            nllf = -lPDFWB-lPDFGP1-lPDFGP2

    return nllf


def fit(par0, data, *args):
    """Ajuste de la distribucion WB-GPD no estacionaria

    Args:
        par0: Parametros del modelo:

            * par[1:mod[0]*2+1]: terminos independientes y de Fourier de mu
            * par[mod[0]*2+1:np.sum(mod[0:2])*2+2]: terminos independiente y de Fourier de sig
            * par[np.sum(mod[0:2])*2+1:np.sum(mod)*2+3]: terminos independientes y de Fourier de xi2
            * par[-2:]: terminos z1 y z2

        data: Valores de la variable
        n: Fechas en escala anual [0 1]
        mod: Orden de aproximacion que se quiere utilizar

    Returns:
        Una tupla de la forma (par,llf).

        * ``par``: parametros de la distribucion.
        * ``llf``: Log Likelihood Function evaluada en los parametros

    """

    if args:
        n, mod = args
        cossen = utils.cossen_n(mod, n)
    else:
        cossen, mod = np.zeros([1, len(data)]), [0, 0, 0]

    comptot = np.sum(mod)
    if len(par0) != comptot*2+5:
        logging.info('par0 no tiene la dimension adecuada')

    parBREAk = par0
    err, it = 1, 1
    bnds = [[] for i in range(len(par0))]
    for i in range(0, len(par0)):
        bnds[i] = -4, 4

    while err:
        logging.info('Comienzo del ajuste a las: ' + str(datetime.now()))
        res = minimize(nllf, par0, args=(data, cossen, mod), method='SLSQP', bounds=bnds, tol=1e-6, options={
            'maxiter': 1e3, 'disp': True})
        if (res['fun'] != 0) & (res['fun'] < 1e8):
            err = 0
            logging.info(res)
        elif ((it > 2e1) & (np.max(mod) != 1)):
            err = 0
            logging.info(res)
            res['x'], res['fun'] = parBREAk, np.nan
        else:
            it += 1
            logging.info('Solucion con parametros no operacionales')
            logging.info('Minimizando de nuevo con diferentes parametros de entrada')
            logging.info('iteracion ' + str(it))
            par0 = parBREAk*np.random.rand(len(parBREAk))*2
    logging.info('Final del analisis a las: ' + str(datetime.now()))

    return res['x'], res['fun']


def n_est(ord_, data, n, par0=0):
    """Ajusta los datos a una distribucion mixta formada por una weibull central y dos paretos generalizadas en los
    extremos aumentando el orden de la aproximacion de Fourier de forma progresiva hasta alcanzar aproximacion de orden
    ORD en los tres parametros que se modelan con la aproximacion de Fourier.

    Args:
        ord_: orden maximo de los modelos
        data: valores de la variables a ajustar
        n: Fechas en escala anual [0 1]
        par0 [opc]: parametros iniciales

    Returns:
        Una tupla de la forma (par, bic)

        * ``par``: Matriz con parametros del ajuste
        * ``bic``: Criterios de informacion Bayesiana calculado del modelo

    """

    logndat = np.log(len(data))
    par = [[[[] for i in xrange(ord_)] for i in xrange(ord_)] for i in xrange(ord_)]
    llf, bic = np.zeros([ord_, ord_, ord_]), np.zeros([ord_, ord_, ord_])
    if any(par0):
        par0 = np.array([par0[0], 0., 0., par0[1], 0., 0., par0[2], 0., 0., par0[3], par0[4]])
    else:
        dat = np.sort(data)
        xp = empirical_distribution.ECDF(dat)
        u1, u2 = np.interp(np.array([0.05, 0.95]), xp(dat), dat)
        par_wb = weibull_min.fit(dat)
        a, b = par_wb[-1], par_wb[-3]
        par_gp = genpareto.fit(dat[dat > u2]-u2)
        xi2 = par_gp[-3]
        par0 = np.array([a, b, xi2, u1, u2])
        par0, llf[0, 0, 0] = fit(par0, data, n, np.zeros(3, dtype=np.int))
        dat = weibull_min.ppf(norm.cdf(par0[3:]), par0[1], par0[0])
        par0 = np.array([par0[0], 0., 0., par0[1], 0., 0., par0[2], 0., 0., par0[3], par0[4]])

    par[0][0][0], llf[0, 0, 0] = fit(par0, data, n, [1, 1, 1])
    bic[0, 0, 0] = -2*llf[0, 0, 0]+logndat*((1+1+1)*2+5)

    for ordact in range(1, ord_+1):
        logging.info('ordact = ' + str(ordact))
        for k in range(1, ordact):
            i = ordact
            for j in range(1, ordact+1):
                logging.info(str(i) + str(j) + str(k) + ':i')
                if i+j+k == 3:
                    continue
                elif ((j == 1) & (k == 1) & (not any(par[i-1][j-1][k-1]))):
                    par0 = par[i-2][j-1][k-1]
                    NPA = (i-1)*2+1
                    par0 = np.concatenate([par0[0:NPA], [0.], [0.], par0[NPA:]])
                    par[i-1][j-1][k-1], llf[i-1, j-1, k-1] = fit(par0, data, n, [i, j, k])
                    bic[i-1, j-1, k-1] = -2*llf[i-1, j-1, k-1]+logndat*((i+k+j)*2+5)

                elif ((j == 1) & (not any(par[i-1][j-1][k-1]))):
                    par0 = par[i-1][j-1][k-2]
                    NPA = (i+j+k-1)*2+3
                    par0 = np.concatenate([par0[0:NPA], [0.], [0.], par0[NPA:]])
                    (par[i-1][j-1][k-1], llf[i-1, j-1, k-1]) = fit(par0, data, n, [i, j, k])
                    bic[i-1, j-1, k-1] = -2*llf[i-1, j-1, k-1]+logndat*((i+k+j)*2+5)

                elif not any(par[i-1][j-1][k-1]):
                    par0 = par[i-1][j-2][k-1]
                    NPA = (i+j-1)*2+2
                    par0 = np.concatenate([par0[0:NPA], [0.], [0.], par0[NPA:]])
                    (par[i-1][j-1][k-1], llf[i-1, j-1, k-1]) = fit(par0, data, n, [i, j, k])
                    bic[i-1, j-1, k-1] = -2*llf[i-1, j-1, k-1]+logndat*((i+k+j)*2+5)

            for i in range(ordact, 0, -1):
                logging.info(str(i) + str(j) + str(k) + ':j')
                if not any(par[i-1][j-1][k-1]):
                    par0 = par[i-1][j-2][k-1]
                    NPA = (i+j-1)*2+2
                    par0 = np.concatenate([par0[0:NPA], [0.], [0.], par0[NPA:]])
                    (par[i-1][j-1][k-1], llf[i-1, j-1, k-1]) = fit(par0, data, n, [i, j, k])
                    bic[i-1, j-1, k-1] = -2*llf[i-1, j-1, k-1]+logndat*((i+k+j)*2+5)

        k = ordact
        for i in range(1, ordact+1):
            for j in range(1, ordact+1):
                if ((k > 1) & (not any(par[i-1][j-1][k-1]))):
                    logging.info(str(i) + str(j) + str(k) + ':k')
                    par0 = par[i-1][j-1][k-2]
                    NPA = (i+j+k-1)*2+3
                    par0 = np.concatenate([par0[0:NPA], [0.], [0.], par0[NPA:]])
                    (par[i-1][j-1][k-1], llf[i-1, j-1, k-1]) = fit(par0, data, n, [i, j, k])
                    bic[i-1, j-1, k-1] = -2*llf[i-1, j-1, k-1]+logndat*((i+k+j)*2+5)

    return par, bic
