# -*- coding: utf-8 -*-

from __future__ import division
import numpy as np
from ttide.t_tide import t_tide, t_predic
from entrada_datos import establecer_cadencia_series, cadencia_df
import matplotlib.dates as dates
import pandas as pd
import matplotlib.pyplot as plt


def distribuir_datos(df, paso=np.nan, lim=[np.nan, np.nan]):
    """Funcion que distribuye los datos de forma uniforme

    Args:
        df: vector de datos
        paso: da el valor de truncamiento constante. Si no es constante, como en el caso de los periodos, esta entrada
        no se necesita.
        lim: son dos valores con los limites superior e inferior. Si no tiene alguna (o ambos) poner NaN en el
        correspondiente.

    Returns:
        Un vector con datos distribuidos

    """

    nomb = df.columns[0]
    if np.isnan(paso):
        c = np.unique(df)
    else:
        c = np.arange(np.min(df), np.max(df), paso)

    lim_int = c[0:-1] + np.diff(c)/2.
    if np.isnan(lim[0]):
        if np.isnan(paso):
            inf = np.hstack((lim_int[0]-(c[1]-c[0]), lim_int))
        else:
            inf = np.hstack((lim_int[0]-paso, lim_int))
    else:
        inf = np.hstack((lim[0], lim_int))

    if np.isnan(lim[1]):
        if np.isnan(paso):
            sup = np.hstack((lim_int, lim_int[-1]+(c[-1]-c[-2])))
        else:
            sup = np.hstack((lim_int, lim_int[-1]+paso))
    else:
        sup = np.hstack((lim_int, lim[1]))

    for i in range(0, len(lim_int)):
        id_ = np.where((df >= inf[i]) & (df < sup[i]))[0]
        if id_.size != 0:
            df.ix[id_, nomb] = np.random.uniform(inf[i], sup[i], len(id_))

    mask = df == lim[0]
    df[mask] = df[mask] + 1e-6

    mask = df == lim[1]
    df[mask] = df[mask] - 1e-6

    return df


def interp_distrib(umb_anio, df):
    """Funcion que realiza dos tareas:

    1. Interpola los datos del periodo con errores.
    2. Distribuye los datos, i.e. les agrega un ruido, para evitar problemas en los ajustes por maxima verosimilitud.

    Args:
        umb_anio: Umbral para descartar anos con menos porcentaje de numero de datos
        df: dataframe con la variable a analizar

    Returns:
        Un dataframe con la nueva serie

    """

    hora = df.index.hour
    an, counts = np.unique(df.index.year, return_counts=True)
    mask = counts/(24./np.remainder(hora[1]-hora[0], 24)*365.) >= umb_anio
    an = an[mask]

    id_ = list()
    for j in range(0, len(an)):
        id_.extend(np.where(df.index.year == an[j])[0])

    var_ = df.columns
    df = df[var_].iloc[id_]

    if ((var_ == 'hs') | (var_ == 'vv')):
        df = distribuir_datos(df, .1, [0., np.nan])
    elif var_ == 'tp':
        df = distribuir_datos(df)
    elif ((var_ == 'dh') | (var_ == 'dv')):
        df = distribuir_datos(df, 1., [0., 360.])
    elif ((var_ == 'niv') | (var_ == 'mme')):
        df = distribuir_datos(df, .005, [0., np.nan])

    return df


def niv2ma_mm(df, name_eta='niv', latitud=36):
    """ Como analisis previo, extrae las componentes armonicas (marea astronomica de la marea) de la señal de nivel.
    Rellena los huecos para mejorar el analisis y lo repite.

    Args:
        amdh: fecha en formato juliano
        niv: nivel del mar
        latitud: si no se especifica se asume 36N

    Returns:
        Una tupla de la forma:

        - ``nameu``: nombre de la constituyentes principales
        - ``fu``: frecuencias de las constituyentes
        - ``tideconout``: amplitud de las constituyentes
        - ``mm``: residuo meteorologico
        - ``mm_min``: nivel minimo del residuo

    """
    df2 = df.copy()
    if len(np.unique(df2.index.year)) > 18:
        df2 = df2[df2.index.year >= df2.index.year.max() - 17]
    df2 = df2.dropna()

    # Serie interpolada
    cads, cad = cadencia_df(df2)
    if cad > 3600:
        df_int = establecer_cadencia_series(df2, 3600, 10)
    else:
        df_int = establecer_cadencia_series(df2, cad, 10)

    # Analisis armonico
    df_int.dropna(inplace=True)
    nivint = df_int[name_eta].resample('H').interpolate()
    stim = dates.date2num(df_int.index[0]) - 366
    nameu, fu, tideconout, nivout = t_tide(nivint.values, dt=cad/3600, stime=stim, lat=latitud)
    df_out1 = pd.DataFrame(nivout, columns=[name_eta], index=nivint.index)
    plt.figure()
    plt.plot(df_int)
    plt.plot(nivint.index, nivout+np.mean(nivint), '.')

    # Sustituye la salida de ttide donde no hay datos
    df2_reind = df2.reindex(df_int.index)
    df2_new = df2_reind.combine_first(df_out1)

    # ANALISIS ARMONICO DE LA NUEVA SERIE
    nivint2 = df2_new[name_eta].values
    stim = dates.date2num(df2_new.index[0]) - 366
    nameu, fu, tideconout, nivout2 = t_tide(nivint2, dt=cad/3600, stime=stim, lat=latitud)

    # SALIDA CON TIEMPO ORIGINAL
    df_niv = df.copy()
    df_niv = df_niv.dropna()

    n = np.array(df_niv.index.map(dates.date2num), dtype=np.float64)
    mast = t_predic(n, nameu, fu, tideconout, lat=latitud)

    df_niv['mas'] = mast + df_niv[name_eta].mean()
    df_niv['mme'] = df_niv[name_eta] - df_niv['mas']

    df_mmet = df_niv.drop([name_eta, 'mas'], 1)
    df_mmas = df_niv.drop([name_eta, 'mme'], 1)

    MM = nivint2 - nivout2[:, 0]

    return nameu, fu, tideconout, MM - np.min(MM), np.min(MM), df_niv[name_eta].mean(), df_niv, df_mmas, df_mmet
