# -*- coding: utf-8 -*-
"""
Created on Thu Jun 26 16:40:39 2014

@author: Pedro Folgueras
"""
import numpy as np
import scipy as sp
import itertools
from collections import deque
# %%
def FDS(x, dt = 1, k = 20, nh = 1, y = [np.nan]):
    """Cálculo de funciones de densidad espectral y funciones de coherencia
            Entradas:
                x, y: series temporales a analizar
                
                : resolución temporal de las series
                k = nº de grados de libertad Chi2
                nh = número de hojas del espectro
            Salidas:
                Sxx, Syy: funciones de densidad espectral de x e y
                Sxy: función de densidad espectral cruzada entre x e y
                Cxy: función de coherencia entre x e y
                f: vector de frecuencias
    """
    N = len(x)
    T = N*dt
    N2 = nextpow2(N)
    L = 2**N2-N
    Be = k/2/T                          # Ancho de ventana efectivo
    ancho = Be*T*(N+L)/N                # Ancho de la ventana para N+L datos
    n = int(np.ceil((ancho - 1)/2))     # Número de datos en cada mitad de la ventana
    
    zL = np.zeros(int(L))
    zn = np.zeros(n)
    
    f   = np.linspace(0, 1, (N+L)/2)/dt/2 
       
    if len(y) == 1:
        xmean = np.mean(x)
        x_z = np.append(x-xmean, zL)
        
        X = np.fft.fft(x_z)/(N+L)
        
        sxx = np.conj(X)*X
        Sxx_ = (N+L)**2*dt*sxx.real/2/np.pi/N
        Sxx_ = np.hstack((zn, Sxx_, zn))
        
        # Se promedia en el ancho de banda
        Sxx2 = np.array(list(movingaverage(Sxx_.tolist(), 2*n+1, data_is_list = True,
		avoid_fp_drift = False)))

        Sxx2 = np.fft.fftshift(Sxx2)
        Sxx = 4*np.pi*Sxx2[int(-(N+L)/2):]
        
        if nh == 2:
            Sxx = Sxx2
            f   = np.linspace(-1, 1, (N+L))*np.pi/dt
        
        return Sxx, f
      
    else:
        xmean = np.mean(x)        
        ymean = np.mean(y)
        x_z = np.append(x-xmean, zL)        
        y_z = np.append(y-ymean, zL)
        
        X = np.fft.fft(x_z)/(N+L)
        Y = np.fft.fft(y_z)/(N+L)
        
        sxx = np.conj(X)*X
        syy = np.conj(Y)*Y
        sxy = np.conj(X)*Y
        
        Sxx_ = (N+L)**2*dt*sxx.real/2/np.pi/N
        Syy_ = (N+L)**2*dt*syy.real/2/np.pi/N
        Sxy_ = (N+L)**2*dt*sxy/2/np.pi/N
        
        Sxx_ = np.hstack((zn, Sxx_, zn))
        Syy_ = np.hstack((zn, Syy_, zn))
        Sxy_ = np.hstack((zn, Sxy_, zn))
        
        # Se promedia en el ancho de banda
        Sxx2 = np.array(list(movingaverage(Sxx_.tolist(), 2*n+1, data_is_list = True,
		avoid_fp_drift = False)))
        Syy2 = np.array(list(movingaverage(Syy_.tolist(), 2*n+1, data_is_list = True,
		avoid_fp_drift = False)))
        Sxy2R = np.array(list(movingaverage(Sxy_.real.tolist(), 2*n+1, data_is_list = True,
		avoid_fp_drift = False)))
        Sxy2I = np.array(list(movingaverage(Sxy_.imag.tolist(), 2*n+1, data_is_list = True,
		avoid_fp_drift = False)))
        Sxy2 = Sxy2R + Sxy2I*1j
  
 
        Sxx2 = np.fft.fftshift(Sxx2)
        Syy2 = np.fft.fftshift(Syy2)
        Sxy2 = np.fft.fftshift(Sxy2)
        
        Sxx = 4*np.pi*Sxx2[int(-(N+L)/2):]
        Syy = 4*np.pi*Syy2[int(-(N+L)/2):]
        Sxy = 4*np.pi*Sxy2[int(-(N+L)/2):]
        
        
        if nh == 2:
            Sxx = Sxx2
            Syy = Syy2
            Sxy = Sxy2
            f   = np.linspace(-1, 1, (N+L))*np.pi/dt
            
        Syx = np.conj(Sxy)
        Cxy = Sxy*Syx/Sxx/Syy    
           
        return Sxx, Syy, Sxy, Cxy, f

# %%
def get_noise_freq_domain_CovarMatrix(comatrix , df , inittime , parityN , seed='none' , N_previous_draws=0):
    """                                                                                                          
    returns the noise time-series given their covariance matrix                                                  
    INPUT:                                                                                                       
        comatrix --- covariance matrix, Nts x Nts x Nf numpy array                                                   
            Nts = number of time-series. 
            Nf number of positive and non-Nyquist frequencies                     
        df --- frequency resolution
        inittime --- initial time of the noise time-series                                                           
        parityN --- is the length of the time-series 'Odd' or 'Even'                                                 
        seed --- seed for the random number generator                                                                
        N_previous_draws --- number of random number draws to discard first                                          
    OUPUT:                                                                                                       
        t --- time [s]                                                                                               
        n --- noise time-series, Nts x N numpy array                                                                 
    """
    if len( comatrix.shape ) != 3 :
       raise NameError('InputError: Input Covariance matrices must be a 3-D numpy array!')
    if comatrix.shape[0]  != comatrix.shape[1] :
        raise NameError('InputError: Covariance matrix must be square at each frequency!')

    Nts , Nf = comatrix.shape[0] , comatrix.shape[2]

    if parityN == 'Odd' :
        N = 2 * Nf + 1
    elif parityN == 'Even' :
        N = 2 * ( Nf + 1 )
    else :
        raise NameError('InputError: "parityN must be either "Odd" or "Even"!')
    stime = 1 / ( N*df )
    t = inittime + stime * np.arange( N )

    if seed == 'none' :
        print 'Not setting the seed for np.random.standard_normal()'
        pass
    elif seed == 'random' :
        np.random.seed( None )
    else :
        np.random.seed( int( seed ) )
    print N_previous_draws
    np.random.standard_normal( N_previous_draws ) ;

    zs = np.array( [ ( np.random.standard_normal((Nf,)) + 1j * np.random.standard_normal((Nf,)) ) / np.sqrt(2)
                 for i in range( Nts ) ] )

    ntilde_p = np.zeros( ( Nts , Nf ) , dtype=complex )
    for k in range( Nf ) :
        C = comatrix[ :,:,k ]
        if not np.allclose( C , np.conj( np.transpose( C ) ) ) :
            print "Covariance matrix NOT Hermitian! Unphysical."
        w , V = np.linalg.eigh( C )
        for m in range( w.shape[0] ) :
            w[m] = np.real( w[m] )
            if np.abs(w[m]) / np.max(w) < 1e-10 :
                w[m] = 0
            if w[m] < 0 :
                print 'Negative eigenvalue! Simulating unpysical signal...'

        ntilde_p[ :,k ] =  np.conj( np.sqrt( N / (2*stime) ) * np.dot( V , np.dot( np.sqrt( np.diag( w ) ) , zs[ :,k ] ) ) )

    zerofill = np.zeros( ( Nts , 1 ) )
    if N % 2 == 0 :
        ntilde = np.concatenate( ( zerofill , ntilde_p , zerofill , np.conj(np.fliplr(ntilde_p)) ) , axis = 1 )
    else :
        ntilde = np.concatenate( ( zerofill , ntilde_p , np.conj(np.fliplr(ntilde_p)) ) , axis = 1 )
    n = np.real( sp.ifft( ntilde , axis = 1 ) )
    return t , n
    
# %%
def clearall():
    """clear all globals"""
    for uniquevar in [var for var in globals().copy() if var[0] != "_" and var != 'clearall']:
        del globals()[uniquevar]
        
# %%      
def nextpow2(n):
    """calcula el exponente de la siguiente potencia de 2"""

    m_f = np.log2(n)
    m_i = np.ceil(m_f)
    return m_i        
    
# %% 
      #  Sean Reifschneider, tummy.com, ltd.  <jafo@tummy.com>
      #  Released into the Public Domain, 2011-02-06
def movingaverage(data, subset_size, data_is_list = None,
		avoid_fp_drift = True):

	'''Return the moving averages of the data, with a window size of
	`subset_size`.  `subset_size` must be an integer greater than 0 and
	less than the length of the input data, or a ValueError will be raised.

	`data_is_list` can be used to tune the algorithm for list or iteratable
	as an input.  The default value, `None` will auto-detect this.
	The algorithm used if `data` is a list is almost twice as fast as if
	it is an iteratable.

	`avoid_fp_drift`, if True (the default) sums every sub-set rather than
	keeping a "rolling sum" (which may be subject to floating-point drift).
	While more correct, it is also dramatically slower for subset sizes
	much larger than 20.

	NOTE: You really should consider setting `avoid_fp_drift = False` unless
	you are dealing with very small numbers (say, far smaller than 0.00001)
	or require extreme accuracy at the cost of execution time.  For
	`subset_size` < 20, the performance difference is very small.
	'''
	if subset_size < 1:
		raise ValueError('subset_size must be 1 or larger')

	if data_is_list is None:
		data_is_list = hasattr(data, '__getslice__')

	divisor = float(subset_size)
	if data_is_list:
		#  This only works if we can re-access old elements, but is much faster.
		#  In other words, it can't be just an iterable, it needs to be a list.

		if subset_size > len(data):
			raise ValueError('subset_size must be smaller than data set size')

		if avoid_fp_drift:
			for x in range(subset_size, len(data) + 1):
				yield sum(data[x - subset_size:x]) / divisor
		else:
			cur = sum(data[0:subset_size])
			yield cur / divisor
			for x in range(subset_size, len(data)):
				cur += data[x] - data[x - subset_size]
				yield cur / divisor
	else:
		#  Based on the recipe at:
		#     http://docs.python.org/library/collections.html#deque-recipes
		it = iter(data)
		d = deque(itertools.islice(it, subset_size))

		if subset_size > len(d):
			raise ValueError('subset_size must be smaller than data set size')

		if avoid_fp_drift:
			yield sum(d) / divisor
			for elem in it:
				d.popleft()
				d.append(elem)
				yield sum(d) / divisor
		else:
			s = sum(d)
			yield s / divisor
			for elem in it:
				s += elem - d.popleft()
				d.append(elem)
				yield s / divisor


##########################
if __name__ == '__main__':
	import unittest

	class TestMovingAverage(unittest.TestCase):
		####################
		def test_List(self):
			try:
				list(movingaverage([1,2,3], 0))
				self.fail('Did not raise ValueError on subset_size=0')
			except ValueError:
				pass

			try:
				list(movingaverage([1,2,3,4,5,6], 7))
				self.fail('Did not raise ValueError on subset_size > len(data)')
			except ValueError:
				pass

			self.assertEqual(list(movingaverage([1,2,3,4,5,6], 1)), [1,2,3,4,5,6])
			self.assertEqual(list(movingaverage([1,2,3,4,5,6], 2)),
					[1.5,2.5,3.5,4.5,5.5])
			self.assertEqual(list(movingaverage(map(float, [1,2,3,4,5,6]), 2)),
					[1.5,2.5,3.5,4.5,5.5])
			self.assertEqual(list(movingaverage([1,2,3,4,5,6], 3)), [2,3,4,5])
			self.assertEqual(list(movingaverage([1,2,3,4,5,6], 4)), [2.5,3.5,4.5])
			self.assertEqual(list(movingaverage([1,2,3,4,5,6], 5)), [3,4])
			self.assertEqual(list(movingaverage([1,2,3,4,5,6], 6)), [3.5])

			self.assertEqual(list(movingaverage([40, 30, 50, 46, 39, 44],
					3, False)), [40.0,42.0,45.0,43.0])
			self.assertEqual(list(movingaverage([40, 30, 50, 46, 39, 44],
					3, True)), [40.0,42.0,45.0,43.0])


		######################
		def test_XRange(self):
			try:
				list(movingaverage(xrange(1, 4), 0))
				self.fail('Did not raise ValueError on subset_size=0')
			except ValueError:
				pass

			try:
				list(movingaverage(xrange(1, 7), 7))
				self.fail('Did not raise ValueError on subset_size > len(data)')
			except ValueError:
				pass

			self.assertEqual(list(movingaverage(xrange(1, 7), 1)), [1,2,3,4,5,6])
			self.assertEqual(list(movingaverage(xrange(1, 7), 2)),
					[1.5,2.5,3.5,4.5,5.5])
			self.assertEqual(list(movingaverage(iter(map(float, xrange(1, 7))),
					2)), [1.5,2.5,3.5,4.5,5.5])
			self.assertEqual(list(movingaverage(xrange(1, 7), 3)), [2,3,4,5])
			self.assertEqual(list(movingaverage(xrange(1, 7), 4)), [2.5,3.5,4.5])
			self.assertEqual(list(movingaverage(xrange(1, 7), 5)), [3,4])
			self.assertEqual(list(movingaverage(xrange(1, 7), 6)), [3.5])


		###########################
		def test_ListRolling(self):
			try:
				list(movingaverage([1,2,3], 0, avoid_fp_drift = False))
				self.fail('Did not raise ValueError on subset_size=0')
			except ValueError:
				pass

			try:
				list(movingaverage([1,2,3,4,5,6], 7, avoid_fp_drift = False))
				self.fail('Did not raise ValueError on subset_size > len(data)')
			except ValueError:
				pass

			self.assertEqual(list(movingaverage([1,2,3,4,5,6], 1,
					avoid_fp_drift = False)), [1,2,3,4,5,6])
			self.assertEqual(list(movingaverage([1,2,3,4,5,6], 2,
					avoid_fp_drift = False)),
					[1.5,2.5,3.5,4.5,5.5])
			self.assertEqual(list(movingaverage(map(float, [1,2,3,4,5,6]), 2,
					avoid_fp_drift = False)), [1.5,2.5,3.5,4.5,5.5])
			self.assertEqual(list(movingaverage([1,2,3,4,5,6], 3,
					avoid_fp_drift = False)), [2,3,4,5])
			self.assertEqual(list(movingaverage([1,2,3,4,5,6], 4,
					avoid_fp_drift = False)), [2.5,3.5,4.5])
			self.assertEqual(list(movingaverage([1,2,3,4,5,6], 5,
					avoid_fp_drift = False)), [3,4])
			self.assertEqual(list(movingaverage([1,2,3,4,5,6], 6,
					avoid_fp_drift = False)), [3.5])

			self.assertEqual(list(movingaverage([40, 30, 50, 46, 39, 44],
					3, False, avoid_fp_drift = False)), [40.0,42.0,45.0,43.0])
			self.assertEqual(list(movingaverage([40, 30, 50, 46, 39, 44],
					3, True, avoid_fp_drift = False)), [40.0,42.0,45.0,43.0])


		#############################
		def test_XRangeRolling(self):
			try:
				list(movingaverage(xrange(1, 4), 0, avoid_fp_drift = False))
				self.fail('Did not raise ValueError on subset_size=0')
			except ValueError:
				pass

			try:
				list(movingaverage(xrange(1, 7), 7, avoid_fp_drift = False))
				self.fail('Did not raise ValueError on subset_size > len(data)')
			except ValueError:
				pass

			self.assertEqual(list(movingaverage(xrange(1, 7), 1,
					avoid_fp_drift = False)), [1,2,3,4,5,6])
			self.assertEqual(list(movingaverage(xrange(1, 7), 2,
					avoid_fp_drift = False)), [1.5,2.5,3.5,4.5,5.5])
			self.assertEqual(list(movingaverage(iter(map(float, xrange(1, 7))),
					2, avoid_fp_drift = False)), [1.5,2.5,3.5,4.5,5.5])
			self.assertEqual(list(movingaverage(xrange(1, 7), 3,
					avoid_fp_drift = False)), [2,3,4,5])
			self.assertEqual(list(movingaverage(xrange(1, 7), 4,
					avoid_fp_drift = False)), [2.5,3.5,4.5])
			self.assertEqual(list(movingaverage(xrange(1, 7), 5,
					avoid_fp_drift = False)), [3,4])
			self.assertEqual(list(movingaverage(xrange(1, 7), 6,
					avoid_fp_drift = False)), [3.5])


	######################################################################
	suite = unittest.TestLoader().loadTestsFromTestCase(TestMovingAverage)
	unittest.TextTestRunner(verbosity = 2).run(suite)    