# -*- coding: utf-8 -*-

import six
import pandas as pd
import os
import numpy as np
import re
import colorsys
import matplotlib.pyplot as plt
import matplotlib.colors as colors
from datetime import datetime


def lectura_wana(ruta, nombre_fichero):
    """Funcion que analiza el fichero de entrada tipo WANA y devuelve un dataframe de oleaje y otro atmosférico.

    Args:
        ruta: directorio (ruta absoluta) del fichero de entrada
        nombre_fichero: fichero del la informacion climatica en emplazamiento.
            debe incluir en el cabecero la linea AA / MM / DD ...)

    Returns:
        Una tupla con:

        * ``df_oleaje``: dataframe de oleaje con hs (altura de ola significante), tp (periodo de pico espectral) y dh
        (direccion media del oleaje).
        * ``df_viento``: dataframe de viento con vv (velocidad de viento) y dv (direccion de viento).

    """

    numskip, has_header, cod = get_line_number(os.path.join(ruta, nombre_fichero))

    data = pd.read_table(os.path.join(ruta, nombre_fichero), delim_whitespace=True,
                         parse_dates=[[0, 1, 2, 3]], skiprows=numskip, index_col=0, header=has_header)
    # data.set_index(pd.datetimeIndex(data.NORIG), inplace=True)
    data = data.drop(['Tm02', 'Hm0_V', 'DirM_V', 'Hm0_F1', 'Tm02_F1', 'DirM_F1',
                      'Hm0_F2', 'Tm02_F2', 'DirM_F2'], 1)
    data.replace(-99.9, np.nan, inplace=True)
    data.replace(-99.99, np.nan, inplace=True)
    data.replace(-9999, np.nan, inplace=True)
    data.replace(990, np.nan, inplace=True)

    df_oleaje = data.drop(['VelV', 'DirV'], 1)
    df_oleaje = df_oleaje.rename(columns={'Hm0': 'hs', 'Tp': 'tp', 'DirM': 'dh'})
    df_hs = df_oleaje.drop(['tp', 'dh'], 1)
    df_tp = df_oleaje.drop(['hs', 'dh'], 1)
    df_dh = df_oleaje.drop(['tp', 'hs'], 1)

    df_viento = data.drop(['Hm0', 'Tp', 'DirM'], 1)
    df_viento = df_viento.rename(columns={'VelV': 'vv', 'DirV': 'dv'})
    df_vv = df_viento.drop(['dv'], 1)
    df_dv = df_viento.drop(['vv'], 1)

    # TODO: devolver tambien el codigo (cod) que se recupera (tambien para todas las funciones siguientes)
    return df_hs, df_tp, df_dh, df_vv, df_dv


def lectura_simar(ruta, nombre_fichero):
    """Funcion que analiza el fichero de entrada tipo SIMAR y devuelve un dataframe de oleaje y otro atmosférico.

    Args:
        ruta: directorio (ruta absoluta) del fichero de entrada
        nombre_fichero: fichero del la informacion climatica en emplazamiento.
            debe incluir en el cabecero la linea AA / MM / DD ...

    Returns:
        Una tupla con:

        * ``df_oleaje``: dataframe de oleaje con hs (altura de ola significante), tp (periodo de pico espectral) y dh
        (direccion media del oleaje).
        * ``df_viento``: dataframe de viento con vv (velocidad de viento) y dv (direccion de viento).

    """

    numskip, has_header, cod = get_line_number(os.path.join(ruta, nombre_fichero))

    data = pd.read_table(os.path.join(ruta, nombre_fichero), delim_whitespace=True,
                         parse_dates=[[0, 1, 2, 3]], skiprows=numskip, index_col=0, header=has_header)
    # data.set_index(pd.datetimeIndex(data.NORIG), inplace=True)
    data = data.drop(['Tm02', 'Hm0_V', 'DirM_V', 'Hm0_F1', 'Tm02_F1', 'DirM_F1',
                      'Hm0_F2', 'Tm02_F2', 'DirM_F2'], 1)
    data.replace(-99.9, np.nan, inplace=True)
    data.replace(-99.99, np.nan, inplace=True)
    data.replace(-9999, np.nan, inplace=True)
    data.replace(990, np.nan, inplace=True)

    df_oleaje = data.drop(['VelV', 'DirV'], 1)
    df_oleaje = df_oleaje.rename(columns={'Hm0': 'hs', 'Tp': 'tp', 'DirM': 'dh'})
    df_hs = df_oleaje.drop(['tp', 'dh'], 1)
    df_tp = df_oleaje.drop(['hs', 'dh'], 1)
    df_dh = df_oleaje.drop(['tp', 'hs'], 1)

    df_viento = data.drop(['Hm0', 'Tp', 'DirM'], 1)
    df_viento = df_viento.rename(columns={'VelV': 'vv', 'DirV': 'dv'})
    df_vv = df_viento.drop(['dv'], 1)
    df_dv = df_viento.drop(['vv'], 1)

    # return cod, df_oleaje, df_viento
    return df_hs, df_tp, df_dh, df_vv, df_dv


def lectura_simar_dfoleaje(ruta, nombre_fichero):
    """Funcion que analiza el fichero de entrada tipo SIMAR y devuelve un dataframe de oleaje y otro atmosférico.

    Args:
        ruta: directorio (ruta absoluta) del fichero de entrada
        nombre_fichero: fichero del la informacion climatica en emplazamiento.
            debe incluir en el cabecero la linea AA / MM / DD ...

    Returns:
        Una tupla con:

        * ``df_oleaje``: dataframe de oleaje con hs (altura de ola significante), tp (periodo de pico espectral) y dh
        (direccion media del oleaje).
        * ``df_viento``: dataframe de viento con vv (velocidad de viento) y dv (direccion de viento).

    """

    numskip, has_header, cod = get_line_number(os.path.join(ruta, nombre_fichero))

    data = pd.read_table(os.path.join(ruta, nombre_fichero), delim_whitespace=True,
                         parse_dates=[[0, 1, 2, 3]], skiprows=numskip, index_col=0, header=has_header)
    # data.set_index(pd.datetimeIndex(data.NORIG), inplace=True)
    data = data.drop(['Tp', 'Hm0_V', 'DirM_V', 'Hm0_F1', 'Tm02_F1', 'DirM_F1',
                      'Hm0_F2', 'Tm02_F2', 'DirM_F2'], 1)
    data.replace(-99.9, np.nan, inplace=True)
    data.replace(-99.99, np.nan, inplace=True)
    data.replace(-9999, np.nan, inplace=True)
    data.replace(990, np.nan, inplace=True)

    df_oleaje = data.drop(['VelV', 'DirV'], 1)
    df_oleaje = df_oleaje.rename(columns={'Hm0': 'hs', 'Tm02': 'tm', 'DirM': 'dh'})
    df_oleaje = df_oleaje.drop(['dh'], 1)
    # df_hs = df_oleaje.drop(['tm', 'dh'], 1)
    # df_tm = df_oleaje.drop(['hs', 'dh'], 1)
    # df_dh = df_oleaje.drop(['tm', 'hs'], 1)
    #
    # df_viento = data.drop(['Hm0', 'Tm02', 'DirM'], 1)
    # df_viento = df_viento.rename(columns={'VelV': 'vv', 'DirV': 'dv'})
    # df_vv = df_viento.drop(['dv'], 1)
    # df_dv = df_viento.drop(['vv'], 1)

    return df_oleaje #df_viento
    # return df_hs, df_tm#, df_dh, df_vv, df_dv

def lectura_redcos(ruta, nombre_fichero):
    """Funcion que analiza el fichero de entrada de boya de la red REDCOS y devuelve un dataframe de oleaje.

    Args:
        ruta: directorio (ruta absoluta) del fichero de entrada
        nombre_fichero: fichero del la informacion climatica en emplazamiento.
            debe incluir en el cabecero la linea AA / MM / DD ...

    Returns:
        ``df_oleaje``: dataframe de oleaje con hs (altura de ola significante), tp (periodo de pico espectral) y dh
        (direccion media del oleaje).

    """

    numskip, has_header, cod = get_line_number(os.path.join(ruta, nombre_fichero))

    data = pd.read_table(os.path.join(ruta, nombre_fichero), delim_whitespace=True,
                         parse_dates=[[0, 1, 2, 3]], skiprows=numskip, index_col=0, header=has_header)
    # data.set_index(pd.datetimeIndex(data.NORIG), inplace=True)
    data = data[data.Qc_e != 3]
    data = data.sort_values('Pro_Oe').groupby(level=0).last()
    data = data.drop(['H13', 'Tm', 'Tm02', 'Hmax', 'Thmax', 'Pro_Oe',
                      'Dmd_P', 'Ds', 'Ds_P', 'Pro_Od', 'Qc_e', 'Ts'], 1)
    data.replace(-99.9, np.nan, inplace=True)
    data.replace(-99.99, np.nan, inplace=True)
    data.replace(-9999, np.nan, inplace=True)
    data.replace(990, np.nan, inplace=True)

    df_oleaje = data.rename(columns={'Hm0': 'hs', 'Tp': 'tp', 'Dmd': 'dh'})
    df_hs = df_oleaje.drop(['tp', 'dh'], 1)
    df_tp = df_oleaje.drop(['hs', 'dh'], 1)
    df_dh = df_oleaje.drop(['tp', 'hs'], 1)

    # return cod, df_oleaje
    return df_hs, df_tp, df_dh


def lectura_redcos_dfoleaje(ruta, nombre_fichero):
    """Funcion que analiza el fichero de entrada de boya de la red REDCOS y devuelve un dataframe de oleaje.

    Args:
        ruta: directorio (ruta absoluta) del fichero de entrada
        nombre_fichero: fichero del la informacion climatica en emplazamiento.
            debe incluir en el cabecero la linea AA / MM / DD ...

    Returns:
        ``df_oleaje``: dataframe de oleaje con hs (altura de ola significante), tp (periodo de pico espectral) y dh
        (direccion media del oleaje).

    """

    numskip, has_header, cod = get_line_number(os.path.join(ruta, nombre_fichero))

    data = pd.read_table(os.path.join(ruta, nombre_fichero), delim_whitespace=True,
                         parse_dates=[[0, 1, 2, 3]], skiprows=numskip, index_col=0, header=has_header)
    # data.set_index(pd.datetimeIndex(data.NORIG), inplace=True)
    data = data[data.Qc_e != 3]
    data = data.sort_values('Pro_Oe').groupby(level=0).last()
    data = data.drop(['H13', 'Tm', 'Tm02', 'Hmax', 'Thmax', 'Pro_Oe',
                      'Dmd_P', 'Ds', 'Ds_P', 'Pro_Od', 'Qc_e', 'Ts'], 1)
    data.replace(-99.9, np.nan, inplace=True)
    data.replace(-99.99, np.nan, inplace=True)
    data.replace(-9999, np.nan, inplace=True)
    data.replace(990, np.nan, inplace=True)

    df_oleaje = data.rename(columns={'Hm0': 'hs', 'Tp': 'tp', 'Dmd': 'dh'})
    df_hs = df_oleaje.drop(['tp', 'dh'], 1)
    df_tp = df_oleaje.drop(['hs', 'dh'], 1)
    df_dh = df_oleaje.drop(['tp', 'hs'], 1)

    return df_oleaje
    # return df_hs, df_tp, df_dh


def lectura_remro(ruta, nombre_fichero):
    """Funcion que analiza el fichero de entrada de boya de la red REMRO y devuelve un dataframe de oleaje.

    Args:
        ruta: directorio (ruta absoluta) del fichero de entrada
        nombre_fichero: fichero del la informacion climatica en emplazamiento.
            debe incluir en el cabecero la linea AA / MM / DD ...

    Returns:
        ``df_oleaje``: dataframe de oleaje con hs (altura de ola significante), tp (periodo de pico espectral) y dh
        (direccion media del oleaje).

    """

    numskip, has_header, cod = get_line_number(os.path.join(ruta, nombre_fichero))

    data = pd.read_table(os.path.join(ruta, nombre_fichero), delim_whitespace=True,
                         parse_dates=[[0, 1, 2, 3]], skiprows=numskip, index_col=0, header=has_header)
    # data.set_index(pd.datetimeIndex(data.NORIG), inplace=True)
    data = data[data.Qc_e != 3]
    data = data.sort_values('Pro_Oe').groupby(level=0).last()
    data = data.drop(['H13', 'Tm', 'Tm02', 'Hmax', 'Thmax', 'Pro_Oe', 'Qc_e'], 1)
    data.replace(-99.9, np.nan, inplace=True)
    data.replace(-99.99, np.nan, inplace=True)
    data.replace(-9999, np.nan, inplace=True)
    data.replace(990, np.nan, inplace=True)

    df_oleaje = data.rename(columns={'Hm0': 'hs', 'Tp': 'tp'})
    df_hs = df_oleaje.drop(['tp', 'dh'], 1)
    df_tp = df_oleaje.drop(['hs', 'dh'], 1)

    # return cod, df_oleaje
    return df_hs, df_tp


def lectura_redext(ruta, nombre_fichero):
    """Funcion que analiza el fichero de entrada de boya de la red REDEXT y devuelve un dataframe de oleaje y otro atmosférico.

    Args:
        ruta: directorio (ruta absoluta) del fichero de entrada
        nombre_fichero: fichero del la informacion climatica en emplazamiento.
            debe incluir en el cabecero la linea AA / MM / DD ...

    Returns:
        ``df_oleaje``: dataframe de oleaje con hs (altura de ola significante), tp (periodo de pico espectral) y dh
        (direccion media del oleaje).

    """

    numskip, has_header, cod = get_line_number(os.path.join(ruta, nombre_fichero))

    data = pd.read_table(os.path.join(ruta, nombre_fichero), delim_whitespace=True,
                         parse_dates=[[0, 1, 2, 3]], skiprows=numskip, index_col=0, header=has_header)
    # data.set_index(pd.datetimeIndex(data.NORIG), inplace=True)
    data = data[data.Qc_e != 3]
    data = data.sort_values('Pro_Oe').groupby(level=0).last()
    data = data.drop(['Tm02', 'Hmax', 'Thmax', 'Pro_Oe', 'Dmd_P', 'Ds_P', 'Pro_Od', 'Ts2', 'Sa2', 'Vc_md', 'Dc_md',
                      'Pro_Oce', 'Ta', 'Pro_Met', 'Qc_e'], 1)
    data.replace(-99.9, np.nan, inplace=True)
    data.replace(-99.99, np.nan, inplace=True)
    data.replace(-9999, np.nan, inplace=True)
    data.replace(990, np.nan, inplace=True)

    df_oleaje = data.drop(['Ps', 'Vv_md', 'Dv_md'], 1)
    df_oleaje = df_oleaje.rename(columns={'Hm0': 'hs', 'Tp': 'tp', 'Dmd': 'dh'})
    df_hs = df_oleaje.drop(['tp', 'dh'], 1)
    df_tp = df_oleaje.drop(['hs', 'dh'], 1)
    df_dh = df_oleaje.drop(['tp', 'hs'], 1)

    df_viento = data.drop(['Hm0', 'Tp', 'Dmd'], 1)
    df_viento = df_viento.rename(columns={'Ps': 'pa', 'Vv_md': 'vv', 'Dv_md': 'dv'})
    df_vv = df_viento.drop(['dv','pa'], 1)
    df_dv = df_viento.drop(['vv', 'pa'], 1)
    df_pa = df_viento.drop(['vv', 'dv'], 1)

    # return cod, df_oleaje, df_viento
    return df_hs, df_tp, df_dh, df_vv, df_dv, df_pa


def lectura_rempor(ruta, nombre_fichero):
    """Funcion que analiza el fichero de entrada tipo REMPOR y devuelve un dataframe de viento.

    Args:
        ruta: directorio (ruta absoluta) del fichero de entrada
        nombre_fichero: fichero del la informacion climatica en emplazamiento.
            debe incluir en el cabecero la linea AA / MM / DD ...

    Returns:
        Una tupla con:

        * ``df_viento``: dataframe de viento con vv (velocidad de viento) y dv (direccion de viento).

    """

    numskip, has_header, cod = get_line_number(os.path.join(ruta, nombre_fichero))

    data = pd.read_table(os.path.join(ruta, nombre_fichero), delim_whitespace=True,
                         parse_dates=[[0, 1, 2, 3]], skiprows=numskip, index_col=0, header=has_header)
    # data.set_index(pd.datetimeIndex(data.NORIG), inplace=True)
    df_viento = data.drop(['Vv_mx', 'Dv_mx', 'Vv_sg', 'Dv_sg'], 1)
    data.replace(-99.9, np.nan, inplace=True)
    data.replace(-99.99, np.nan, inplace=True)
    data.replace(-9999, np.nan, inplace=True)
    data.replace(990, np.nan, inplace=True)

    df_viento = df_viento.rename(columns={'Vv_md': 'vv', 'Dv_md': 'dv'})
    df_vv = df_viento.drop(['dv'], 1)
    df_dv = df_viento.drop(['vv'], 1)

    # return cod, df_oleaje, df_viento
    return df_vv, df_dv


def lectura_nivmar(ruta, nombre_fichero):
    """Funcion que analiza el fichero de entrada tipo NivMar y devuelve un dataframe de nivel de mar.

    Args:
        ruta: directorio (ruta absoluta) del fichero de entrada
        nombre_fichero: fichero del la informacion climatica en emplazamiento.
            debe incluir en el cabecero la linea YY / MM / DD ...

    Returns:
        Un dataframe de nivel de mar con niv (nivel medio del mar).

    """

    numskip, has_header, cod = get_line_number(os.path.join(ruta, nombre_fichero))

    data = pd.read_table(os.path.join(ruta, nombre_fichero), delim_whitespace=True,
                         parse_dates=[[0, 1, 2, 3, 4]], skiprows=numskip, index_col=0, header=has_header)
    # data.set_index(pd.datetimeIndex(data.NORIG), inplace=True)
    data = data.drop(['Ps', 'Vv_md', 'Dv_md', 'Mar', 'Res'], 1)
    data = data.rename(columns={'Nivel': 'niv'})
    data.niv /= 100
    data.replace(-99.9, np.nan, inplace=True)
    data.replace(-99.99, np.nan, inplace=True)
    data.replace(-9999, np.nan, inplace=True)
    data.replace(990, np.nan, inplace=True)

    # return cod, data
    return data


def lectura_mareografo(ruta, nombre_fichero):
    """Funcion que analiza el fichero de entrada tipo REDMAR y devuelve un dataframe de nivel de mar.

    Args:
        ruta: directorio (ruta absoluta) del fichero de entrada
        nombre_fichero: fichero del la informacion climatica en emplazamiento.
            debe incluir en el cabecero la linea YY / MM / DD ...


    Returns:
        Un dataframe de nivel de mar con niv (nivel medio del mar).

    """

    numskip, has_header, cod = get_line_number(os.path.join(ruta, nombre_fichero))

    data = pd.read_table(os.path.join(ruta, nombre_fichero), delim_whitespace=True,
                         parse_dates=[[0, 1, 2, 3]], skiprows=numskip, index_col=0, header=has_header)
    # data.set_index(pd.datetimeIndex(data.NORIG), inplace=True)
    data = data.drop(['Niv_00', 'Niv_05', 'Niv_10', 'Niv_15', 'Niv_20', 'Niv_25', 'Niv_30',
                      'Niv_35', 'Niv_40', 'Niv_45', 'Niv_50', 'Niv_55', 'Pro', 'Mar_H', 'Res_H'], 1)
    data = data.rename(columns={'Niv_H': 'niv'})
    data.niv /= 100
    data.replace(-99.9, np.nan, inplace=True)
    data.replace(-99.99, np.nan, inplace=True)
    data.replace(-9999, np.nan, inplace=True)
    data.replace(990, np.nan, inplace=True)

    # return cod, data
    return data


def lectura_viento(ruta, nombre_fichero):
    """Funcion que analiza el fichero de entrada de viento tipo ERA - WRF y devuelve un dataframe de viento.

    Args:
        ruta: directorio (ruta absoluta) del fichero de entrada
        nombre_fichero: fichero del la informacion climatica en emplazamiento.
            debe tener las columnas 'YY', 'MM', 'DD', 'hh', 'mm', 'ss', 'vv', 'dv'

    Returns:
        Un dataframe de viento con vv (velocidad de viento) y dv (direccion de viento).

    """
    data = pd.read_table(os.path.join(ruta, nombre_fichero), delim_whitespace=True, header=None)

    date_lambda = lambda x: '%04d-%02d-%02d %02d:%02d:%02d' % (x.iloc[0], x.iloc[1], x.iloc[2], x.iloc[3], x.iloc[4],
                                                               x.iloc[5])
    data_date_column = data.apply(date_lambda, axis=1)
    data.insert(0, 'date', pd.to_datetime(data_date_column))
    data.set_index(data.date, inplace=True)
    data.drop([0, 1, 2, 3, 4, 5, 'date'], 1, inplace=True)
    data.columns = ['vv', 'dv']
    data = data[(data.vv != 0) & (data.dv != 0)]
    data.replace(-99.9, np.nan, inplace=True)
    data.replace(-99.99, np.nan, inplace=True)
    data.replace(-9999, np.nan, inplace=True)
    data.replace(990, np.nan, inplace=True)

    df_vv = data.drop(['dv'], 1)
    df_dv = data.drop(['vv'], 1)

    return df_vv, df_dv


def lectura_generica(ruta, nombre_fichero):
    """Funcion que analiza el fichero de entrada generico y devuelve un dataframe de oleaje, viento y nivel de mar.

    Args:
        ruta: directorio (ruta absoluta) del fichero de entrada
        nombre_fichero: fichero del la informacion climatica en emplazamiento. Hs en metros, Tp en segundos, direcciones
            en grados, nivel del mar en metros.

    Returns:
        Una tupla con:

        * ``df_oleaje``: dataframe de oleaje con hs (altura de ola significante), tp (periodo de pico espectral) y dh
        (direccion media del oleaje).
        * ``df_viento``: dataframe de viento con vv (velocidad de viento) y dv (direccion de viento).
        * ``df_mar``: dataframe de nivel de mar con niv (nivel medio del mar)
    """

    data = pd.read_table(os.path.join(ruta, nombre_fichero), delim_whitespace=True, header=None)

    date_lambda = lambda x: '%04d-%02d-%02d %02d' % (x.iloc[0], x.iloc[1], x.iloc[2], x.iloc[3])
    data_date_column = data.apply(date_lambda, axis=1)
    data.insert(0, 'date', pd.to_datetime(data_date_column))
    data.set_index(data.date, inplace=True)
    data.drop([0, 1, 2, 3, 'date'], 1, inplace=True)
    data.columns = ['hs', 'tp', 'dh', 'vv', 'dv', 'niv']

    data.replace(-99.9, np.nan, inplace=True)
    data.replace(-99.99, np.nan, inplace=True)
    data.replace(-9999, np.nan, inplace=True)
    data.replace(990, np.nan, inplace=True)

    df_oleaje = data.drop(['vv', 'dv', 'niv'], 1)
    df_hs = df_oleaje.drop(['tp', 'dh'], 1)
    df_tp = df_oleaje.drop(['hs', 'dh'], 1)
    df_dh = df_oleaje.drop(['tp', 'hs'], 1)

    df_viento = data.drop(['hs', 'tp', 'dh', 'niv'], 1)
    df_vv = df_viento.drop(['dv'], 1)
    df_dv = df_viento.drop(['vv'], 1)

    df_niv = data.drop(['hs', 'tp', 'dh', 'vv', 'dv'], 1)
    # df_mar.niv /= 100

    return df_hs, df_tp, df_dh, df_vv, df_dv, df_niv


def lectura_propagacion_oleaje(ruta, nombre_fichero):
    """Funcion que analiza el fichero de entrada generico y devuelve un dataframe de oleaje.

    Args:
        ruta: directorio (ruta absoluta) del fichero de entrada
        nombre_fichero: fichero del la informacion climatica en emplazamiento. Hs en metros, Tp en segundos, direcciones
            en grados

    Returns:
        Un dataframe de oleaje con hs (altura de ola significante), tp (periodo de pico espectral) y dh
        (direccion media del oleaje).
    """

    data = pd.read_table(os.path.join(ruta, nombre_fichero), delim_whitespace=True, header=None)

    date_lambda = lambda x: '%04d-%02d-%02d %02d:%02d:%02d' % (x.iloc[0], x.iloc[1], x.iloc[2], x.iloc[3], x.iloc[4],
                                                               x.iloc[5])
    data_date_column = data.apply(date_lambda, axis=1)
    data.insert(0, 'date', pd.to_datetime(data_date_column))
    data.set_index(data.date, inplace=True)
    data.drop([0, 1, 2, 3, 4, 5, 'date'], 1, inplace=True)

    data.columns = ['hs', 'tp', 'dh']

    data.replace(-99.9, np.nan, inplace=True)
    data.replace(-99.99, np.nan, inplace=True)
    data.replace(-9999, np.nan, inplace=True)
    data.replace(990, np.nan, inplace=True)

    df_hs = data.drop(['tp', 'dh'], 1)
    df_tp = data.drop(['hs', 'dh'], 1)
    df_dh = data.drop(['tp', 'hs'], 1)

    return data, df_hs, df_tp, df_dh


def lectura_reconstr_marea(ruta, nombre_fichero):
    """Funcion que analiza el fichero de entrada generico y devuelve un dataframe de marea meteorologica.

    Args:
        ruta: directorio (ruta absoluta) del fichero de entrada
        nombre_fichero: fichero del la informacion climatica en emplazamiento.

    Returns:
        Un dataframe de oleaje con hs (altura de ola significante), tp (periodo de pico espectral) y dh
        (direccion media del oleaje).
    """

    data = pd.read_table(os.path.join(ruta, nombre_fichero), delim_whitespace=True, header=None)

    date_lambda = lambda x: '%04d-%02d-%02d %02d:%02d:%02d' % (x.iloc[0], x.iloc[1], x.iloc[2], x.iloc[3], x.iloc[4],
                                                               x.iloc[5])
    data_date_column = data.apply(date_lambda, axis=1)
    data.insert(0, 'date', pd.to_datetime(data_date_column))
    data.set_index(data.date, inplace=True)
    data.drop([0, 1, 2, 3, 4, 5, 'date'], 1, inplace=True)

    data.columns = ['vv', 'dv', 'mme']

    data.replace(-99.9, np.nan, inplace=True)
    data.replace(-99.99, np.nan, inplace=True)
    data.replace(-9999, np.nan, inplace=True)
    data.replace(990, np.nan, inplace=True)

    df_mme = data.drop(['vv', 'dv'], 1)

    return df_mme


def get_line_number(file_name):
    noline = -1
    cod = -1

    cod_pattern = 'CODIGO\D*(\d+)'
    data_pattern = '\d{4}\s*(\d{1,2}\s*){3,4}\s*((-)?\d*(\.\d+)*\s*)*'

    with open(file_name) as f:
        lines = f.readlines()

    for i, line in enumerate(lines):
        m_cod = re.search(cod_pattern, line)
        m_data = re.search(data_pattern, line)
        if m_cod:
            cod = m_cod.group(1)
        elif m_data:
            if i > 0:
                while not lines[i-1].strip():
                    i -= 1
                noline = i - 1
                has_header = 'infer'
            else:
                noline = i
                has_header = None
            break

    return noline, has_header, cod


def plot_series_temporales(df, nombre):
    """Funcion que dibuja las series temporales de las distintas variables del agente
        Args:
            df: dataframe del agente
            nvars: número de variables del agente incluidas en el dataframe

        Returns:
            Salidas gráficas de las variables
    """

    nvars = len(df.columns)

    colors = ['b', 'purple', 'g', 'k', 'gray', 'orange', 'brown', 'g', 'k', 'gray', 'b', 'purple',]
    plt.style.use('ggplot')
    fig = plt.figure()
    fig.suptitle(nombre, fontweight='bold', fontsize=16)

    name_df = ['hs', 'tp', 'dh', 'vv', 'dv', 'pa', 'niv', 'mas', 'mme', 'tm', 'per']
    name_ylabel = ['$H_s (m)$', '$T_p (s)$', r'$\theta(^\circ)$', '$U (m/s)$', r'$\theta_U(^\circ)$', '$P_s (Hpa)$',
                   r'$\eta_{NM} (m)$', r'$\eta_{MAs} (m)$', r'$\eta_{MMe} (m)$', '$T_{m02} (s)$', '$H_s/L$']

    for j in range(nvars):
        plt.subplot(nvars, 1, j+1)
        plt.plot(df.index, df.iloc[:, j], '.', color=colors[j], markersize=5)
        ind_ylb = name_df.index(df.iloc[:, j].name)
        plt.ylabel(name_ylabel[ind_ylb])
        plt.xticks(rotation=10)

        # if j < nvars-1:
        #     plt.xticks(visible=False)
        # else:
        #     plt.xticks(rotation=30)

    plt.tight_layout(pad=0.5)


def plot_eventos_individuales(lista_df, nombre, dur_event):
    """Funcion que dibuja las series temporales de las distintas variables del agente
        Args:
            df: dataframe del agente
            nvars: número de variables del agente incluidas en el dataframe

        Returns:
            Salidas gráficas de las variables
    """

    nvars = len(lista_df[0].columns)

    plt.style.use('ggplot')
    fig = plt.figure()
    fig.suptitle(nombre, fontweight='bold', fontsize=16)

    name_df = ['hs', 'tp', 'dh', 'vv', 'dv', 'pa', 'niv', 'mas', 'mme', 'tm', 'per']
    name_ylabel = ['$H_s (m)$', '$T_p (s)$', r'$\theta(^\circ)$', '$U (m/s)$', r'$\theta_U(^\circ)$', '$P_s (Hpa)$',
                   r'$\eta_{NM} (m)$', r'$\eta_{MAs} (m)$', r'$\eta_{MMe} (m)$', '$T_m02 (s)$', '$H_s/L$']

    lista_colors = list(six.iteritems(colors.cnames))
    colors_ = lista_colors
    while len(lista_df) > len(colors_): colors_.extend(lista_colors)

    for j in range(nvars):
        coli = 0
        plt.subplot(nvars, 1, j+1)

        for i, df in enumerate(lista_df):
            if len(df) > dur_event:
                plt.plot(df.iloc[:, j].values, '.-', color=colors_[coli][0], markersize=5)
                coli += 1

        ind_ylb = name_df.index(lista_df[0].iloc[:, j].name)
        plt.ylabel(name_ylabel[ind_ylb])
        plt.xticks(rotation=10)

        # if j < nvars-1:
        #     plt.xticks(visible=False)
        # else:
        #     plt.xticks(rotation=30)

    plt.xlabel('horas')
    plt.tight_layout(pad=0.5)


def pandas_to_numpy(df_clima):
    df_clima.dropna(inplace=True)
    dat = df_clima.as_matrix()
    h, t, d, v, dv, niv = (dat[:, 0], dat[:, 1], dat[:, 2], dat[:, 3], dat[:, 4], dat[:, 5])

    yy = df_clima.index.year
    mm = df_clima.index.month
    dd = df_clima.index.day
    hh = df_clima.index.hour

    amdh = np.vstack((yy, mm, dd, hh)).T

    n = np.zeros(len(h))
    for i in range(0, len(h)):
        n[i] = datetime(int(amdh[i, 0]), int(amdh[i, 1]), int(amdh[i, 2])).toordinal() + amdh[i, 3]/24.

    return amdh, h, t, d, v, dv, niv, n


def pandas_to_numpy_5v(df_clima):
    dat = df_clima.as_matrix()
    h, t, d, v, dv = (dat[:, 0], dat[:, 1], dat[:, 2], dat[:, 3], dat[:, 4])

    yy = df_clima.index.year
    mm = df_clima.index.month
    dd = df_clima.index.day
    hh = df_clima.index.hour

    amdh = np.vstack((yy, mm, dd, hh)).T

    n = np.zeros(len(h))
    for i in range(0, len(h)):
        n[i] = datetime(int(amdh[i, 0]), int(amdh[i, 1]), int(amdh[i, 2])).toordinal() + amdh[i, 3]/24.

    return amdh, h, t, d, v, dv, n

