# -*- coding: utf-8 -*-
from __future__ import division
import numpy as np
import matplotlib.pyplot as plt
import normalizar


def mda_casos(n, h,t,d,ncasos):
    hnorm_cen = normalizar.normalizar(h, 0)
    tnorm_cen = normalizar.normalizar(t, 0)
    dnorm_cen = normalizar.normalizar(d, 1)

    x = np.vstack((hnorm_cen, tnorm_cen, dnorm_cen)).T # variables normalizadas
    casos = mda(x, [0, 1], [2], ncasos, 0)

    n_casos = n[casos]
    d_j = x[casos, :]

    return x, d_j, n_casos


def mda(x, esc, circ, m, id_):
    """Funcion para aplicar el Algoritmo de Maxima Disimilitud (MDA)

    Args:
        x: Matriz de datos (series temporales por columnas).
        esc: Columnas de ``x`` correspondientes a variables escalares.
        circ: Columnas de ``x`` correspondientes a variables circulares.
        m: Numero de casos representativos para seleccionar.
        id_: Identificador de la variable que determina el primer punto del subconjunto de tamano ``m``.

    Returns:
        Vector de posiciones en filas de x correspondientes a los m casos seleccionados.

    """

    n = x.shape[0]
    pos = np.zeros(m, dtype=np.int)

    # Seleccion del primer punto
    pos1 = np.argmax(x[:, id_])
    pos[0] = pos1

    # Seleccion del segundo punto
    ds2 = np.zeros(n)
    dc2 = np.zeros(n)
    for j, l in enumerate(esc):
        ds2 += ((x[:, l]-x[pos1, l]) ** 2)
    for j, l in enumerate(circ):
        aux = np.abs(x[:, l]-x[pos1, l])
        dc2 += ((np.minimum(aux, 2-aux)) ** 2)
    dissim = np.sqrt(ds2+dc2)
    pos2 = np.argmax(dissim)
    pos[1] = pos2

    # Seleccion del resto de puntos
    for l in range(2, m):
        ds2 = np.zeros(n)
        dc2 = np.zeros(n)
        for j, k in enumerate(esc):
            ds2 += ((x[:, k] - x[pos[l-1], k]) ** 2)
        for j, k in enumerate(circ):
            aux = np.abs(x[:, k] - x[pos[l-1], k])
            dc2 += ((np.minimum(aux, 2-aux)) ** 2)
        d = np.sqrt(ds2 + dc2)
        dissim = np.min(np.vstack((dissim, d)), 0)
        posk = np.argmax(dissim)
        pos[l] = posk

    return pos


def plot_data_norm(x_in, y_in, xlabel, ylabel):
    colors = ['gray', 'orange']

    plt.style.use('ggplot')
    plt.figure()
    plt.plot(x_in, y_in, '.', color=colors[0])
    plt.xlabel(xlabel, fontweight='bold')
    plt.ylabel(ylabel, fontweight='bold')


def plot_data_casos(x_in, y_in, x_casos, y_casos, xlabel, ylabel):
    colors = ['gray', 'orange']

    plt.style.use('ggplot')
    plt.figure()
    plt.plot(x_in, y_in, '.', color=colors[0])
    plt.plot(x_casos, y_casos, '.', color=colors[1])
    plt.xlabel(xlabel, fontweight='bold')
    plt.ylabel(ylabel, fontweight='bold')
    plt.legend(('Datos', 'Casos representativos'), loc='best')


def plot_series_reconstruidas(serie_reconst, casos_prop, nvars):
    """Funcion que dibuja las series temporales de las distintas variables del agente
        Args:
            serie_reconst: serie reconstruida
            casos_prop: casos propagados
            nvars: número de variables del agente incluidas en el dataframe

        Returns:
            Salidas gráficas de las variables
    """
    colors = ['gray', 'b']
    plt.style.use('ggplot')
    plt.figure()

    name_ylabel = ['$H_s (m)$', '$T_z (s)$', r'$\theta(^\circ)$']

    for j in range(nvars):
        plt.subplot(nvars, 1, j+1)
        plt.plot(serie_reconst[:, 0], serie_reconst[:, j+1], '.', color=colors[0])
        plt.plot(casos_prop[:, 0], casos_prop[:, j+1], '.', color=colors[1])
        plt.legend(('Serie reconstruida', 'Valores propagados'), loc='best')
        plt.ylabel(name_ylabel[j])
        plt.xticks(visible=False)
