# -*- coding: utf-8 -*-
from __future__ import division
import numpy as np
from dist_cnorm import dist_cnorm


def radial_basis_functions(d_j, v_pj, esc, circ, c):
    """Determinacion de las funciones de interpolacion de base radial

    Args:
        d_j: Casos representativos de las condiciones de entrada (matriz de m casos x nvar variables).
        v_pj: Condiciones de salida de los casos anteriores para la variable de estudio (vector de m casos x 1).
        esc: Columnas de ``dj`` correspondientes a variables escalares.
        circ: Columnas de ``dj`` correspondientes a variables circulares.
        c: Estimacion del paramero de forma de las funciones gaussianas.

    Returns:
        Un tupla de la forma (a, b, e).

        * ``a``: Coeficientes de ajuste de las funciones gaussianas.
        * ``b``: Coeficientes de ajuste de las funciones lineales.
        * ``e``: Norma del error para el parametro c seleccionado.

    """

    m = d_j.shape[0]
    nvar = esc.shape[0] + circ.shape[0]

    # Matriz de coeficientes
    # Composicion de la primera submatriz
    a_ij = np.zeros((m, m))
    for j in range(m):
        for k in range(m):
            ds2, dc2 = 0, 0
            for l, n in enumerate(esc):
                ds2 += (d_j[j, n] - d_j[k, n]) ** 2
            for l, n in enumerate(circ):
                dc2 += (dist_cnorm(d_j[j, n], d_j[k, n])) ** 2
            dist2 = ds2 + dc2
            a_ij[j, k] = np.exp(-dist2 / (2 * (c ** 2)))

    # Composicion de la segunda submatriz
    f_ij = np.hstack((np.ones((m, 1)), d_j))

    # Composicion de la sumatriz de ceros
    c_ij = np.zeros((f_ij.shape[1], f_ij.shape[1]))

    # Matriz final
    a1 = np.hstack((a_ij, f_ij))
    a2 = np.hstack((f_ij.T, c_ij))
    a_final = np.vstack((a1, a2))

    # Vector de terminos independientes
    y = np.hstack((v_pj, np.zeros((nvar + 1))))

    # Coeficientes
    coefs = np.linalg.solve(a_final, y)
    a = coefs[0:m]
    b = coefs[m:]

    # Calculo del error
    i = np.eye(a_final.shape[0], a_final.shape[1])
    e_k = np.zeros((m, 1))
    for k in range(m):
        xk = np.linalg.solve(a_final, i[:, k])
        e_k[k] = coefs[k] / xk[k]
    e = np.linalg.norm(e_k)

    return (a, b, e)
