# -*- coding: utf-8 -*-
from __future__ import division
import numpy as np
import pandas as pd
from scipy.special import gamma as fun_gamma
from verificacion.util.monomios import monomios, obtener_coeficientes_tabla


def altura_total(tipo, posicion, hi_rms, h, k_pico, par_dique, kr, phi):
    """Esta funcion calcula los parametros de la distribucion de altura total frente al dique

    Args:
        tipo: tipologia de dique con los siguientes codigos

            * Tipo = A -> Porous vertical breakwater (PVB)
            * Tipo = B -> Composite breakwater (CB)
            * Tipo = C1 -> Low mound breakwater (LMB)
            * Tipo = C2 -> High mound breakwater (HMB)
            * Tipo = C3 -> High mound composite breakwater (HCMB)
            * Tipo = D -> Rubble mound breakwater with crown wall (RMB-CW)
            * Tipo = E -> Rubble mound breakwater
            * Tipo = F -> S breakwater

        posicion: un valor con los siguientes codigos

            * x1: pie de berma
            * x2: 1/3 del pie de berma
            * x3: 2/3 del pie de berma
            * x4: frente al espaldon

        hi_rms: estadistico rms de la altura de ola incidente en el estado
        h: nivel del mar en el estado
        k_pico: numero de onda asociado al periodo de pico
        par_dique: un diccionario con los parametros geometricos del dique
        kr: coeficiente de reflexion
        phi: fase de la reflexion

    Returns:
        Una tupla con la forma (hw_rms, beta_hw, mono_ht, ensayado_ht)

        * ``ht_rms``: valor rms de la altura de ola total en la posicion considerada
        * ``beta_ht``: parametro de forma de la distribucion de weibull de la altura de ola total en la
            posicion considerada
        * ``mono_ht``: diccionario con los valores de los monomios empleados en el calculo
        * ``ensayado_hy``: un diccionario que indica mediante un booleano si el rango de los monomios de
            entrada esta ensayado (True) o no (False).

    """
    if tipo.startswith('C'):
        tipo = 'C'

    if tipo == 'E':
        tipo = 'D'
        print('Warning: la altura total corresponde a un dique tipo "D"')

    mono_ht, ensayado_ht, m4_pico = monomios('altura_total', tipo, hi_rms, h, k_pico, par_dique)

    coefw = pd.Series()
    if posicion == 'x1':
        ht_rms = hi_rms*np.sqrt(1 + kr**2 + 2*kr*np.cos(phi))
        beta_ht = 2
    else:
        coefw = obtener_coeficientes_tabla('altura_total', tipo, mono_ht, posicion)
        hp_rms = hi_rms*np.sqrt(1 + kr**2 + 2*kr*np.cos(phi))
        beta_ht = coefw['b_b1'] * m4_pico**2 + coefw['b_b2'] * m4_pico + coefw['b_b3']

        alpha_ht = coefw['a_a1'] * m4_pico**2 + coefw['a_a2'] * m4_pico + coefw['a_a3']
        ht_rms = np.sqrt((alpha_ht * hp_rms)**2 * fun_gamma(1 + 2 / beta_ht) - (
            alpha_ht * hp_rms * fun_gamma(1 + 1 / beta_ht))**2)

    return {'ht_rms': ht_rms, 'beta_ht': beta_ht, 'mono_ht': mono_ht, 'ensayado_ht': ensayado_ht}, coefw
