# -*- coding: utf-8 -*-
from __future__ import division
import numpy as np
import pandas as pd

from verificacion.util.monomios import monomios, obtener_coeficientes_tabla
from verificacion.util.transformaciones import sigmoide


def comportamiento_hidraulico(tipo, hi_rms, h, k_pico, par_dique):
    """Esta funcion calcula los coeficientes de reflexion (modulo y fase), transmision
    y dispacion en funcion de las variables geometricas y de oleaje

    Args:
        tipo: tipologia de dique con los siguientes codigos

            * Tipo = A -> Porous vertical breakwater (PVB)
            * Tipo = B -> Composite breakwater (CB)
            * Tipo = C1 -> Low mound breakwater (LMB)
            * Tipo = C2 -> High mound breakwater (HMB)
            * Tipo = C3 -> High mound composite breakwater (HCMB)
            * Tipo = D -> Rubble mound breakwater with crown wall (RMB-CW)
            * Tipo = E -> Rubble mound breakwater
            * Tipo = F -> S breakwater

        hi_rms: estadistico rms de la altura de ola incidente en el estado
        h: nivel del mar en el estado
        k_pico: numero de onda asociado al periodo de pico
        par_dique: un diccionario con los parametros geometricos del dique

    Returns:
        Un tupla de la forma (kr, kt, d_ast, phi).

        * ``kr``: Coeficiente de reflexion
        * ``kt``: Coeficiente de transmision
        * ``d_ast``: Coeficiente de disipacion
        * ``phi``: Fase de la reflexion
        * ``mono_sig``: diccionario con los valores de los monomios empleados en el calculo
        * ``ensayado_sig``: un diccionario que indica mediante un booleano si el rango de los monomios de
            entrada esta ensayado (True) o no (False).
        * ``coef``: un diccionario con los coeficientes empleados en calculo

    """
    subtipo = tipo
    if tipo.startswith('C'):
        tipo = 'C'

    mono_sig, ensayado_sig = monomios('comportamiento_hidraulico', tipo, hi_rms, h, k_pico, par_dique)

    gamma_r, axr, kr0, kr1, gamma_t, axt, kt0, kt1, gamma_phi, ax_phi, x_0, x_1 = parametros_sigmoide(subtipo, mono_sig)

    x = mono_sig['m1']

    k_r = sigmoide(x, kr0, kr1, gamma_r, axr)

    k_t = 0
    if tipo != 'E' or tipo != 'F':
        k_t = sigmoide(x, kt0, kt1, gamma_t, axt)

    dis = 1 - k_r ** 2 - k_t ** 2

    phi_ad = sigmoide(x, x_0, x_1, gamma_phi, ax_phi)
    phi = phi_ad * (4 * np.pi)

    coef = pd.Series((gamma_r, axr, kr0, kr1, gamma_t, axt, kt0, kt1, gamma_phi, ax_phi, x_0, x_1), index=(
        'gamma_r', 'axr', 'kr0', 'kr1', 'gamma_t', 'axt', 'kt0', 'kt1', 'gamma_phi', 'ax_phi', 'x_0', 'x_1'),
        name='coef_sigmoide')

    return k_r, k_t, dis, phi, mono_sig, ensayado_sig, coef


def parametros_sigmoide(tipo, mono):
    """Esta funcion calcula los parametros para el calculo de los coeficientes de reflexion (modulo y fase),
    transmision y dispacion

    Args:
        tipo: tipologia de dique con los siguientes codigos

            * Tipo = A -> Porous vertical breakwater (PVB)
            * Tipo = B -> Composite breakwater (CB)
            * Tipo = C1 -> Low mound breakwater (LMB)
            * Tipo = C2 -> High mound breakwater (HMB)
            * Tipo = C3 -> High mound composite breakwater (HCMB)
            * Tipo = D -> Rubble mound breakwater with crown wall (RMB-CW)
            * Tipo = E -> Rubble mound breakwater
            * Tipo = F -> S breakwater

        mono: Diccionario de monomios. Los monomios no definidos toman el valor -1

            0.  m0: Fb/Hirms: Rebasabilidad del dique
            1.  m1: Aeq/L^2: Parametro de dispersion en 2D
            2.  m2: Dk: Diametro relativo del material granular
            3.  m3: Hirms/L: Peralte incidente
            4.  m4: h/L: Profundidad relativa
            5.  m5: B/L: Anchura relativa del dique
            6.  m6: hb/h: Altura relativa de la cimentacion
            7.  m7: Bb/h: Anchura relativa de la berma
            8.  m8: FMT/h: Altura relativa de la berma
            9.  m9: Aeq_b/L^2: Area equivalente relativa de la berma
            10. m10: D_bk: Diametro relativo del material de la berma
            11. m11: Tipo de pieza del manto
            12. m12: ns: Porosidad del manto
            13. m13: alphaT: Pendiente del talud a barlomar

    Returns:
        Un tupla con los coeficientes de la sigmoide para el cálculo de la reflexion, la transmision
         y la fase

    """
    kr0, kr1, kt0, kt1, axr, axt, gamma_r, gamma_t, ax_phi = 0, 0, 0, 0, 0, 0, 0, 0, 0

    coefs = obtener_coeficientes_tabla('comportamiento_hidraulico', tipo, mono)

    hbh = mono['m6']
    fmth = mono['m8']
    pieza = mono['m11']
    ns = mono['m12']

    gamma_phi = coefs['a_1phi']
    x_0 = coefs['c_1phi']
    x_1 = coefs['d_1phi']

    if tipo == 'A':
        gamma_r = coefs['a_1r']
        axr = coefs['b_1r']
        kr0 = coefs['c_1r']
        kr1 = coefs['d_1r']

        gamma_t = coefs['a_1t']
        axt = coefs['b_1t']
        kt0 = coefs['c_1t']
        kt1 = coefs['d_1t']

        ax_phi = coefs['b_1phi'] * 1 + coefs['b_2phi']
    elif tipo == 'B':
        gamma_r = coefs['a_1r'] * hbh ** coefs['a_2r'] + coefs['a_3r']
        axr = coefs['b_1r'] * hbh ** coefs['b_2r'] + coefs['b_3r']
        kr0 = coefs['c_1r'] * hbh ** 2 + coefs['c_2r'] * hbh + coefs['c_3r']
        kr1 = coefs['d_1r']

        gamma_t = coefs['a_1t'] * hbh ** coefs['a_2t'] + coefs['a_3t']
        axt = coefs['b_1t'] * hbh ** coefs['b_2t'] + coefs['b_3t']
        kt0 = coefs['c_1t'] * hbh ** coefs['c_2t'] + coefs['c_3t']
        kt1 = coefs['d_1t']

        ax_phi = coefs['b_1phi'] * hbh + coefs['b_2phi']
    elif tipo.startswith('C'):
        gamma_r = coefs['a_1r'] * fmth ** coefs['a_2r'] + coefs['a_3r']
        axr = coefs['b_1r'] * fmth ** coefs['b_2r'] + coefs['b_3r']
        kr0 = coefs['c_1r'] * fmth ** coefs['c_2r'] + coefs['c_3r']
        kr1 = coefs['d_1r']

        gamma_t = coefs['a_1t']
        axt = coefs['b_1t'] * fmth ** coefs['b_2t'] + coefs['b_3t']
        kt0 = coefs['c_1t'] * fmth ** coefs['c_2t'] + coefs['c_3t']
        kt1 = coefs['d_1t']

        ax_phi = coefs['b_1phi'] * fmth + coefs['b_2phi']
    elif tipo == 'D':
        gamma_r = coefs['a_1r'] * fmth ** coefs['a_2r'] + coefs['a_3r']
        axr = coefs['b_1r'] * fmth ** coefs['b_2r'] + coefs['b_3r']
        kr0 = coefs['c_1r'] * hbh ** 2 + coefs['c_2r'] * fmth + coefs['c_3r']
        kr1 = coefs['d_1r']

        gamma_t = coefs['a_1t']
        axt = coefs['b_1t'] * fmth ** coefs['b_2t'] + coefs['b_3t']
        kt0 = coefs['c_1t'] * hbh ** coefs['c_2t'] + coefs['c_3t']
        kt1 = coefs['d_1t']

        ax_phi = coefs['b_1phi'] * fmth + coefs['b_2phi']
    elif tipo == 'E':
        gamma_r = coefs['a_1r']
        kr1 = coefs['d_1r']
        if pieza:
            axr = coefs['b_1r'] * ns + coefs['b_2r']
            kr0 = coefs['c_1r'] * ns + coefs['c_2r']
        else:
            axr = coefs['b_1r']
            kr0 = coefs['c_1r']

        ax_phi = coefs['b_1phi']
    elif tipo == 'F':
        gamma_r = coefs['a_1r']
        axr = coefs['b_1r']
        kr0 = coefs['c_1r']
        kr1 = coefs['d_1r']

        ax_phi = coefs['b_1phi']

    return gamma_r, axr, kr0, kr1, gamma_t, axt, kt0, kt1, gamma_phi, ax_phi, x_0, x_1
