# -*- coding: utf-8 -*-
from __future__ import division
import numpy as np
import pandas as pd
from scipy.special import gamma as fun_gamma
from verificacion.util.monomios import monomios, obtener_coeficientes_tabla


def deslizamiento(tipo, hi_rms, h, k_pico, par_dique, k_t, ht):
    """Esta funcion verifica el modo de fallo de deslizamiento del espaldon

    Args:
        tipo: tipologia de dique con los siguientes codigos

            * Tipo = A -> Porous vertical breakwater (PVB)
            * Tipo = B -> Composite breakwater (CB)
            * Tipo = C1 -> Low mound breakwater (LMB)
            * Tipo = C2 -> High mound breakwater (HMB)
            * Tipo = C3 -> High mound composite breakwater (HCMB)
            * Tipo = D -> Rubble mound breakwater with crown wall (RMB-CW)
            * Tipo = E -> Rubble mound breakwater
            * Tipo = F -> S breakwater

        hi_rms: estadistico rms de la altura de ola incidente en el estado
        h: nivel del mar en el estado
        k_pico: numero de onda asociado al periodo de pico
        par_dique: un diccionario con los parametros geometricos del dique
        k_t: coeficiente de transmision
        ht: un diccionario con los estadisticos de la altura de ola total en el frente del espaldon

    Returns:
        devuelve: una serie con la forma (beta, s_mean, s_var)

        * ``beta``: indice de fiabilidad
        * ``s_mean``: valor medio del margen de seguridad
        * ``s_var``: varianza del margen de seguridad

        mono_des: diccionario con los valores de los monomios empleados en el calculo
        ensayado_des: un diccionario que indica mediante un booleano si el rango de los monomios de
            entrada esta ensayado (True) o no (False).
        coef: un diccionario con los coeficientes empleados en calculo

    """
    G = 9.81  # m/s2
    DENS_W = 1025  # kg/m3

    subtipo = tipo
    if tipo.startswith('C'):
        tipo = 'C'

    ht_rms = ht['ht_rms']
    beta_ht = ht['beta_ht']

    mono_des, ensayado_des, m4_pico = monomios('deslizamiento', tipo, hi_rms, h, k_pico, par_dique)

    # Variables derivadas
    # dens_espaldon = DENS_W * (par_dique['dens_rel'] + 1)
    dens_espaldon = 2700
    if tipo == 'C':
        par_dique['w1'] = (par_dique['f_c'] + (h - par_dique['h_b'])) * par_dique['b'] * dens_espaldon * G - (
                         h - par_dique['h_b']) * par_dique['b'] * par_dique['dens_rel'] * DENS_W * G
    if tipo == 'D':
        if h - par_dique['h_b'] <= 0:
            par_dique['w1'] = (par_dique['b'] * par_dique['h_fc'] + (par_dique['f_c'] + h - par_dique['h_b'] - par_dique[
                'h_fc']) * par_dique['h_fc']) * dens_espaldon * G
        else:
            par_dique['w1'] = ((
                                   par_dique['b'] * par_dique['h_fc'] + (
                                   par_dique['f_c'] + h - par_dique['h_b'] - par_dique[
                                       'h_fc']) * par_dique['h_fc']) * dens_espaldon * G) - (
                              (h - par_dique['h_b']) * par_dique['b'] * par_dique['dens_rel'] * DENS_W * G)

    # TODO: comprobar para cuales tipos debe entrar y el peso del espaldón que debería ser

    # Transformacion del oleaje
    coef = obtener_coeficientes_tabla('deslizamiento', subtipo, mono_des)

    # Fuerzas horizontales
    if subtipo == 'C3':
        alpha_i = coef['alpha_I1'] * mono_des['m7'] * mono_des['m4'] + coef['alpha_I2']
    else:
        alpha_i = 1

    if tipo == 'C':
        fh_rms = DENS_W * G * ht_rms / 8 * (ht_rms + 2 * (
                (h - par_dique['f_mt']) + (h - par_dique['h_b'])*np.cosh(m4_pico * mono_des['m8'])/np.cosh(m4_pico) +
                (par_dique['f_mt'] - par_dique['h_b'])*np.cosh(
                    m4_pico * mono_des['m6'])/np.cosh(m4_pico) * coef['alpha_2'])
                ) * coef['alpha_1'] * alpha_i
    else:
        fh_rms = DENS_W*G*ht_rms/8*(ht_rms + 2*(h - par_dique['h_b'])*(
                1 + np.cosh(m4_pico * mono_des['m6'])/np.cosh(m4_pico)*coef['alpha_2'])) * coef['alpha_1'] * alpha_i

    delta_fh = coef['c_rFH'] * fh_rms
    beta_fh = beta_ht*(coef['a_beta_F1H'] * m4_pico**2 + coef['a_beta_F2H'] * m4_pico + coef['a_beta_F3H'])

    fh_mean = delta_fh*fun_gamma(1 + 1/beta_fh)
    fh_var = delta_fh**2*fun_gamma(1 + 2/beta_fh) - fh_mean**2

    # Fuerzas verticales
    alpha_3 = coef['a_alpha3'] * mono_des['m5']**coef['b_alpha3']

    fv_rms = DENS_W*G*ht_rms/4*(
        par_dique['b'] * np.cosh(m4_pico * mono_des['m6'])/np.cosh(m4_pico)*(
            coef['alpha_1'] * coef['alpha_2']*alpha_i)*(1 + alpha_3 * k_t))
    delta_fv = coef['c_rFV'] * fv_rms
    beta_fv = beta_ht*(coef['a_beta_F1V'] * m4_pico**2 + coef['a_beta_F2V'] * m4_pico + coef['a_beta_F3V'])

    fv_mean = delta_fv * fun_gamma(1 + 1/beta_fv)
    fv_var = delta_fv**2 * fun_gamma(1 + 2/beta_fv) - fv_mean ** 2

    # Correlaciones
    rho_fv_fh = coef['a_rhor_FHFV']*np.exp(m4_pico) + 1
    cov_fv_fh = rho_fv_fh*np.sqrt(fv_var*fh_var)

    # Margen de seguridad
    s_mean = par_dique['mu_mean']*par_dique['w1'] - par_dique['mu_mean']*fv_mean - fh_mean  # N/m
    s_var = (par_dique['w1'] - fv_mean)**2 * par_dique['mu_var'] + fh_var + (
        par_dique['mu_mean']**2 * fv_var + 2 * par_dique['mu_mean'] * cov_fv_fh)
    s_des = np.sqrt(s_var)
    beta = s_mean/np.sqrt(s_var)

    # Pasamos a kN/m
    s_mean = s_mean / 1000
    s_des = s_des / 1000
    s_var = s_var / 1000

    devuelve = pd.Series((beta, s_mean, s_var, s_des), index=('beta', 'mean', 'var', 'dest'), name='deslizamiento')
    return devuelve, mono_des, ensayado_des, coef
