# -*- coding: utf-8 -*-
from __future__ import division
import numpy as np
import pandas as pd
from scipy.special import gamma as fun_gamma
from verificacion.util.monomios import monomios, obtener_coeficientes_tabla
from verificacion.util.transformaciones import sigmoide


def estabilidad_berma(tipo, hi_rms, h, k_pico, par_dique, ht, nivel_averia):
    """Esta funcion verifica el modo de fallo de estabilidad de la berma

    Args:
        tipo: tipologia de dique con los siguientes codigos

            * Tipo = A -> Porous vertical breakwater (PVB)
            * Tipo = B -> Composite breakwater (CB)
            * Tipo = C1 -> Low mound breakwater (LMB)
            * Tipo = C2 -> High mound breakwater (HMB)
            * Tipo = C3 -> High mound composite breakwater (HCMB)
            * Tipo = D -> Rubble mound breakwater with crown wall (RMB-CW)
            * Tipo = E -> Rubble mound breakwater
            * Tipo = F -> S breakwater

        hi_rms: estadistico rms de la altura de ola incidente en el estado
        h: nivel del mar en el estado
        k_pico: numero de onda asociado al periodo de pico
        par_dique: un diccionario con los parametros geometricos del dique
        ht: un diccionario con los estadisticos de la altura de ola total en el frente del espaldon
        nivel_averia: Nivel de danio para el que se analiza el modo de fallo. Toma los siguientes valores:

            * IA: Inicio de averia
            * DI: Danio intermedio
            * D: Destruccion

    Returns:
        devuelve: una serie con la forma (beta, s_mean, s_var)

        * ``beta``: indice de fiabilidad
        * ``s_mean``: valor medio del margen de seguridad
        * ``s_var``: varianza del margen de seguridad

        mono_des: diccionario con los valores de los monomios empleados en el calculo
        ensayado_des: un diccionario que indica mediante un booleano si el rango de los monomios de
            entrada esta ensayado (True) o no (False).
        coef: un diccionario con los coeficientes empleados en calculo

    """
    # subtipo = tipo
    # if tipo.startswith('C'):
    #     tipo = 'C'

    ht_rms = ht['ht_rms']
    beta_ht = ht['beta_ht']

    mono_estab_berma, ensayado_estab_berma, m4_pico = monomios('estabilidad_berma', tipo, hi_rms, h, k_pico, par_dique)

    mono_estab_berma['nivel_averia'] = nivel_averia
    # Transformacion del oleaje
    coef = obtener_coeficientes_tabla('estabilidad_berma', tipo, mono_estab_berma)

    par_est = sigmoide(mono_estab_berma['m9'], coef['nsb_0'], coef['nsb_1'], coef['gamma_nsb'], coef['ax_nsb'])
    delta_ht = ht_rms / np.sqrt(fun_gamma(1 + 2 / beta_ht))

    ht_mean = delta_ht * fun_gamma(1 + 1 / beta_ht)
    ht_var = delta_ht ** 2 * fun_gamma(1 + 2 / beta_ht) - ht_mean ** 2

    # Margen de seguridad
    s_mean = par_dique['d_b_mean'] - ht_mean / par_dique['dens_rel'] / par_est
    s_var = par_dique['d_b_var'] + ht_var / (par_dique['dens_rel'] * par_est) ** 2
    beta = s_mean / np.sqrt(s_var)
    s_des = np.sqrt(s_var)

    devuelve = pd.Series((beta, s_mean, s_var, s_des), index=('beta', 'mean', 'var', 'dest'), name='estabilidad_berma')
    return devuelve, mono_estab_berma, ensayado_estab_berma, coef
