# -*- coding: utf-8 -*-
from __future__ import division
import numpy as np
import pandas as pd
from verificacion.util.monomios import cargar_monomios
import os
from scipy.special import gamma as fun_gamma



def estabilidad_talud(hi_rms, tp, par_dique, ht):
    """Esta funcion verifica el modo de fallo de estabilidad de piezas del talud

        Args:
            hi_rms: estadistico rms de la altura de ola incidente en el estado
            h: nivel del mar en el estado
            k_pico: numero de onda asociado al periodo de pico
            par_dique: un diccionario con los parametros geometricos del dique
            ht: un diccionario con los estadisticos de la altura de ola total en el frente del espaldon
            nivel_averia: nivel de danio para el que se analiza el modo de fallo. Toma los siguientes valores:

                * IA: Inicio de averia
                * DI: Danio intermedio
                * D: Destruccion

        Returns:
            devuelve: una serie con la forma (beta, s_mean, s_var)

            * ``beta``: indice de fiabilidad
            * ``s_mean``: valor medio del margen de seguridad
            * ``s_var``: varianza del margen de seguridad

            mono_des: diccionario con los valores de los monomios empleados en el calculo
            ensayado_des: un diccionario que indica mediante un booleano si el rango de los monomios de
                entrada esta ensayado (True) o no (False).
            coef: un diccionario con los coeficientes empleados en calculo

        """


    beta_ht = ht['beta_ht']

    # Calculo de variables para fórmula de Losada
    L0 = (9.81 * tp ** 2) / (2 * np.pi)
    tan_alpha = 1 / par_dique['alpha_t']
    Ir0 = 2.654 * (tan_alpha)

    # Lectura tabla para coeficientes A_talud y B_talud
    directorio_actual = os.path.dirname(__file__)
    coef = cargar_coefs_piezas(directorio_actual, 'estabilidad_talud', ['tipo_pieza', 'alpha'])
    coef = coef.loc[par_dique['tipo_pieza'], str(par_dique['alpha_t'])]

    delta_ht = hi_rms / np.sqrt(fun_gamma(1 + 2 / beta_ht))

    ht_mean = delta_ht * fun_gamma(1 + 1 / beta_ht)
    ht_var = delta_ht ** 2 * fun_gamma(1 + 2 / beta_ht) - ht_mean ** 2

    Ir_mean = (tan_alpha) / np.sqrt(ht_mean / L0)

    # Margen de seguridad
    s_mean = par_dique['d_mean'] - (ht_mean * (
    coef['A_talud'] * (Ir_mean - Ir0) * np.exp(coef['B_talud'] * (Ir_mean - Ir0))) ** (1 / 3)) / (par_dique['dens_rel'])

    s_var = par_dique['d_var'] + ht_var * ((ht_mean * (
    (coef['A_talud'] * tan_alpha * np.exp((coef['B_talud'] * tan_alpha) / (ht_mean / L0) ** (1 / 2) - Ir0)) / (
    2 * L0 * (ht_mean / L0) ** (3 / 2)) - (coef['A_talud'] * coef['B_talud'] * tan_alpha * np.exp(
        (coef['B_talud'] * tan_alpha) / (ht_mean / L0) ** (1 / 2) - Ir0) * (
                                               Ir0 - tan_alpha / (ht_mean / L0) ** (1 / 2))) / (
    2 * L0 * (ht_mean / L0) ** (3 / 2)))) / (3 * par_dique['dens_rel'] * (
    -coef['A_talud'] * np.exp((coef['B_talud'] * tan_alpha) / (ht_mean / L0) ** (1 / 2) - Ir0) * (
        Ir0 - tan_alpha / (ht_mean / L0) ** (1 / 2))) ** (2 / 3)) - (-coef['A_talud'] * np.exp(
        (coef['B_talud'] * tan_alpha) / (ht_mean / L0) ** (1 / 2) - Ir0) * (
                                                                         Ir0 - tan_alpha / (ht_mean / L0) ** (
                                                                         1 / 2))) ** (1 / 3) / par_dique[
                                               'dens_rel']) ** 2
    
    beta = s_mean / np.sqrt(s_var)
    s_des = np.sqrt(s_var)

    devuelve = pd.Series((beta, s_mean, s_var, s_des), index=('beta', 'mean', 'var', 'dest'), name='estabilidad_talud')

    return devuelve

def cargar_coefs_piezas(folder, tabla, cols):

    df_coefs = cargar_monomios(folder, tabla, cols)

    return df_coefs