from __future__ import division
import numpy as np
import ast
import pandas as pd
import os
from verificacion.util.transformaciones import k2k0


def monomios(tabla, tipo, hi_rms, h, k_pico, par_dique):
    """ Esta funcion calcula los monomios empleados en la verificacion

        Args:
            tabla: El nombre de la tabla de la que se van a leer los intervalos ensayados.
            tipo: tipologia de dique con los siguientes codigos

                * Tipo = A -> Porous vertical breakwater (PVB)
                * Tipo = B -> Composite breakwater (CB)
                * Tipo = C1 -> Low mound breakwater (LMB)
                * Tipo = C2 -> High mound breakwater (HMB)
                * Tipo = C3 -> High mound composite breakwater (HCMB)
                * Tipo = D -> Rubble mound breakwater with crown wall (RMB-CW)
                * Tipo = E -> Rubble mound breakwater
                * Tipo = F -> S breakwater

            hi_rms: estadistico rms de la altura de ola incidente
            h: nivel del mar en el estado
            k_pico: numero de onda asociado al periodo de pico
            par_dique: un diccionario con los parametros geometricos del dique

                * a_eq -> area unitaria del medio poroso bajo el nivel medio
                * d -> tamanio de grano
                * b -> ancho del espaldon
                * hb -> profundidad de la base del espaldon
                * f_mt -> altura de la berma
                * b_b -> ancho de la berma
                * q_p -> porosidad del manto principal
                * alpha_t -> pendiente del talud a barlomar
                * a_eq_b -> area de la berma de pie

        Returns:
            mono_t: Diccionario con los util dentro del rango de ensayo.
                Los util no definidos toman el valor -1
            ensayado_t: Diccionario que indica si el monomio de entrada esta dentro
                del rango ensyao (True/False)
    """
    mono = calcula_monomios(tipo, hi_rms, h, k_pico, par_dique)
    mono_t, ensayado_t = comprueba_mono(tabla, tipo, mono)

    if tabla != 'comportamiento_hidraulico':
        k_medio_aux = mono_t['m4'] * 2 * np.pi / h
        k_pico_aux = k2k0(k_medio_aux, h)

        m4_pico = k_pico_aux * h

        devuelve = mono_t, ensayado_t, m4_pico
    else:
        devuelve = mono_t, ensayado_t

    return devuelve


def calcula_monomios(tipo, hi_rms, h, k_pico, par_dique):
    """Esta funcion calcula los monomios adimensionales que definen el comportamiento
    del dique en un estado

    Args:
        tipo: tipologia de dique con los siguientes codigos

            * Tipo = 1 -> Porous vertical breakwater (PVB)
            * Tipo = 2 -> Composite breakwater (CB)
            * Tipo = 3 -> Low mound breakwater (LMB)
            * Tipo = 4 -> High mound breakwater (HMB)
            * Tipo = 5 -> High mound composite breakwater (HCMB)
            * Tipo = 6 -> Rubble mound breakwater with crown wall (RMB-CW)
            * Tipo = 7 -> Rubble mound breakwater
            * Tipo = 8 -> S breakwater
            * Tipo = 9 -> Cualquiera de los tipos (LMB, HMB, HCMB)

        hi_rms: estadistico rms de la altura de ola incidente
        h: nivel del mar en el estado
        k_pico: numero de onda asociado al periodo de pico
        par_dique: un diccionario con los parametros geometricos del dique

            * a_eq -> area unitaria del medio poroso bajo el nivel medio
            * d -> tamanio de grano
            * b -> ancho del espaldon
            * hb -> profundidad de la base del espaldon
            * f_mt -> altura de la berma
            * b_b -> ancho de la berma
            * q_p -> porosidad del manto principal
            * alpha_t -> pendiente del talud a barlomar
            * a_eq_b -> area de la berma de pie


    Returns:
        mono: Diccionario de monomios. Los monomios no definidos toman el valor -1

            0.  m0: Fb/Hirms: Rebasabilidad del dique
            1.  m1: Aeq/L^2: Parametro de dispersion en 2D
            2.  m2: Dk: Diametro relativo del material granular
            3.  m3: Hirms/L: Peralte incidente
            4.  m4: h/L: Profundidad relativa
            5.  m5: B/L: Anchura relativa del dique
            6.  m6: hb/h: Altura relativa de la cimentacion
            7.  m7: Bb/h: Anchura relativa de la berma
            8.  m8: FMT/h: Altura relativa de la berma
            9.  m9: Aeq_b/L^2: Area equivalente relativa de la berma
            10. m10: D_bk: Diametro relativo del material de la berma
            11. m11: Tipo de pieza del manto
            12. m12: ns: Porosidad del manto
            13. m13: alphaT: Pendiente del talud a barlomar

    """

    if tipo.startswith('C'):
        tipo = 'C'

    k_medio = k2k0(k_pico, h, True)

    mono = pd.Series(index=('m0', 'm1', 'm2', 'm3', 'm4', 'm5', 'm6', 'm7', 'm8', 'm9', 'm10', 'm11', 'm12', 'm13'),
                     name='entrada')

    mono['m0'] = par_dique['f_c'] / hi_rms
    mono['m1'] = par_dique['a_eq'] / 4 / np.pi**2 * k_medio**2
    mono['m2'] = par_dique['d'] * k_medio
    mono['m3'] = hi_rms * k_medio / 2 / np.pi
    mono['m4'] = h * k_medio / 2 / np.pi

    if tipo == 'A':
        mono['m5'] = par_dique['b']*k_medio/2/np.pi
    elif tipo == 'B':
        mono['m5'] = par_dique['b']*k_medio/2/np.pi
        mono['m6'] = par_dique['h_b']/h
    elif tipo == 'C' or tipo == 'D':
        mono['m5'] = par_dique['b']*k_medio/2/np.pi
        mono['m6'] = par_dique['h_b']/h
        mono['m7'] = par_dique['b_b']/h
        mono['m8'] = par_dique['f_mt']/h
        mono['m9'] = par_dique['a_eq_b']/4/np.pi**2*k_medio**2
        mono['m10'] = par_dique['d_b'] * k_medio
        mono['m11'] = par_dique['pieza']
        mono['m12'] = par_dique['ns']
        mono['m13'] = par_dique['alpha_t']
    elif tipo == 'E' or tipo == 'F':
        mono['m9'] = par_dique['a_eq_b']/4/np.pi**2*k_medio**2
        mono['m10'] = par_dique['d_b']*k_medio
        mono['m11'] = par_dique['pieza']
        mono['m12'] = par_dique['ns']
        mono['m13'] = par_dique['alpha_t']
    return mono


def comprueba_mono(tabla, tipo, mono):
    """ Esta funcion devuelve la lista de util validos para los rangos de datos disponibles.

    Args:
        tabla: El nombre de la tabla de la que se van a leer los intervalos ensayados.
        tipo: tipologia de dique con los siguientes codigos

            * Tipo = A -> Porous vertical breakwater (PVB)
            * Tipo = B -> Composite breakwater (CB)
            * Tipo = C1 -> Low mound breakwater (LMB)
            * Tipo = C2 -> High mound breakwater (HMB)
            * Tipo = C3 -> High mound composite breakwater (HCMB)
            * Tipo = D -> Rubble mound breakwater with crown wall (RMB-CW)
            * Tipo = E -> Rubble mound breakwater
            * Tipo = F -> S breakwater

        mono: Diccionario de util. Los util no definidos toman el valor -1

            0.  m0: Fc/Hirms: Rebasabilidad del dique
            1.  m1: Aeq/L^2: Parametro de dispersion en 2D
            2.  m2: Dk: Diametro relativo del material granular
            3.  m3: Hirms/L: Peralte incidente
            4.  m4: h/L: Profundidad relativa
            5.  m5: B/L: Anchura relativa del dique
            6.  m6: hb/h: Altura relativa de la cimentacion
            7.  m7: Bb/h: Anchura relativa de la berma
            8.  m8: FMT/h: Altura relativa de la berma
            9.  m9: Aeq_b/L^2: Area equivalente relativa de la berma
            10. m10: D_bk: Diametro relativo del material de la berma
            11. m11: Tipo de pieza del manto
            12. m12: ns: Porosidad del manto
            13. m13: alphaT: Pendiente del talud a barlomar

    Returns:
        dict_mono_out: Diccionario con los util dentro del rango de ensayo.
            Los util no definidos toman el valor -1

        dict_ensayado_out: Diccionario que indica si el monomio de entrada esta dentro
            del rango ensyao (True/False)

    """
    dict_mono_out, dict_ensayado_out = pd.Series(name='salida_'+tabla), pd.Series(name=tabla)

    directorio_actual = os.path.dirname(__file__)
    df_intervalo = cargar_intervalos_csv(os.path.join(directorio_actual, '..', tabla), tabla)

    df_mono_filtrado = df_intervalo.loc[df_intervalo.Tipo == tipo]

    for m, v in mono.iteritems():
        valores_medios = df_mono_filtrado.loc[df_mono_filtrado.Rango == 'Medio', m]
        valores_inf = df_mono_filtrado.loc[df_mono_filtrado.Rango == 'Inferior', m]
        valores_sup = df_mono_filtrado.loc[df_mono_filtrado.Rango == 'Superior', m]

        mono_out, ensayado_out = mono_ensayo(v, valores_medios.iloc[0], valores_inf.iloc[0], valores_sup.iloc[0])
        dict_mono_out[m] = mono_out
        dict_ensayado_out[m] = ensayado_out

    return dict_mono_out, dict_ensayado_out


def mono_ensayo(v, valores_medios, valores_inf, valores_sup):
    """ Esta funcion devuelve la lista de util validos para los rangos de datos disponibles.

    Args:
        v: Valor del monomio a analizar
        valores_medios: Lista que puede contener lo siguiente:

            * 'abierto': indica que el monomio de salida puede tomar cualquier valor
            * 'cerrado': indica que el monomio de salida puede tomar cualquier valor
            dentro de los rangos ensayados.
            * valores numericos: indica que el monomio de salida solo puede tomar los
            valores indicados.

        valores_inf: Lista con los intervalos inferiores de cada rango ensayado.
        valores_sup: Lista con los intervalos superiores de cada rango ensayado.


    Returns:
        mono_out: Valor del monomio dentro del rango de ensayo.
            Si el monomio no esta definido se devuelve el valor -1

        ensayado_out: Valor logico que indica si el monomio de entrada esta dentro
            del rango ensyao (True/False)

    """
    mono_out, ensayado_out = np.NaN, False

    for i, _ in enumerate(valores_inf):
        if valores_sup[i] >= v > valores_inf[i]:
            ensayado_out = True
            if valores_medios[i] == 'abierto' or valores_medios[i] == 'cerrado':
                mono_out = v
            else:
                mono_out = valores_medios[i]

    if valores_inf:
        if not ensayado_out:
            if valores_medios[0] == 'abierto':
                mono_out = v
            elif valores_medios[0] == 'cerrado':
                if v < min(valores_inf):
                    mono_out = min(valores_inf)
                else:
                    mono_out = max(valores_sup)
            else:
                if v < min(valores_inf):
                    mono_out = valores_medios[0]
                else:
                    mono_out = valores_medios[-1]

    return mono_out, ensayado_out


def cargar_intervalos_csv(folder, tabla):
    ruta = os.path.join(folder, 'intervalo_' + tabla + '.csv')

    df = pd.read_csv(ruta)

    for columna in df.columns[3:]:
        df.ix[:, columna] = df.ix[:, columna].apply(ast.literal_eval)

    return df


def cargar_monomios(folder, tabla, cols):
    ruta = os.path.join(folder, tabla + '.csv')

    types = dict((c, object) for c in cols)
    df_mono = pd.read_csv(ruta, dtype=types)
    df_mono.set_index(cols, inplace=True)

    return df_mono


def obtener_coeficientes_tabla(tabla, tipo, valor_mono, pos=None):
    cols = pd.Series()

    cols['tipo'] = tipo

    if pos:
        cols['pos'] = pos

    lista_monomios = ()
    if tabla == 'altura_total':
        lista_monomios = ('m8', 'm7', 'm2')
    elif tabla == 'comportamiento_hidraulico':
        lista_monomios = ('m11', 'm12', 'm7', 'm2')
    elif tabla == 'deslizamiento' or tabla == 'vuelco':
        lista_monomios = ('m2', 'm3', 'm5', 'm6', 'm7', 'm8')
    if tabla == 'estabilidad_berma' or tabla == 'estabilidad_talud':
        lista_monomios = 'nivel_averia'

    for (k, v) in valor_mono.iteritems():
        if k in lista_monomios:
            if pd.notnull(v):
                cols[k] = str(v)
            else:
                cols[k] = np.NaN

    directorio_actual = os.path.dirname(__file__)
    mono = cargar_monomios(os.path.join(directorio_actual, '..', tabla), tabla, list(cols.index))

    cols_filtrado = cols.dropna()
    indices = list(cols_filtrado.index.values)
    valores_cols = list(cols_filtrado.values)

    coefs = mono.xs(valores_cols, level=indices)

    return coefs.iloc[0].rename('coef_' + tabla)
