# -*- coding: utf-8 -*-
from __future__ import division

import numpy as np
from scipy.optimize import fsolve
from scipy.special import erfc
from scipy.stats import rayleigh


def wavenum(t, h):
    """ Esta funcion calcula el numero de onda asociado a un periodo y una profundidad por medio
    de la ecuacion de la dispersion

    Args:
        t: periodo del oleaje
        h: profundidad

    Returns:
        k: numero de onda

    """
    G = 9.81
    gamma = (2*np.pi/t)**2*h/G
    sigma2 = (2*np.pi/t)**2

    def func(k):
        return sigma2/G - k*np.tanh(k*h)

    if gamma > np.pi**2:
        ko = np.sqrt(gamma/h)
        k = fsolve(func, ko)
    else:
        ko = gamma/h
        k = fsolve(func, ko)

    k = abs(k)
    return k[0]


def k2k0(k_input, h, inv=False):
    """ Esta funcion relaciona el numero de onda asociado al periodo medio con el numero de onda
    asociado al periodo de pico (Goda, 1985)

    Args:
        k_input: numero de onda de entrada. Tiene los siguientes significados en funcion de la variable inv:

            * asociado al periodo medio del oleaje (si inv == False)
            * asociado al periodo de pico del oleaje (si inv == True)

        h: profundidad
        inv: un booleano que vale False si se desea convertir k_medio -> k_pico y
            True si se desea convertir k_pico -> k_medio

    Returns:
        k_output: numero de onda de salida. Tiene los siguientes significados en funcion de la variable inv:

            * asociado al periodo de pico del oleaje (si inv == False)
            * asociado al periodo medio del oleaje (si inv == True)

    """
    G = 9.81

    if not inv:
        t_m = 2*np.pi*np.sqrt(1/(G*k_input*np.tanh(k_input*h)))
        t_p = t_m*1.1
        k_output = wavenum(t_p, h)
    else:
        t_p = 2*np.pi*np.sqrt(1/(G*k_input*np.tanh(k_input*h)))
        t_m = t_p / 1.1
        k_output = wavenum(t_m, h)

    return k_output


def hrms2hs(h_input, inv=False):
    """ Esta funcion relaciona es estadistico altura de ola hrms con el estadistico altura de ola significante
    supuesta distribucion de rayleigh

    Args:
        h_input: estadistico de altura de ola de salida. Tiene los siguientes significados en funcion de la
            variable inv:

            * Altura de ola RMS (si inv == False)
            * Altura de ola significante (si inv == True)

        inv: un booleano que vale 0 si se desea convertir hrms -> hs y 1 si se desea convertir hs -> hrms

    Returns:
        h_output: estadistico de altura de ola de salida. Tiene los siguientes significados en funcion de la
            variable inv:

            * Altura de ola significante (si inv == False)
            * Altura de ola RMS (si inv == True)

    """
    if not inv:
        par_ray = h_input / np.sqrt(2)
        u = rayleigh.isf(1 / 3, loc=0, scale=par_ray)
        dx = 0.001
        x = np.arange(u, 20, dx)
        dist = rayleigh(loc=0, scale=par_ray)
        h_output = 3 * np.sum(x * dist.pdf(x)) * dx
    else:
        par_ray = np.sqrt(2)*h_input/(3*np.sqrt(np.pi)*(erfc(np.sqrt(np.log(3)))) + 2*np.sqrt(np.log(3)))
        h_output = par_ray*np.sqrt(2)

    return h_output


def sigmoide(v, x_0, x_1, gamma, a_r):
    """ Esta funcion evalua la funcion sigmoide

        Args:
           v: valor de la abscisa
           x_0: parametro 1 de la sigmoide
           x_1: parametro 2 de la sigmoide
           gamma: parametro 2 de la sigmoide
           a_r: parametro 2 de la sigmoide

        Returns:
            Valor de la funcion sigmoide
       """
    return (x_1 - x_0) * (1 + (v / a_r) ** gamma) ** (- 1) + x_0
