# -*- coding: utf-8 -*-
from __future__ import division

import pandas as pd
import numpy as np
from deslizamiento.deslizamiento import deslizamiento
from estabilidad_berma.estabilidad_berma import estabilidad_berma
from vuelco.vuelco import vuelco
from estabilidad_talud.estabilidad_talud import estabilidad_talud
from altura_total.altura_total import altura_total
from comportamiento_hidraulico.comportamiento_hidraulico import comportamiento_hidraulico
from util.monomios import calcula_monomios
from util import transformaciones


def verificacion_simultanea(estado, lista_modos_fallo, profundidad, tipo, par_dique, nivel_averia):
    """Esta funcion verifica los distintos modos de fallo del dique en un estado determinado

    Args:
        estado: vector con la siguiente informacion

            0. Anio
            1. Mes
            2. Dia
            3. Hora
            4. Altura de ola significante
            5. Periodo de pico
            6. Direccion del oleaje
            7. Veleocidad media del viento a 10 m
            8. Direccion media del viento
            9. Marea astronomica + nivel medio
            10. Marea meteorologica

        profundidad: nivel medio del mar
        tipo: tipologia de dique con los siguientes codigos

            * Tipo = A -> Porous vertical breakwater (PVB)
            * Tipo = B -> Composite breakwater (CB)
            * Tipo = C1 -> Low mound breakwater (LMB)
            * Tipo = C2 -> High mound breakwater (HMB)
            * Tipo = C3 -> High mound composite breakwater (HCMB)
            * Tipo = D -> Rubble mound breakwater with crown wall (RMB-CW)
            * Tipo = E -> Rubble mound breakwater
            * Tipo = F -> S breakwater

        par_dique: Un diccionario que indica las caracteristicas geometricas
            y fisicas del dique

        nivel_averia: Nivel de danio para el que se analiza el modo de fallo. Toma los siguientes valores:

            * IA: Inicio de averia
            * DI: Danio intermedio
            * D: Destruccion

    Returns:

    """
    df_modos_fallo = pd.DataFrame()
    df_monomios = pd.DataFrame()
    df_ensayado = pd.DataFrame()
    df_coef = pd.DataFrame()

    h, hi_rms, k_pico = lectura_estado(estado, profundidad)
    tp = estado['tp']

    # L0 = (9.81 * tp ** 2) / (2 * np.pi)
    # print('Rotura - Miche:' + str(hi_rms/L0) + '--' + str(0.14*np.tanh(k_pico*h)))

    monomios, ensayado, coef, ht, k_r, k_t, phi, dis = parametros_interaccion(h, hi_rms, k_pico, par_dique, tipo)
    df_monomios = df_monomios.append(monomios)
    df_ensayado = df_ensayado.append(ensayado)
    df_coef = df_coef.append(coef)

    for modo_fallo in lista_modos_fallo:
        mf, monomios, ensayado, coef = verificacion_modo_fallo(modo_fallo, tipo, hi_rms, h, k_pico, par_dique, k_t, ht,
                                                               nivel_averia, tp)
        df_modos_fallo = df_modos_fallo.append(mf)
        df_monomios = df_monomios.append(monomios)
        df_ensayado = df_ensayado.append(ensayado)
        df_coef = df_coef.append(coef)

    info_mf = {'monomios': df_monomios, 'ensayado': df_ensayado, 'coef': df_coef,
               'ht': ht, 'k_r': k_r, 'k_t': k_t, 'phi': phi, 'dis': dis, 'prof':h}

    return df_modos_fallo, info_mf


def lectura_estado(estado, profundidad):
    hs = estado['hs']
    hi_rms = transformaciones.hrms2hs(hs, 1)
    h = estado['marea'] + profundidad
    k_pico = transformaciones.wavenum(estado['tp'], h)

    return h, hi_rms, k_pico


def verificacion_modo_fallo(modo_fallo, tipo, hi_rms, h, k_pico, par_dique, k_t, ht, nivel_averia, tp):
    mf = pd.DataFrame()
    info = pd.DataFrame()
    ensayado = pd.DataFrame()
    coef = pd.DataFrame()

    if modo_fallo == 'deslizamiento':
        mf, info, ensayado, coef = deslizamiento(tipo, hi_rms, h, k_pico, par_dique, k_t, ht['x4'])
    elif modo_fallo == 'vuelco':
        mf, info, ensayado, coef = vuelco(tipo, hi_rms, h, k_pico, par_dique, k_t, ht['x4'])
    elif modo_fallo == 'estabilidad_berma':
        mf, info, ensayado, coef = estabilidad_berma(tipo, hi_rms, h, k_pico, par_dique, ht['x1'], nivel_averia)
    elif modo_fallo == 'estabilidad_talud':
        if tipo == 'D' or tipo == 'E':
            mf = estabilidad_talud(hi_rms, tp, par_dique, ht['x1'])
        else:
            print('No procede hacer el cálculo de modo de fallo de pérdida de piezas del talud')

    return mf, info, ensayado, coef


def parametros_interaccion(h, hi_rms, k_pico, par_dique, tipo):
    ht = {}
    coef_ht = {}
    coefs = pd.DataFrame()

    info_entrada = calcula_monomios(tipo, hi_rms, h, k_pico, par_dique)
    k_r, k_t, dis, phi, info_sig, ensayado_sig, coef = comportamiento_hidraulico(tipo, hi_rms, h, k_pico, par_dique)
    coefs = coefs.append(coef)

    info = [info_entrada, info_sig]
    ensayado = [ensayado_sig]

    for pos in ['x1', 'x4']:
        ht[pos], coef_ht[pos] = altura_total(tipo, pos, hi_rms, h, k_pico, par_dique, k_r, phi)
        info.append(ht[pos]['mono_ht'].rename('salida_altura_total_' + pos))
        if not coef_ht[pos].empty:
            coefs = coefs.append(coef_ht[pos].rename('altura_total_' + pos))
        ensayado.append(ht[pos]['mono_ht'].rename('salida_altura_total_' + pos))

    return info, ensayado, coefs, ht, k_r, k_t, phi, dis


def calcular_beta_referencia(estado, lista_modos_fallo, profundidad, tipo, par_dique, nivel_averia):
    beta_referencia, _ = verificacion_simultanea(estado, lista_modos_fallo,
                                                 profundidad, tipo, par_dique, nivel_averia)

    return beta_referencia['beta']


def verifica_beta(df_modos_fallo, beta):
    return df_modos_fallo['beta'] <= beta
