# -*- coding: utf-8 -*-
from __future__ import division
import numpy as np
import pandas as pd
from scipy.special import gamma as fun_gamma
from verificacion.util.monomios import monomios, obtener_coeficientes_tabla


def vuelco(tipo, hi_rms, h, k_pico, par_dique, k_t, ht):
    """Esta funcion verifica el modo de fallo de vuelco del espaldon

    Args:
        tipo: tipologia de dique con los siguientes codigos

            * Tipo = 1 -> Porous vertical breakwater (PVB)
            * Tipo = 2 -> Composite breakwater (CB)
            * Tipo = 3 -> Low mound breakwater (LMB)
            * Tipo = 4 -> High mound breakwater (HMB)
            * Tipo = 5 -> High mound composite breakwater (HCMB)
            * Tipo = 6 -> Rubble mound breakwater with crown wall (RMB-CW)
            * Tipo = 7 -> Rubble mound breakwater
            * Tipo = 8 -> S breakwater
            * Tipo = 9 -> Cualquiera de los tipos (LMB, HMB, HCMB)

        mono: Diccionario de util. Las variables contenidas dependeran del tipo de dique.
            Los util no definidos timan el valor -1

            0.  m0: Fb/Hirms: Rebasabilidad del dique
            1.  m1: Aeq/L^2: Parametro de dispersion en 2D
            2.  m2: Dk: Diametro relativo del material granular
            3.  m3: Hirms/L: Peralte incidente
            4.  m4: h/L: Profundidad relativa
            5.  m5: B/L: Anchura relativa del dique
            6.  m6: hb/h: Altura relativa de la cimentacion
            7.  m7: Bb/h: Anchura relativa de la berma
            8.  m8: FMT/h: Altura relativa de la berma
            9.  m9: Aeq_b/L^2: Area equivalente relativa de la berma
            10. m10: D_bk: Diametro relativo del material de la berma
            11. m11: Tipo de pieza del manto
            12. m12: ns: Porosidad del manto
            13. m13: alphaT: Pendiente del talud a barlomar

        ht_rms: estadistico rms de la altura de ola total en el estado
        beta_ht: parametro de forma de la altura de ola total en el estado
        h: nivel del mar en el estado
        par_dique: Un diccionario que indica las caracteristicas geometricas
            y fisicas del dique

            * w1 -> peso del espaldon
            * b -> ancho del espaldon
            * h_b -> profundidad de la base del espaldon
            * f_mt -> altura de la berma

        kt: alpha_iciente de transmision del dique


    Returns:
        Una tupla con la forma (beta, s_mean, s_var)

        * ``beta``: indice de fiabilidad
        * ``s_mean``: valor medio del margen de seguridad
        * ``s_var``: varianza del margen de seguridad

    """
    G = 9.81  # m/s2
    DENS_W = 1025  # kg/m3

    subtipo = tipo
    if tipo.startswith('C'):
        tipo = 'C'

    ht_rms = ht['ht_rms']
    beta_ht = ht['beta_ht']

    mono_vuelco, ensayado_vuelco, m4_pico = monomios('vuelco', tipo, hi_rms, h, k_pico, par_dique)

    # Variables derivadas
    # dens_espaldon = DENS_W * (par_dique['dens_rel'] + 1)
    dens_espaldon = 2700
    if tipo == 'C':
        par_dique['w1'] = (par_dique['f_c'] + (h - par_dique['h_b'])) * par_dique['b'] * dens_espaldon * G - (
                                                                                                                 h -
                                                                                                                 par_dique[
                                                                                                                     'h_b']) * \
                                                                                                             par_dique[
                                                                                                                 'b'] * \
                                                                                                             par_dique[
                                                                                                                 'dens_rel'] * DENS_W * G
    if tipo == 'D':
        if h - par_dique['h_b'] <= 0:
            par_dique['w1'] = (
                              par_dique['b'] * par_dique['h_fc'] + (par_dique['f_c'] + h - par_dique['h_b'] - par_dique[
                                  'h_fc']) * par_dique['h_fc']) * dens_espaldon * G
        else:
            par_dique['w1'] = ((
                                   par_dique['b'] * par_dique['h_fc'] + (
                                       par_dique['f_c'] + h - par_dique['h_b'] - par_dique[
                                           'h_fc']) * par_dique['h_fc']) * dens_espaldon * G) - (
                                  (h - par_dique['h_b']) * par_dique['b'] * par_dique['dens_rel'] * DENS_W * G)

    # TODO: comprobar para cuales tipos debe entrar y el peso del espaldón que debería ser

    # Transformacion del oleaje
    coef = obtener_coeficientes_tabla('vuelco', subtipo, mono_vuelco)

    # Momentos horizontales
    if subtipo == 'C3':
        alpha_i = coef['alpha_I1'] * mono_vuelco['m7'] * mono_vuelco['m4'] + coef['alpha_I2']
    else:
        alpha_i = 1

    if tipo == 'C':
        mh_rms = DENS_W * G * ht_rms / 48 * (ht_rms ** 2 + 8 * (h - par_dique['f_mt']) ** 2 + 6 * ht_rms * (
            h - par_dique['f_mt']) + 6 * ht_rms * (par_dique['f_mt'] - par_dique['h_b']) + 12 * (
            h - par_dique['f_mt']) * (par_dique['f_mt'] - par_dique['h_b']) + 2 * (
            np.cosh(m4_pico * mono_vuelco['m8']) / np.cosh(m4_pico) * (2 * (h - par_dique['f_mt']) ** 2 + 4 * (
                par_dique['f_mt'] - par_dique['h_b']) ** 2 + 9 * (h - par_dique['f_mt']) * (
                par_dique['f_mt'] - par_dique['h_b']))) + 4 * coef['alpha_2'] * (
            par_dique['f_mt'] - par_dique['h_b']) ** 2 * (
            np.cosh(m4_pico * mono_vuelco['m6']) / np.cosh(m4_pico))) * coef['alpha_1'] * alpha_i
    else:
        mh_rms = DENS_W * G * ht_rms / 48 * (
            ht_rms ** 2 + 6 * (h - par_dique['h_b']) + 4 * (h - par_dique['h_b']) ** 2 * (
                2 + np.cosh(
                    m4_pico * mono_vuelco['m6']) / np.cosh(m4_pico)) * coef['alpha_2']) * coef['alpha_1'] * alpha_i

    delta_mh = coef['c_rMH'] * mh_rms
    beta_mh = beta_ht * (coef['a_beta_M1H'] * m4_pico ** 2 + coef['a_beta_M2H'] * m4_pico + coef['a_beta_M3H'])

    mh_mean = delta_mh * fun_gamma(1 + 1 / beta_mh)
    mh_var = delta_mh ** 2 * fun_gamma(1 + 2 / beta_mh) - mh_mean ** 2

    # Momentos verticales
    alpha_3 = coef['a_alpha3'] * mono_vuelco['m5']**coef['b_alpha3']

    mv_rms = DENS_W * G * ht_rms / 12 * (
        par_dique['b'] ** 2 * np.cosh(m4_pico * mono_vuelco['m6']) / np.cosh(m4_pico)) * (
                 coef['alpha_1'] * coef['alpha_2'] * alpha_i) * (2 + alpha_3 * k_t)

    delta_mv = coef['c_rMV'] * mv_rms
    beta_mv = beta_ht * (coef['a_beta_M1V'] * m4_pico ** 2 + coef['a_beta_M2V'] * m4_pico + coef['a_beta_M3V'])

    mv_mean = delta_mv * fun_gamma(1 + 1 / beta_mv)
    mv_var = delta_mv ** 2 * fun_gamma(1 + 2 / beta_mv) - mv_mean ** 2

    # Correlaciones
    rho_mv_mh = coef['a_rhor_MHMV'] * np.exp(m4_pico) + 1
    cov_mv_mh = rho_mv_mh * np.sqrt(mv_var * mh_var)

    # Margen de seguridad
    s_mean = par_dique['b'] * par_dique['w1'] / 2 - mh_mean - mv_mean  # (N*m)/m
    s_var = mh_var + mv_var + 2 * cov_mv_mh
    beta = s_mean / np.sqrt(s_var)
    s_des = np.sqrt(s_var)

    # Pasamos a kN
    s_mean = s_mean / 1000
    s_des = s_des / 1000
    s_var = s_var / 1000

    devuelve = pd.Series((beta, s_mean, s_var, s_des), index=('beta', 'mean', 'var', 'dest'), name='vuelco')
    return devuelve, mono_vuelco, ensayado_vuelco, coef
